/*
 * arch/arm/mach-epxa/hrtime.c
 *
 * High-Res Timer Implementation for the ARM EPXA1 Excalibur
 *
 * Author: 
 * Pavel Kiryukhin <savl@dev.rtsoft.ru>
 *
 * Copyright 2003 (c) MontaVista, Software, Inc. 
 * 
 * This file is licensed under  the terms of the GNU General Public 
 * License version 2. This program is licensed "as is" without any 
 * warranty of any kind, whether express or implied.
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/timex.h>
#include <linux/kernel.h>
#include <linux/interrupt.h>
#include <asm/errno.h>
#include <asm/hardware.h>
#include <asm/mach/irq.h>
#include <asm/irq.h>
#include <asm/arch/timer00.h>
#include <asm/arch/hrtime.h>

#define TIMER00_TYPE (volatile unsigned int*)

int
schedule_hr_timer_int(unsigned ref_jiffies, int ref_cycles)
{
	unsigned int temp_cycles;

	BUG_ON(ref_cycles < 0);

	/*
	 * Get offset from last jiffy
	 */
	temp_cycles = (ref_jiffies - jiffies) * arch_cycles_per_jiffy +
	    ref_cycles - get_arch_cycles(jiffies);

	if (unlikely(temp_cycles <= 0))
		return -ETIME;

//      BUG_ON(temp_cycles > 0xffff);

	*TIMER1_LIMIT(IO_ADDRESS(EXC_TIMER00_BASE)) = temp_cycles;

	*TIMER1_CR(IO_ADDRESS(EXC_TIMER00_BASE)) = TIMER1_CR_IE_MSK | TIMER1_CR_S_MSK | TIMER1_CR_MODE_ONE;	/* enable timer in one-shot mode */
	return 0;
}

int
get_arch_cycles(unsigned ref_jiffies)
{
	int ret;
	unsigned temp_jiffies;
	unsigned diff_jiffies;

	do {
		/* snapshot jiffies */
		temp_jiffies = jiffies;
		barrier();

		/* calculate cycles since the current jiffy */

		ret = *TIMER0_READ(IO_ADDRESS(EXC_TIMER00_BASE));

		/* compensate for ref_jiffies in the past */
		if (unlikely(diff_jiffies = jiffies - ref_jiffies))
			ret += diff_jiffies * arch_cycles_per_jiffy;

		barrier();
		/* repeat if we didn't have a consistent view of the world */
	} while (unlikely(temp_jiffies != jiffies));

	return ret;
}

static void
hr_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	*TIMER1_CR(IO_ADDRESS(EXC_TIMER00_BASE)) = TIMER1_CR_CI_MSK;	/*clear interrupt, disable interrupts, stop timer */
	do_hr_timer_int();
}

static struct irqaction hr_timer_irq = {
	.name = "high-res timer",
	.handler = hr_timer_interrupt,
	.flags = SA_INTERRUPT
};

static __init int
hr_timer_init(void)
{
	int ret;
	*TIMER1_CR(IO_ADDRESS(EXC_TIMER00_BASE)) = TIMER1_CR_CI_MSK;	/*clear interrupt, disable interrupts, stop timer */
	ret = setup_arm_irq(IRQ_TIMER1, &hr_timer_irq);

	return ret;
}

__initcall(hr_timer_init);
