/*
 * FILE NAME: arch/arm/mach-EPXA/kgdb-serial.c
 *
 * BRIEF MODULE DESCRIPTION:
 *  Provides low level kgdb serial support hooks for EPXA1 target.
 *
 *  Author: 
 *  Pavel Kiryukhin <savl@dev.rtsoft.ru>
 *  
 * 2003 (c) MontaVista Software, Inc.  This file is licensed under
 * the terms of the GNU General Public License version 2.  This program
 * is licensed "as is" without any warranty of any kind, whether
 * express or implied.
 *  
 */

#include <linux/config.h>
#include <linux/types.h>

#include <asm/io.h>
#include <asm/kgdb.h>

#include "asm/arch/platform.h"
#include "asm/arch/hardware.h"
#define UART00_TYPE (volatile unsigned int*)
#include "asm/arch/uart00.h"

#define DIVISOR_FROM_BAUD(baud,clk) ((clk) /(16*(baud)))

void
kgdb_serial_init(void)
{

	int baudrate;
	char bdr;

	switch (kgdb_baud) {
	case 115200:
	case 57600:
	case 38400:
	case 19200:
	case 9600:
		baudrate = kgdb_baud;
		break;
	default:
		baudrate = 57600;
	}

	*UART_MC(IO_ADDRESS(EXC_UART00_BASE)) = UART_MC_CLS_CHARLEN_8;
	*UART_DIV_LO(IO_ADDRESS(EXC_UART00_BASE)) =
	    DIVISOR_FROM_BAUD(baudrate, EXC_AHB2_CLK_FREQUENCY) & 0xFF;
	*UART_DIV_HI(IO_ADDRESS(EXC_UART00_BASE)) =
	    (DIVISOR_FROM_BAUD(baudrate, EXC_AHB2_CLK_FREQUENCY) & 0xFF00) >> 8;

}

void
kgdb_serial_putchar(unsigned char ch)
{
	while ((*UART_TSR(IO_ADDRESS(EXC_UART00_BASE)) &
		UART_TSR_TX_LEVEL_MSK) == 15) ;

	*UART_TD(IO_ADDRESS(EXC_UART00_BASE)) = ch;
}

unsigned char
kgdb_serial_getchar(void)
{
	while (!
	       (*UART_RSR(IO_ADDRESS(EXC_UART00_BASE)) &
		UART_RSR_RX_LEVEL_MSK)) ;

	return *UART_RD(IO_ADDRESS(EXC_UART00_BASE));
}
