/*
 * arch/arm/mach-ixp2000/ixp2000-gpio.c
 *
 * GPIO code for IXP2000 board
 *
 * Author: Jeff Daly <jeffrey.daly@intel.com>
 *	Using mach-ixp425/ixp425-gpio.c as a model
 *
 * Copyright 2002 Intel Corp.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 */
#include <linux/module.h>
#include <linux/sched.h>

#include <asm/system.h>
#include <asm/irq.h>
#include <asm/mach/irq.h>
#include <asm/arch/hardware.h>

static int GPIO_IRQ_rising_edge;
static int GPIO_IRQ_falling_edge;
static int GPIO_IRQ_level_low;
static int GPIO_IRQ_level_high;

void set_GPIO_IRQ_edge(int gpio_nr, int edge)
{
	long flags;

	local_irq_save(flags);
	ixp_reg_write(IXP2000_GPIO_PDCR, (1 << gpio_nr));

	if (edge & GPIO_FALLING_EDGE)
		GPIO_IRQ_falling_edge |= (1 << gpio_nr);
	else
		GPIO_IRQ_falling_edge &= ~(1 << gpio_nr);
	if (edge & GPIO_RISING_EDGE)
		GPIO_IRQ_rising_edge |= (1 << gpio_nr);
	else
		GPIO_IRQ_rising_edge &= ~(1 << gpio_nr);
	GPIO_IRQ_level_high &= ~(GPIO_IRQ_falling_edge | GPIO_IRQ_rising_edge);
	GPIO_IRQ_level_low &= ~(GPIO_IRQ_falling_edge | GPIO_IRQ_rising_edge);

	ixp_reg_write(IXP2000_GPIO_FEDR, GPIO_IRQ_falling_edge);
	ixp_reg_write(IXP2000_GPIO_REDR, GPIO_IRQ_rising_edge);
	ixp_reg_write(IXP2000_GPIO_LSHR, GPIO_IRQ_level_high);
	ixp_reg_write(IXP2000_GPIO_LSLR, GPIO_IRQ_level_low);

	irq_desc[gpio_nr+IRQ_IXP2000_GPIO0].valid = 1;

	local_irq_restore(flags);
}

void set_GPIO_IRQ_level(int gpio_nr, int level)
{
	long flags;

	local_irq_save(flags);
	ixp_reg_write(IXP2000_GPIO_PDCR, (1 << gpio_nr));

	if (level & GPIO_LEVEL_LOW) {
		GPIO_IRQ_level_low |= (1 << gpio_nr);
		GPIO_IRQ_level_high &= ~(1 << gpio_nr);
	}
	else {
		GPIO_IRQ_level_low &= ~(1 << gpio_nr);
		GPIO_IRQ_level_high |= (1 << gpio_nr);
	}
	GPIO_IRQ_rising_edge &= ~(GPIO_IRQ_level_low | GPIO_IRQ_level_high);
	GPIO_IRQ_falling_edge &= ~(GPIO_IRQ_level_low | GPIO_IRQ_level_high);

	ixp_reg_write(IXP2000_GPIO_FEDR, GPIO_IRQ_falling_edge);
	ixp_reg_write(IXP2000_GPIO_REDR, GPIO_IRQ_rising_edge);
	ixp_reg_write(IXP2000_GPIO_LSHR, GPIO_IRQ_level_high);
	ixp_reg_write(IXP2000_GPIO_LSLR, GPIO_IRQ_level_low);

	irq_desc[gpio_nr+IRQ_IXP2000_GPIO0].valid = 1;

	local_irq_restore(flags);
}


void gpio_line_config(int line, int style)
{
	int flags;

	local_irq_save(flags);

	if(style == GPIO_OUT) {
		/* if it's an output, it ain't an interrupt anymore */
		ixp_reg_write(IXP2000_GPIO_PDSR, (1 << line));
		GPIO_IRQ_falling_edge &= ~(1 << line);
		GPIO_IRQ_rising_edge &= ~(1 << line);
		GPIO_IRQ_level_low &= ~(1 << line);
		GPIO_IRQ_level_high &= ~(1 << line);
		ixp_reg_write(IXP2000_GPIO_FEDR, GPIO_IRQ_falling_edge);
		ixp_reg_write(IXP2000_GPIO_REDR, GPIO_IRQ_rising_edge);
		ixp_reg_write(IXP2000_GPIO_LSHR, GPIO_IRQ_level_high);
		ixp_reg_write(IXP2000_GPIO_LSLR, GPIO_IRQ_level_low);
		irq_desc[line+IRQ_IXP2000_GPIO0].valid = 0;
	} else if(style == GPIO_IN) {
		ixp_reg_write(IXP2000_GPIO_PDCR, (1 << line));
	}
		
	local_irq_restore(flags);
}	

EXPORT_SYMBOL(set_GPIO_IRQ_edge);
EXPORT_SYMBOL(set_GPIO_IRQ_level);
EXPORT_SYMBOL(gpio_line_config);


