/*
 *  linux/arch/arm/mach-mx1ads/irq.c
 *
 *  Copyright (C) 1999 ARM Limited
 *  Copyright (C) 2002 Shane Nay (shane@minirl.com)
 *  Copyright (C) 2003 MontaVista Software Inc. <source@mvista.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/bitops.h>

#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/io.h>

#include <asm/mach/irq.h>
#include <asm/errno.h>

/*
 *
 * We simply use the ENABLE DISABLE registers inside of the MX1
 * to turn on/off specific interrupts.  FIXME- We should
 * also add support for the accelerated interrupt controller
 * by putting offets to irq jump code in the appropriate
 * places.
 *
 */

#define INTENNUM_OFF              0x8
#define INTDISNUM_OFF             0xC

#define VA_AITC_BASE              IO_ADDRESS(AITC_BASE)
#define MX1ADS_AITC_INTDISNUM     (VA_AITC_BASE + INTDISNUM_OFF)
#define MX1ADS_AITC_INTENNUM      (VA_AITC_BASE + INTENNUM_OFF)

#ifndef U32
typedef unsigned long U32;
#endif

#define _reg_AITC_INTENNUM 	(*((volatile U32 *)(VA_AITC_BASE+0x08)))
#define _reg_AITC_INTDISNUM	(*((volatile U32 *)(VA_AITC_BASE+0x0C)))
#define _reg_AITC_INTTYPEH		(*((volatile U32 *)(VA_AITC_BASE+0x18)))
#define _reg_AITC_INTTYPEL		(*((volatile U32 *)(VA_AITC_BASE+0x1C)))
#define _reg_AITC_FIVECSR		(*((volatile U32 *)(VA_AITC_BASE+0x44)))
#define TYPE_BIT_MASK(n) 		(1 << n)

static void
mx1ads_mask_irq(unsigned int irq)
{
	__raw_writel(irq, MX1ADS_AITC_INTDISNUM);
}

static void
mx1ads_unmask_irq(unsigned int irq)
{
	__raw_writel(irq, MX1ADS_AITC_INTENNUM);
}

void
mx1ads_mask_fiq(unsigned int irq)
{
	_reg_AITC_INTDISNUM = (unsigned long) irq;
}

void
mx1ads_unmask_fiq(unsigned int irq)
{
	_reg_AITC_INTENNUM = (unsigned long) irq;
}

void
mx1ads_set_type_to_fiq(unsigned int irq)
{
	if (irq >= 32) {
		_reg_AITC_INTTYPEH |= (TYPE_BIT_MASK(irq - 32));
	} else {
		_reg_AITC_INTTYPEL |= (TYPE_BIT_MASK(irq));
	}
}

void
mx1ads_set_type_to_irq(unsigned int irq)
{
	if (irq >= 32) {
		_reg_AITC_INTTYPEH &= ~TYPE_BIT_MASK(irq - 32);
	} else {
		_reg_AITC_INTTYPEL &= ~TYPE_BIT_MASK(irq);
	}
}

unsigned int
mx1ads_get_fiq_num(void)
{
	return (_reg_AITC_FIVECSR);
}

void __init
mx1ads_init_irq(void)
{
	unsigned int i;

	for (i = 0; i < NR_IRQS; i++) {
		irq_desc[i].valid = 1;
		irq_desc[i].probe_ok = 1;
		irq_desc[i].mask_ack = mx1ads_mask_irq;
		irq_desc[i].mask = mx1ads_mask_irq;
		irq_desc[i].unmask = mx1ads_unmask_irq;
		switch (i) {
		case 23:
		case 24:
		case 35:
			irq_desc[i].noautoenable = 1;
			break;
		}
	}

	/* Disable all interrupts initially. */
	/* In MX1 this is done in the bootloader. */

}
