/*
 * FILE NAME: arch/arm/mach-mx1ads/kgdb-serial.c
 *
 * BRIEF MODULE DESCRIPTION:
 *  Provides low level kgdb serial support hooks for MX1.
 *
 * Copyright 2001, 2003 MontaVista Software Inc. <source@<source@mvista.com>
 *
 * Copyright 1999 ARM Limited
 * Copyright (C) 2000 Deep Blue Solutions Ltd.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE	LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/types.h>
#include <linux/serial.h>
#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/kgdb.h>

#define UART_GET_CHAR(p)	inl((p) + UART_RXDATA)
#define UART_PUT_CHAR(p, c)	outl((c), (p) + UART_TXDATA)
#define UART_PUT_UBIR(p,c)	outl((c), (p) + UART_BIR)
#define UART_PUT_UBMR(p,c)	outl((c), (p) + UART_BMR)
#define UART_PUT_UCR1(p,c)	outl((c), (p) + UART_CR1)
#define UART_PUT_UCR2(p,c)	outl((c), (p) + UART_CR2)
#define UART_PUT_UCR3(p,c)	outl((c), (p) + UART_CR3)
#define UART_PUT_UCR4(p,c)	outl((c), (p) + UART_CR4)
#define UART_PUT_UFCR(p,c)	outl((c), (p) + UART_UFCR)
#define UART_GET_USR2(p)	inl((p) + UART_SR2)
#define _reg_PCDR (*((volatile unsigned long *)(IO_ADDRESS(PLL_BASE)+PLL_PCDR)))

typedef unsigned char U8;	/* unsigned 8 bit data  */
typedef unsigned short U16;	/* unsigned 16 bit data */
typedef unsigned int U32;	/* unsigned 32 bit data */

#define TXFE_MASK   0x4000	// Tx buffer empty
#define RDR_MASK    0x0001	// receive data ready

static volatile unsigned char *port = NULL;

void
kgdb_serial_init(void)
{

	_reg_PCDR &= 0xFFFFFFF0;
	_reg_PCDR |= 0x5;	// 16 MHz

	switch (kgdb_ttyS) {
	case 1:
		port = UART2_BASE;
		break;
	case 0:
	default:
		port = UART1_BASE;
	}

	UART_PUT_UCR1(port, 5);
	UART_PUT_UCR2(port, 0x4027);
	UART_PUT_UCR4(port, 1);
	UART_PUT_UFCR(port, 0xA81);

	UART_PUT_UBIR(port, kgdb_baud / 100 - 1);
	UART_PUT_UBMR(port, 10000 - 1);

	return;
}

void
kgdb_serial_putchar(unsigned char ch)
{

	while (!(UART_GET_USR2(port) & TXFE_MASK)) ;	// wait until TXFE bit set
	UART_PUT_CHAR(port, ch);
}

unsigned char
kgdb_serial_getchar(void)
{

	while (!(UART_GET_USR2(port) & RDR_MASK)) ;	// wait until RDR bit set
	return (unsigned char) UART_GET_CHAR(port);

}
