/* mx1ads-pm.c:	Power management for the MX1ADS
 *
 * Copyright 2003 MontaVista Software Inc.
 * Author: MontaVista Software, Inc.
 *         	           source@mvista.com
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/init.h>		/* __initcall */
#include <linux/kernel.h>	/* printk */
#include <linux/sysctl.h>	/* ctl_table */
#include <linux/pm.h>
#include <linux/delay.h>

#include <asm/hardware.h>
#include <asm/system.h>
#include <asm/io.h>

#define PREFIX		"MX1ADS PM: "	/* for printk */

/* number for the MX1ADS sleep sysctl()s
 *
 * This is picked at random out of thin air, hoping that it won't
 * clash with someone.  That's really ugly, but appears to be
 * "standard" practice (!?).  Oh well, with any luck we can throw
 * these away and replace them with sysfs parameters, in the
 * not-too-distant future...
 */

#define CTL_PM_MX1ADS 0x81c4

static int
suspend_handler(void)
{
	unsigned long cscr, intenableh, intenablel, uart_cr3, uart_sr1;
	int retval;

	if ((retval = pm_send_all(PM_SUSPEND, (void *) 3))) {
		printk(KERN_ERR PREFIX "Device suspend failed. \n");
		return retval;
	}

	intenableh = inl(IO_ADDRESS(AITC_BASE) + AITC_INTENABLEH);	/* Save all enabled interrupts */
	intenablel = inl(IO_ADDRESS(AITC_BASE) + AITC_INTENABLEL);
	outl(0x0, IO_ADDRESS(AITC_BASE) + AITC_INTENABLEH);	/* Disable all interrupts */
	outl(0x10000000, IO_ADDRESS(AITC_BASE) + AITC_INTENABLEL);	/* Enable UART1_MINT_UARTC for wakeup */

	cscr = inl(IO_ADDRESS(PLL_BASE) + PLL_CSCR);
	cscr &= ~(CSCR_UPEN | CSCR_MPEN);
	outl(cscr, IO_ADDRESS(PLL_BASE) + PLL_CSCR);	/* Stop PLLs */
	__asm__("mcr p15,0,r1,c7,c0,4");

	outl(intenableh, IO_ADDRESS(AITC_BASE) + AITC_INTENABLEH);	/* Restore all enabled interrupts */
	outl(intenablel, IO_ADDRESS(AITC_BASE) + AITC_INTENABLEL);

	if ((retval = pm_send_all(PM_RESUME, (void *) 0))) {
		printk(KERN_ERR PREFIX "Device resume failed. \n");
		return retval;
	}

	return 0;		/* success */
}

/*
 * /proc interface for power managment
 * 
 */

static int
suspend_proc_handler(ctl_table * ctl, int write, struct file *filp,
		     void *buffer, size_t * lenp)
{
	return suspend_handler();
}

static struct ctl_table pm_table[] = {
	{
	 .ctl_name = CTL_PM_MX1ADS,
	 .procname = "sleep",
	 .mode = 0200,		/* write-only to trigger sleep */
	 .proc_handler = &suspend_proc_handler,
	 },
	{0}
};

/*
 * Initialize power interface
 *
 */

static int __init
mx1ads_pm_init(void)
{
	int err;

	register_sysctl_table(pm_table, 1);

	return 0;
}

__initcall(mx1ads_pm_init);
