/*
 *  linux/arch/arm/mach-mx1ads/time.c
 *
 *  Copyright (C) 2000-2001 Deep Blue Solutions
 *  Copyright (C) 2002 Shane Nay (shane@minirl.com)
 *  Copyright (C) 2003 MontaVista Software Inc. <source@mvista.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/init.h>

#include <asm/hardware.h>
#include <asm/io.h>

extern int (*set_rtc) (void);

/* FIXME-
 * When we have an external RTC part,
 * put the mapping in for that part.
 *
 * The internal RTC within the MX1 is not sufficient
 * for tracking time other than time of day, or
 * date over very short periods of time.
 *
 */

#ifdef CONFIG_MX1ADS_RTC
extern int mx1ads_update_rtc(struct timeval *);
extern int mx1ads_rtc_start(void);
#endif

#define HOURMIN			IO_ADDRESS( (MX1ADS_RTC_BASE + 0x00 ) )
#define SECONDS			IO_ADDRESS( (MX1ADS_RTC_BASE + 0x04 ) )
#define DAYR			IO_ADDRESS( (MX1ADS_RTC_BASE + 0x20 ) )

static int
mx1ads_set_rtc(void)
{
	int sec, hrs, min;
	unsigned long flags;
	extern rwlock_t xtime_lock;

	read_lock_irqsave(&xtime_lock, flags);
	sec = xtime.tv_sec;
	read_unlock_irqrestore(&xtime_lock, flags);
	min = sec / 60;
	sec = sec % 60;
	hrs = min / 60;
	min = min % 60;
	*((volatile __u32 *) HOURMIN) = hrs << 8 | min;
	*((volatile __u32 *) SECONDS) = sec;
	return 0;
}

static int
mx1ads_rtc_init(void)
{
	int hh, mm, ss, day, mon, year;

	set_rtc = mx1ads_set_rtc;
	return 0;
}

__initcall(mx1ads_rtc_init);
