/*
 * arch/arm/mach-omap1510/hrtime.c
 *
 * High-Res Timer Implementation for TI OMAP1510 boards
 *
 * Author: George G. Davis <gdavis@mvista.com>
 *
 * Copyright 2003 (c) MontaVista, Software, Inc. 
 * 
 * This file is licensed under  the terms of the GNU General Public 
 * License version 2. This program is licensed "as is" without any 
 * warranty of any kind, whether express or implied.
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/time.h>
#include <linux/hrtime.h>
#include <linux/timex.h>
#include <linux/kernel.h>
#include <linux/interrupt.h>
#include <asm/errno.h>
#include <asm/hardware.h>
#include <asm/preem_latency.h>
#include <asm/mach/irq.h>
#include <asm/irq.h>


int
schedule_hr_timer_int(unsigned ref_jiffies, int ref_cycles)
{
	int temp_cycles;
	volatile mputimer_regs_t * subhz_timer = mputimer_base(2);

	BUG_ON(ref_cycles < 0);

	/*
	 * Get offset from last jiffy
	 */
	temp_cycles = (ref_jiffies - jiffies) * arch_cycles_per_jiffy +
		ref_cycles - get_arch_cycles(jiffies);

	if(unlikely(temp_cycles <= 0))
		return -ETIME;

	subhz_timer->cntl = MPUTIM_CLOCK_ENABLE;
	subhz_timer->load_tim = temp_cycles;
	subhz_timer->cntl = MPUTIM_CLOCK_ENABLE | MPUTIM_ST;

	return 0;
}

int
get_arch_cycles(unsigned ref_jiffies)
{
	extern unsigned long systimer_mark;
	extern unsigned long do_getmachinecycles(void);
	int ret;
	unsigned temp_jiffies;
	unsigned diff_jiffies;

	do {
                /* snapshot jiffies */
		temp_jiffies = jiffies;
		barrier();

                /* calculate cycles since the current jiffy */
		ret = do_getmachinecycles() - systimer_mark;

                /* compensate for ref_jiffies in the past */
                if (unlikely(diff_jiffies = jiffies - ref_jiffies))
                        ret += diff_jiffies * arch_cycles_per_jiffy;

		barrier();	
                /* repeat if we didn't have a consistent view of the world */
	} while(unlikely(temp_jiffies != jiffies));

	return ret;
}

static void
hr_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	volatile mputimer_regs_t * subhz_timer = mputimer_base(2);
	subhz_timer->cntl = 0;
	do_hr_timer_int();
}

static struct irqaction hr_timer_irq = {
	.name		= "high-res timer",
	.handler	= hr_timer_interrupt,
	.flags		= SA_INTERRUPT
};

static int 
hr_timer_init(void)
{
	int ret;
	volatile mputimer_regs_t * subhz_timer = mputimer_base(2);

	subhz_timer->cntl = 0;
	ret = setup_arm_irq(INT_TIMER3, &hr_timer_irq);

	return ret;
}

__initcall(hr_timer_init);
