/*
 * arch/arm/mach-s3c2800/camelot.c
 *
 * mappings for Fujitsu Caemlot board
 *
 * Author: <source@mvista.com>
 *
 * 2003 (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2. This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 */

#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/init.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/page.h>
 
#include <asm/mach/map.h>

/*
 * map internal registers
 *
 * we don't really need 64k granularity as it could all be done with
 * two 1MB entries, but this is more clear...
 */ 
static struct map_desc standard_io_desc[] __initdata = {
 /* virtual		physical        length domain     r w c b */
 { VA_CKPO_BASE,	PA_CKPO_BASE,	SZ_64K, DOMAIN_IO, 0,1,0,0},
 { VA_MEM_CTR_BASE,	PA_MEM_CTR_BASE,SZ_64K, DOMAIN_IO, 0,1,0,0},
 { VA_INT_BASE,		PA_INT_BASE,	SZ_64K, DOMAIN_IO, 0,1,0,0},
 { VA_DMA_BASE,		PA_DMA_BASE,	SZ_256K, DOMAIN_IO, 0,1,0,0},
 { VA_PCI_BASE,		PA_PCI_BASE,	SZ_64K, DOMAIN_IO, 0,1,0,0},
 { VA_IO_PORT_BASE,	PA_IO_PORT_BASE,SZ_64K, DOMAIN_IO, 0,1,0,0},
 { VA_RRCR_BASE,	PA_RRCR_BASE,	SZ_64K, DOMAIN_IO, 0,1,0,0},
 { VA_WD_BASE,		PA_WD_BASE,	SZ_64K, DOMAIN_IO, 0,1,0,0},
 { VA_TIMER_BASE,	PA_TIMER_BASE,	SZ_256K, DOMAIN_IO, 0,1,0,0},
 { VA_RTC_BASE,		PA_RTC_BASE,	SZ_64K, DOMAIN_IO, 0,1,0,0},
 { VA_UART_BASE,	PA_UART_BASE,	SZ_128K, DOMAIN_IO, 0,1,0,0},
 { VA_IIC_BASE,		PA_IIC_BASE,	SZ_128K, DOMAIN_IO, 0,1,0,0},
 { VA_DUAL_PORT_BASE,	PA_DUAL_PORT_BASE, SZ_8K, DOMAIN_IO, 0,1,0,0},
 { VA_CAMELOT_BASE,	PA_CAMELOT_BASE,SZ_16K, DOMAIN_IO, 0,1,0,0},
 LAST_DESC
};


extern void __init fujitsu_chamelot_map_io(void)
{
	iotable_init(standard_io_desc);
}
