/*
 * arch/arm/mach-s3c2800/hrtime.c
 *
 * High-Res Timer Implementation for S3C2800 boards
 *
 * Author: George G. Davis <gdavis@mvista.com>
 *
 * Copyright 2003 (c) MontaVista, Software, Inc. 
 * 
 * This file is licensed under  the terms of the GNU General Public 
 * License version 2. This program is licensed "as is" without any 
 * warranty of any kind, whether express or implied.
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/time.h>
#include <linux/hrtime.h>
#include <linux/timex.h>
#include <linux/kernel.h>
#include <linux/interrupt.h>
#include <asm/errno.h>
#include <asm/hardware.h>
#include <asm/preem_latency.h>
#include <asm/mach/irq.h>
#include <asm/irq.h>


int
schedule_hr_timer_int(unsigned ref_jiffies, int ref_cycles)
{
	int temp_cycles;

	BUG_ON(ref_cycles < 0);

	/*
	 * Get offset from last jiffy
	 */
	temp_cycles = (ref_jiffies - jiffies) * arch_cycles_per_jiffy +
		ref_cycles;

	if(unlikely(temp_cycles <= 0))
		return -ETIME;

	/* (50750 / 4) / 14 = 1.1us */
#define SYS_TIMER1_PRESCALER	14

	rTMCON1 = SYS_TIMER_MUX_1_4 | SYS_TIMER_INT_DMA_ENA;
	rTMDATA1 = (SYS_TIMER1_PRESCALER << 16) | temp_cycles;

	/* start timer 1 */
	rTMCON1 |= SYS_TIMER_COUNT_ENA;

	return 0;
}

int
get_arch_cycles(unsigned ref_jiffies)
{
	long flags, cycles;

	local_irq_save(flags);

	cycles = ~(rTMCNT1) + 
		(jiffies - ref_jiffies + 1) * arch_cycles_per_jiffy;

	local_irq_restore(flags);

	return cycles;
}

static void
hr_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	do_hr_timer_int();
}

static struct irqaction hr_timer_irq = {
	.name		= "high-res timer",
	.handler	= hr_timer_interrupt,
	.flags		= SA_INTERRUPT
};

static int 
hr_timer_init(void)
{
	int ret;

	ret = setup_arm_irq(IRQ_TIMER1, &hr_timer_irq);

	return ret;
}

__initcall(hr_timer_init);
