/*
 * arch/arm/mach-s3c2800/irq.c
 *
 * Samsung S3C2800 cpu interrupt support
 *
 * Author: <source@mvista.com>
 *
 * 2003 (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2. This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 */


#include <linux/init.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/ptrace.h>

#include <asm/hardware.h>
#include <asm/irq.h> /* contains asm/arch/irqs.h */
#include <asm/mach/irq.h>
#include <asm/arch/irq.h>


static void s3c2800_mask_ack_irq(unsigned int irq)
{
	if (0) printk("s3c2800_mask_ack_irq(irq=%d)\n", irq);
        rINTMSK &= ~((unsigned long)1 << irq);
        rSRCPND = ((unsigned long)1 << irq);
}

static void s3c2800_mask_irq(unsigned int irq)
{
	if (0) printk("s3c2800_mask_irq(irq=%d)\n", irq);
        rINTMSK &= ~((unsigned long)1 << irq);
}

static void s3c2800_unmask_irq(unsigned int irq)
{
	if (0) printk("s3c2800_unmask_irq(irq=%d)\n", irq);
        rINTMSK |= ((unsigned long) 1 << irq);
}

static void dummy_mask_ack_irq(unsigned int irq)
{
	printk("s3c2800_dummy_mask_ack_irq(irq=%d)\n", irq);
}

static void dummy_mask_irq(unsigned int irq)
{
	printk("s3c2800_dummy_mask_irq(irq=%d)\n", irq);
}

static void dummy_unmask_irq(unsigned int irq)
{
	printk("s3c2800_dummy_unmask_irq(irq=%d)\n", irq);
}

extern unsigned int fixup_irq (int i )
{
	return i;
}

extern void __init s3c2800_init_irq(void)
{
	unsigned long flags;
        int irq;

	if (0) printk("s3c2800_init_irq()\n");
//	request_resource(&iomem_resource, &irq_resource);

	save_flags_cli (flags);

	/* mask all interrupts */
        rINTMSK = 0x00000000;
	rSRCPND = 0xffffffff;

	/* all IRQs are IRQ, not FIQ */
        rINTMOD = 0x00000000; 


	restore_flags (flags);


        for (irq = 0; irq <= IRQ_RTC; irq++) {
                irq_desc[irq].valid     = 1;
                irq_desc[irq].probe_ok  = 1;
                irq_desc[irq].mask_ack  = s3c2800_mask_ack_irq;
                irq_desc[irq].mask      = s3c2800_mask_irq;
                irq_desc[irq].unmask    = s3c2800_unmask_irq;
        }

	{	/*  irq == IRQ_DUMMY */
	        irq_desc[irq].valid     = 1;
                irq_desc[irq].probe_ok  = 1;
                irq_desc[irq].mask_ack  = dummy_mask_ack_irq;
                irq_desc[irq].mask      = dummy_mask_irq;
                irq_desc[irq].unmask    = dummy_unmask_irq;
	}
}
