/*
 * arch/arm/mach-s3c2800/kgdb-serial.c
 *
 * Provides low level kgdb serial support hooks for S3C2800.
 *
 * Author: Brad Parker <brad@heeltoe.com>
 *
 * Copyright 2003 (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2. This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 */

#include <linux/config.h>
#include <linux/types.h>
#include <linux/serial.h>
#include <linux/serialP.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/kgdb.h>
#include <asm/hardware/serial_s3c2800.h>

#define UART_GET_CHAR(p)	readb((p) + S3C2800_UARTRXH0_OFF)
#define UART_PUT_CHAR(p,c)	writeb((c), (p) + S3C2800_UARTTXH0_OFF)

#define UART_GET_ULCON(p)      	readl((p) + S3C2800_UARTLCON_OFF)
#define UART_GET_UCON(p)       	readl((p) + S3C2800_UARTCON_OFF)
#define UART_GET_UFCON(p)      	readl((p) + S3C2800_UARTFCON_OFF)
#define UART_GET_UMCON(p)      	readl((p) + S3C2800_UARTMCON_OFF)
#define UART_GET_UBRDIV(p)     	readl((p) + S3C2800_UARTBRDIV_OFF)

#define UART_GET_UTRSTAT(p)   	readl((p) + S3C2800_UARTTRSTAT_OFF)
#define UART_GET_UERSTAT(p)    	readl((p) + S3C2800_UARTERSTAT_OFF)
#define UART_GET_UFSTAT(p)     	readl((p) + S3C2800_UARTFSTAT_OFF)
#define UART_GET_UMSTAT(p)     	readl((p) + S3C2800_UARTMSTAT_OFF)

#define UART_PUT_ULCON(p,c)    	writel(c, (p) + S3C2800_UARTLCON_OFF)
#define UART_PUT_UCON(p,c)     	writel(c, (p) + S3C2800_UARTCON_OFF)
#define UART_PUT_UFCON(p,c)    	writel(c, (p) + S3C2800_UARTFCON_OFF)
#define UART_PUT_UMCON(p,c)    	writel(c, (p) + S3C2800_UARTMCON_OFF)
#define UART_PUT_UBRDIV(p,c)   	writel(c, (p) + S3C2800_UARTBRDIV_OFF)

#define UART_RX_DATA(s)		(((s) & S3C2800_UTRSTAT_RXDR) == S3C2800_UTRSTAT_RXDR)
#define UART_TX_READY(s)	(((s) & S3C2800_UTRSTAT_TXFE) == S3C2800_UTRSTAT_TXFE)

static volatile unsigned char *port = NULL;

void
kgdb_serial_init(void)
{
	switch (kgdb_ttyS) {
#if 0
	case 1:
		port = VA_UART_BASE + S3C2800_UART1_OFF;
		break;
#endif
	case 0:
	default:
		port = VA_UART_BASE;
	}

	UART_PUT_UCON(port, S3C2800_UCON_DEFAULT);
	UART_PUT_UFCON(port, S3C2800_UFCON_DEFAULT);
	UART_PUT_UMCON(port, 0);
	UART_PUT_UBRDIV(port, (int)((PCLK/(kgdb_baud*16)))-1);
}

void
kgdb_serial_putchar(unsigned char ch)
{
        unsigned int status;

	do {
		status = UART_GET_UTRSTAT(port);
	} while (!UART_TX_READY(status));

	UART_PUT_CHAR(port, ch);
}

unsigned char
kgdb_serial_getchar(void)
{
        unsigned int status;
	unsigned char ch;

	do {
		status = UART_GET_UTRSTAT(port);
	} while (!UART_RX_DATA(status));

	ch = UART_GET_CHAR(port);

	return ch;
}
