/*
 * arch/arm/mach-s3c2800/leds.c
 *
 * Fujitsu Camelot board LED mappings
 *
 * Author: <source@mvista.com>
 *
 * 2003 (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2. This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 */

#include <linux/kernel.h>
#include <linux/init.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/leds.h>
#include <asm/system.h>
#include <asm/mach-types.h>

extern void led_display(int data);

/* 
 * called by arch_idle in include/asm/arch/system.h
 */

void set_led_state(int data)
{
	rPDATF = rPDATF | (0xf<<4);	/* All led off  */
	rPDATF = (rPDATF & (0xffffff0f)) | ((~data & 0xf)<<4);
}

void
set_led_bit(int n)
{
	u32 bit = (1 << n) << 4;
	rPDATF |= bit;
}

void
clear_led_bit(int n)
{
	u32 bit = (1 << n) << 4;
	rPDATF &= ~bit;
}

void
toggle_led_bit(int n)
{
	u32 bit = (1 << n) << 4;
	rPDATF ^= bit;
}

extern void s3c2800_leds_event(led_event_t ledevt)
{
	unsigned long flags;

	local_irq_save(flags);

	switch(ledevt) {
	case led_idle_start:
		set_led_bit(0);
		break;

	case led_idle_end:
		clear_led_bit(0);
		break;

	case led_timer:
		toggle_led_bit(1);
		break;

	case led_red_on:
		set_led_bit(3);
		break;
	case led_red_off:
		clear_led_bit(3);
		break;
	default:
		break;
	}

	local_irq_restore(flags);	 
}

static int __init leds_init(void)
{
	clear_led_bit(0);
	clear_led_bit(1);
	clear_led_bit(2);
	clear_led_bit(3);

	leds_event = s3c2800_leds_event;

	return 0;
}

__initcall(leds_init);
