/***************************************************************************
 *  This Program will build /var/udhcpd.conf according to /etc/system.conf  *
 ***************************************************************************/
#include <stdio.h>
#include <string.h>
#include <arpa/inet.h>
#include "h_var.h"
#include "mkdhcpdconf.h"
#include "lan.h"

static int ReadLanSettingsToStruct(struct config_keyword *pConf);
static int WriteSettingsToFile(FILE *fp,struct config_keyword *pConf);
static struct in_addr *ParserResolvConf(void);

static struct config_keyword keywords_default[] = {
    /* Keyword          default */
    { "start",          DHCPD_START_DEFAULT },
    { "end",            DHCPD_END_DEFAULT   },
    { "interface",      DHCPD_IFACE_DEFAULT },
    { "opt dns",        DHCPD_DNS_DEFAULT   },
    { "opt dns",        DHCPD_DNS_DEFAULT   },
    { "opt dns",        DHCPD_DNS_DEFAULT   },
    { "option subnet",  DHCPD_SUBNET_DEFAULT},
    { "option router",  DHCPD_ROUTER_DEFAULT},
    { "option domain",  DHCPD_DOMAIN_DEFAULT},
    { "option lease",   DHCPD_LEASE_DEFAULT }
};

int mkdhcpdconf_main(void){
    FILE *fp;
    struct config_keyword  config[DHCPD_CONF_NUM];
    fp = fopen(UDHCPD_CONF_FILE,"w");

    if(!fp){
        return MKDHCPD_WRITE_ERR;
    }

    ReadLanSettingsToStruct(config);

    WriteSettingsToFile(fp,config);

    fclose(fp);
    return MKDHCPD_OK;
}

/*
 *  This function will read LAN settings to pConf.
 *  Default is keywords_default.
 */
static int ReadLanSettingsToStruct(struct config_keyword *pConf){
    BYTE *p;
    struct in_addr *dns_p;
    /* Read LAN settings from system.conf to lan_conf. */
    ReadLanSettings(&lan_conf);

    /* Copy default value at first. */
    memcpy(pConf,keywords_default,(sizeof(struct config_keyword)*DHCPD_CONF_NUM));

    /* start */
    p = (BYTE *)&lan_conf.StartIp;
    sprintf(pConf[DHCPD_CONF_START].def,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* end */
    p = (BYTE *)&lan_conf.FinishIp;
    sprintf(pConf[DHCPD_CONF_END].def,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* interface */
    /* Now I don't know how to config it . Wait for wireless part. */

    /* I need read /etc/resolv.conf for dns options */
    dns_p = ParserResolvConf();
#if 0
    /* dns1 */
    sprintf(pConf[DHCPD_CONF_DNS1].def, "%s" , inet_ntoa(dns_p[0]));

    /* dns2 */
    sprintf(pConf[DHCPD_CONF_DNS2].def, "%s" , inet_ntoa(dns_p[1]));

    /* dns3 */
    sprintf(pConf[DHCPD_CONF_DNS3].def, "%s" , inet_ntoa(dns_p[2]));
#endif

    /* subnet */
    sprintf(pConf[DHCPD_CONF_SUBNET].def,"%u.%u.%u.%u"
        ,255,255,255,0);

    /* router */
    p = (BYTE *)&lan_conf.LanIPAddr;
    sprintf(pConf[DHCPD_CONF_ROUTER].def,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* dns ip */
    sprintf(pConf[DHCPD_CONF_DNS1].def,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));
    /* Domain */
    /* Don't know now ...... */

    /* lease */
    /* Perhaps no need to change it. */
    return 0;
}

/*
 * Write settings to file , we already had the write permission to fp.
 */

static int WriteSettingsToFile(FILE *fp,struct config_keyword *pConf){
    int i;
	PC_DB_CONF *p_pcdb_conf=&pc_db_conf;
    PC_DB_CONF_s *p_entry;
    unsigned char mac_addr[18];

	ReadPCDBConf(p_pcdb_conf);

    if(!fp || !pConf){
        return -1;
    }
    rewind(fp);


    for(i=0;i<DHCPD_CONF_NUM;i++){
        fprintf(fp,"%s      %s\n",pConf[i].keyword,pConf[i].def);
    }


/*  following is for fixed PC */
    for (i=0, p_entry=p_pcdb_conf->p_database; i<p_pcdb_conf->number && p_entry; i++)
    {
        fprintf(fp, "host %s {\n", p_entry->name);
	    printf_haddr(mac_addr, p_entry->mac_addr, ':');
        fprintf(fp, "hardware ethernet %s\n", mac_addr);
        fprintf(fp, "fixed-address %u.%u.%u.%u\n", p_entry->ip_addr[0], p_entry->ip_addr[1], p_entry->ip_addr[2], p_entry->ip_addr[3]);
        fprintf(fp, "}\n");
        p_entry = p_entry->next;
    }


/*
    fprintf(fp, "subnet 192.168.0.0 netmask 255.255.255.0 {\n");
    fprintf(fp, "range 192.168.0.11 192.168.0.60\n");

    fprintf(fp, "host Bobby_xp {\n");
    fprintf(fp, "hardware ethernet 00:30:F1:04:88:A3\n");
    fprintf(fp, "fixed-address 192.168.0.51\n");
    fprintf(fp, "filename \"\"\n");
    fprintf(fp, "}\n");
    fprintf(fp, "}\n");
*/
/*  Fixed PC end */
    return 0;
}

/* Read /etc/resolv.conf , get dns to dns[3] , if dns number less then 3 , it will be zero. */
static struct in_addr *ParserResolvConf(void){
    static struct in_addr dns[MAX_DNS_NUM];
    char line[MAX_LINE_LEN];
    FILE *fp;
    char *nameServerLoc,*commentLoc,*p,*p1;
    int dnsIndex;

    memset(dns,0,sizeof(dns));

    fp = fopen(RESOLV_CONF,"r");
    if(!fp){
        return dns;
    }
    dnsIndex = 0;
    while((fgets(line,MAX_LINE_LEN,fp))){
        nameServerLoc = strstr(line,NAMESERVER);
        if(!nameServerLoc){ /* Can not find "nameserver" */
            continue;
        }

        commentLoc = strchr(line,COMMENT);
        if( commentLoc && commentLoc < nameServerLoc ){ /* This line must be commented. */
            continue;
        }

        p = nameServerLoc + strlen(NAMESERVER);
        /*
         * "nameserver        202.102.14.141"
         *            ^
         *            p
         */

        for(;*p && isSeparate(*p);p++);
        if(!*p || !isDecimal(*p)){ /* Something must be wrong. */
            continue;
        }
        /*
         * "nameserver        202.102.14.141"
         *                    ^
         *                    p
         */

        for(p1=p;(isDecimal(*p1) || isDot(*p1));p1++);
        if(*p1 && !isSeparate(*p1) && !isLineEnd(*p1)){ /* Something must be wrong. */
            continue;
        }
        *p1 = '\0';
        /*
         * "nameserver        202.102.14.141"
         *                    ^             ^
         *                    p             p1
         */

        if(!inet_aton(p,&dns[dnsIndex])){ /* Something must be wrong.          */
            continue;                     /* Can not convert the IP address.   */
        }
        dnsIndex ++;
        if(dnsIndex >= MAX_DNS_NUM){
            break;
        }
    }
    return dns;
}


