/***************************************************************************
 *  This Program will build /var/optfile according to /etc/system.conf     *
 *  These value will be used by /sbin/lan and /sbin/wan                    *
 *                                                  Argon   2003-5-31      *
 ***************************************************************************/
 
#include <stdio.h>
#include <string.h>
#include <arpa/inet.h>
#include "h_var.h"
#include "mkoptfile.h"
#include "system.h"
#include "lan.h"
#include "wan.h"
#include "dev_opt.h"

static int ReadSettingsToStruct(struct option *pConf);
static int WriteSettingsToFile(FILE *fp,struct option *pConf);
static int WriteAppendItems(FILE *fp,struct option *pConf);
static MULTI_PPPOE_CONF mpoe_conf;

static struct option option_default[] = {
    /* Key                      Value */
/* For /sbin/lan */
     { "LAN_IP",                 LAN_IP_DEFAULT          }
    ,{ "LAN_MASK",               LAN_MASK_DEFAULT        }
    ,{ "LAN_IP2",                LAN_IP2_DEFAULT         }
    ,{ "LAN_MASK2",              LAN_MASK2_DEFAULT       }
    ,{ "LAN_BROADCAST",          LAN_BROADCAST_DEFAULT   }
    ,{ "START_IP",               LAN_START_IP_DEFAULT    }
    ,{ "FINISH_IP",              LAN_END_IP_DEFAULT      }
    ,{ "DHCPD_ENABLE",           LAN_DHCPD_ENABLE_DEFAULT}

/* For /sbin/wan */
    ,{ "WAN_CONNECTION",         WAN_CONNECTION_DEFAULT  }
    ,{ "WAN_DHCPC_TYPE",         WAN_DHCPC_TYPE_DEFAULT  }
    ,{ "WAN_IP",                 WAN_IP_DEFAULT          }
    ,{ "WAN_MASK",               WAN_MASK_DEFAULT        }
    ,{ "WAN_BROADCAST",          WAN_BROADCAST_DEFAULT   }
    ,{ "WAN_GATEWAY",            WAN_GATEWAY_DEFAULT     }
    ,{ "WAN_DNS",                WAN_DNS_DEFAULT         }
    ,{ "WAN_SPPPoE_DNS1",        WAN_DNS_DEFAULT         }
    ,{ "WAN_SPPPoE_DNS2",        WAN_DNS_DEFAULT         }
/* These two value should be same as /etc/ppp/pppoe.conf*/
    ,{ "WAN_SPPPoE_DEMAND",      WAN_SPPPoE_DEMAND_DEFAULT      }
    ,{ "WAN_SPPPoE_IDLE_TIME",   WAN_SPPPoE_IDLE_TIME_DEFAULT   }

/* For /sbin/modem-ppp-{start,stop} */
    ,{ "MDM_TELEPHONE",          MDM_TELEPHONE_DEFAULT   }
    ,{ "MDM_SPEED",              MDM_SPEED_DEFAULT       }
    ,{ "MDM_RETRY",              MDM_RETRY_DEFAULT       }
    ,{ "MDM_DEMAND",             MDM_DEMAND_DEFAULT      }
    ,{ "MDM_IDLETIME",           MDM_IDLETIME_DEFAULT    }
    ,{ "MDM_FIXDNS",             MDM_FIXDNS_DEFAULT      }
    ,{ "MDM_DNS1",               MDM_DNS1_DEFAULT        }
    ,{ "MDM_DNS2",               MDM_DNS2_DEFAULT        }

/* Backup DNS */

    ,{ "BACKUP_DNS1",            WAN_DNS_DEFAULT         }
    ,{ "BACKUP_DNS2",            WAN_DNS_DEFAULT         }
    
};

/* 1. No need change value 
 * 2. The length of value is bigger then MAX_VALUE_LEN
 */ 
static struct option option_append[] = {
/* For /sbin/wan */
    { "WAN_DHCP",                ""                      }
    ,{ "WAN_SPPPoE",             ""                      }
    ,{ "WAN_MPPPoE",             ""                      }
    ,{ "WAN_LOCAL",             ""                      }
    ,{ "WAN_MODEM",              ""                      }
    ,{ "WAN_DHCPC_TYPE_DYNAMIC", ""                      }
    ,{ "WAN_DHCPC_TYPE_FIXED",   ""                      }
    ,{ "WAN_SPOE_USERNAME",      ""                      }
    ,{ "WAN_SPOE_PASSWD",        ""                      }
    ,{ "WAN_MPOE_USERNAME0",      ""                      }
    ,{ "WAN_MPOE_PASSWD0",        ""                      }
    ,{ "WAN_MPOE_USERNAME1",      ""                      }
    ,{ "WAN_MPOE_PASSWD1",        ""                      }    
/* For /sbin/modem-ppp-{start,stop} */
    ,{ "MDM_USERNAME",           ""                      }
    ,{ "MDM_PASSWORD",           ""                      }
};



int mkoptfile_main(void){
    FILE *fp;
    static struct option  key_value[MKOPT_ITEM_MAX];
    fp = fopen(OPTFILE,"w");
        
    if(!fp){
        return MKOPTFILE_FAIL;
    }
    
    ReadSettingsToStruct(key_value);
    
    WriteSettingsToFile(fp,key_value);
    
    fclose(fp);
    return MKOPTFILE_OK;
}

/* 
 *  This function will read settings to pConf.
 *  Default is option_default.
 */
static int ReadSettingsToStruct(struct option *pConf){
    BYTE *p;
    DWORD broadcast;
    int i;

    /************************************************************
     *                                                          *
     *                      /sbin/lan                           *
     *                                                          *
     ************************************************************/
     
    /* Read LAN settings from system.conf to lan_conf. */
    ReadLanSettings(&lan_conf);
    
    /* Copy default value at first. */
    for(i=0;i<MKOPT_ITEM_MAX;i++){
        
        strcpy( pConf[i].key , option_default[i].key );
        strcpy( pConf[i].value , option_default[i].value );

    }

    /* LAN_IP */
    p = (BYTE *)&lan_conf.LanIPAddr;
    sprintf(pConf[MKOPT_ITEM_LAN_IP].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));
    
    /* LAN_MASK */
    p = (BYTE *)&lan_conf.NetMaskIPAddr;
    sprintf(pConf[MKOPT_ITEM_LAN_MASK].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));
    
    /* LAN_IP2 */
    p = (BYTE *)&lan_conf.LanIPAddr2;
    sprintf(pConf[MKOPT_ITEM_LAN_IP2].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));
    
    /* LAN_MASK2 */
    p = (BYTE *)&lan_conf.NetMaskIPAddr2;
    sprintf(pConf[MKOPT_ITEM_LAN_MASK2].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* LAN_BROADCAST */
    broadcast = lan_conf.LanIPAddr | (~lan_conf.NetMaskIPAddr);
    p = (BYTE *)&broadcast;
    sprintf(pConf[MKOPT_ITEM_LAN_BROADCAST].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));    

    /* START_IP */
    p = (BYTE *)&lan_conf.StartIp;
    sprintf(pConf[MKOPT_ITEM_START_IP].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));
    
    /* FINISH_IP */
    p = (BYTE *)&lan_conf.FinishIp;
    sprintf(pConf[MKOPT_ITEM_FINISH_IP].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));


    /* DHCPD_ENABLE */
    if(lan_conf.DhcpEnable){
        sprintf(pConf[MKOPT_ITEM_DHCPD_ENABLE].value,"%s","yes");
    }
    else{
        sprintf(pConf[MKOPT_ITEM_DHCPD_ENABLE].value,"%s","no");
    }
 

    /************************************************************
     *                                                          *
     *                      /sbin/wan                           *
     *                                                          *
     ************************************************************/

    /* Read system, dhcpc,single PPPoE settings from system.conf to system_conf, dhcp_conf,single_pppoe_conf. */
    ReadSystemSettings(&system_conf);
    ReadDHCPSettings(&dhcp_conf);
    ReadSinglePPPoESettings(&single_pppoe_conf);
    ReadMultiPPPoESettings(&mpoe_conf);
    
    /* WAN_CONNECTION */    
    sprintf(pConf[MKOPT_ITEM_WAN_CONNECTION].value,"%s"
        ,Connection_db[(BYTE)system_conf.Connection]);
    
    /* WAN_DHCPC_TYPE */
    sprintf(pConf[MKOPT_ITEM_WAN_DHCPC_TYPE].value,"%s"
        ,IPDNSTypeDB[(BYTE)dhcp_conf.dhcpc_type]);
    
    /* WAN_IP */
    p = (BYTE *)&dhcp_conf.ipAddress;
    sprintf(pConf[MKOPT_ITEM_WAN_IP].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* WAN_MASK */
    p = (BYTE *)&dhcp_conf.netMask;
    sprintf(pConf[MKOPT_ITEM_WAN_MASK].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* WAN_BROADCAST */
    broadcast = dhcp_conf.ipAddress & dhcp_conf.netMask;
    p = (BYTE *)&broadcast;
    sprintf(pConf[MKOPT_ITEM_WAN_BROADCAST].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* WAN_GATEWAY */
    p = (BYTE *)&dhcp_conf.Gateway;
    sprintf(pConf[MKOPT_ITEM_WAN_GATEWAY].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* WAN_DNS */
    p = (BYTE *)&dhcp_conf.dns[0];
    sprintf(pConf[MKOPT_ITEM_WAN_DNS].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));
        
    /* WAN_SPPPoE_DNS1 */
    p = (BYTE *)&single_pppoe_conf.dns[0];
    sprintf(pConf[MKOPT_ITEM_WAN_SPPPoE_DNS1].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* WAN_SPPPoE_DNS2 */
    p = (BYTE *)&single_pppoe_conf.dns[1];
    sprintf(pConf[MKOPT_ITEM_WAN_SPPPoE_DNS2].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* WAN_SPPPoE_DEMAND */
    if(DOD == single_pppoe_conf.dialMode){
        sprintf(pConf[MKOPT_ITEM_WAN_SPPPoE_DEMAND].value,"%d",single_pppoe_conf.DOD_idleTime*60);
    }
    else{
        sprintf(pConf[MKOPT_ITEM_WAN_SPPPoE_DEMAND].value,"%s",yes_no[YESNO_NO]);
    }

    /* WAN_SPPPoE_IDLE_TIME (This option valid only when $WAN_SPPPoE_DEMAND is "no") */
    if(MANUALLY == single_pppoe_conf.dialMode){
        sprintf(pConf[MKOPT_ITEM_WAN_SPPPoE_IDLE_TIME].value,"%d",single_pppoe_conf.Manual_idelTime*60);
    }
    else{
        sprintf(pConf[MKOPT_ITEM_WAN_SPPPoE_IDLE_TIME].value,"%d",0);
    }
    
    /************************************************************
     *                                                          *
     *             /sbin/modem-ppp-{start,stop}                 *
     *                                                          *
     ************************************************************/
#ifdef _WAN_MODEM_
    ReadModemSettings(&mdm_conf);


    /* MDM_TELEPHONE */
    sprintf(pConf[MKOPT_ITEM_MDM_TELEPHONE].value,"%s"
        ,mdm_conf.mdm_telephone);
    
    /* MDM_SPEED */
    sprintf(pConf[MKOPT_ITEM_MDM_SPEED].value,"%d"
        ,mdm_conf.mdm_speed);
    
    /* MDM_RETRY */
    sprintf(pConf[MKOPT_ITEM_MDM_RETRY].value,"%s"
        ,mdm_conf.mdm_pppd_retry_enable?yes_no[YESNO_YES]:yes_no[YESNO_NO]);

    /* MDM_DEMAND */
    if(DOD == mdm_conf.mdm_dialMode){
        sprintf(pConf[MKOPT_ITEM_DEMAND].value,"%u",mdm_conf.mdm_DOD_idleTime*60);
    }
    else{
        sprintf(pConf[MKOPT_ITEM_DEMAND].value,"%s",yes_no[YESNO_NO]);
    }

    /* MDM_IDLETIME (This option valid only when $DEMAND is "no") */
    if(MANUALLY == mdm_conf.mdm_dialMode){
        sprintf(pConf[MKOPT_ITEM_MDM_IDLETIME].value,"%d",mdm_conf.mdm_Manual_idelTime*60);
    }
    else{
        sprintf(pConf[MKOPT_ITEM_MDM_IDLETIME].value,"%d",0);
    }    

    /* MDM_FIXDNS */
    sprintf(pConf[MKOPT_ITEM_MDM_FIXDNS].value,"%s"
        ,mdm_conf.mdm_dns_enable?yes_no[YESNO_YES]:yes_no[YESNO_NO]);

    /* MDM_DNS1 */
    p = (BYTE *)&mdm_conf.mdm_dns1;
    sprintf(pConf[MKOPT_ITEM_MDM_DNS1].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* MDM_DNS2 */
    p = (BYTE *)&mdm_conf.mdm_dns2;
    sprintf(pConf[MKOPT_ITEM_MDM_DNS2].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));
#endif
    /************************************************************
     *                                                          *
     *                        Backup DNS                        *
     *                                                          *
     ************************************************************/

    ReadDevConf(&dev_opt_conf,-1);

    /* BACKUP_DNS1 */
    p = dev_opt_conf.dev_dns_1;
    sprintf(pConf[MKOPT_ITEM_BACKUP_DNS1].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));

    /* BACKUP_DNS2 */
    p = dev_opt_conf.dev_dns_2;
    sprintf(pConf[MKOPT_ITEM_BACKUP_DNS2].value,"%u.%u.%u.%u"
        ,*(p),*(p+1),*(p+2),*(p+3));


    return 0;
}

/*
 * Write settings to file , we already had the write permission to fp.
 */

static int WriteSettingsToFile(FILE *fp,struct option *pConf){
    int i;
    
    if(!fp || !pConf){
        return -1;
    }
    rewind(fp);
    
    
    for(i=0;i<MKOPT_ITEM_MAX;i++){
        fprintf(fp,"%s=%s\n",pConf[i].key,pConf[i].value);
    }
    
    WriteAppendItems(fp,option_append);
    return 0;
}

/* Write some append value */
static int WriteAppendItems(FILE *fp,struct option *pConf){
    /* /sbin/wan */

    if(!fp || !pConf){
        return -1;
    }
    
    /* WAN_DHCP */
    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_DHCP].key, Connection_db[CONNECTION_DHCP]);

    /* WAN_SPPPoE */
    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_SPPPoE].key, Connection_db[CONNECTION_SINGLE_PPPOE]);
    
    /* WAN_MPPPoE */
    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_MPPPoE].key, Connection_db[CONNECTION_MULTI_PPPOE]);

    /* WAN_LOCALOFF */  //ada
    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_LOCALOFF].key, Connection_db[CONNECTION_LOCAL_OFFICE]);    //ada

    /* WAN_DHCPC_TYPE_DYNAMIC */
    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_DHCPC_TYPE_DYNAMIC].key, IPDNSTypeDB[0]);

    /* WAN_DHCPC_TYPE_FIXED */
    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_DHCPC_TYPE_FIXED].key, IPDNSTypeDB[1]);

    /* WAN_SPOE_USERNAME */
    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_SPOE_USERNAME].key, single_pppoe_conf.userName);

    /* WAN_SPOE_PASSWD */
    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_SPOE_PASSWD].key, single_pppoe_conf.password);
    
    if(mpoe_conf.session[0]!=-1){
	    /* WAN_MPOE_USERNAME0 */
	    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_MPOE_USERNAME0].key, mpoe_conf.account[mpoe_conf.session[0]].poe_user);
	
	    /* WAN_MPOE_PASSWD0 */
	    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_MPOE_PASSWD0].key, mpoe_conf.account[mpoe_conf.session[0]].poe_pass);    
    }
    
    if(mpoe_conf.session[0]!=-1){    
	    /* WAN_MPOE_USERNAME1 */
	    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_MPOE_USERNAME1].key, mpoe_conf.account[mpoe_conf.session[1]].poe_user);
	
	    /* WAN_MPOE_PASSWD1 */
	    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_WAN_MPOE_PASSWD1].key, mpoe_conf.account[mpoe_conf.session[1]].poe_pass);
    }
    
#ifdef _WAN_MODEM_
    /* MDM_USERNAME */
    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_MDM_USERNAME].key, mdm_conf.mdm_user_name);

    /* MDM_PASSWORD */
    fprintf(fp,"%s=%s\n", pConf[MKOPT_APPEND_ITEM_MDM_PASSWORD].key, mdm_conf.mdm_password);
#endif
    return 0;
}


