/***************************************************************************
 *  This Program will build /etc/pppoe.conf according to /etc/system.conf  *
 ***************************************************************************/ 
#include <stdio.h>
#include <string.h>
#include "wan.h"
#include "system.h"
#include "h_var.h"
#include "mkpoeconf.h"

static int ReadSinglePPPoESettingsToStruct(PPPOE_CONF_CONF *pConf);
static int WriteSettingsToFile(FILE *fp,PPPOE_CONF_CONF *pConf);
static int WriteAppendItems(FILE *fp);
static int ReadMultiPPPoESettingsToStruct(PPPOE_CONF_CONF *pConf, int poe_session_index);

static char *null_str="";
static PPPOE_CONF_CONF poe_conf_conf;
static MULTI_PPPOE_CONF mpoe_conf;
static char pid_file[40];
int mkpoeconf_main(void){
    FILE *fp;
    char fname[20];
    int i,ret;
    ReadSystemSettings(&system_conf);
    if( system_conf.Connection != CONNECTION_SINGLE_PPPOE
     && system_conf.Connection != CONNECTION_MULTI_PPPOE){
        /* Not PPPoE , no need make PPPOE_CONF. */
        return MKPOE_NOT_POE;
    }

	for(i=0;i<MAX_PPPOE_SESSION;i++){
    	sprintf(fname,PPPOE_CONF_FILE,i);
        fp = fopen(fname,"w");
        
        if(!fp){
            return MKPOE_WRITE_ERR;
        }	
        if(CONNECTION_SINGLE_PPPOE == system_conf.Connection)	      
        	ret=ReadSinglePPPoESettingsToStruct(&poe_conf_conf); 
        else 
        	ret=ReadMultiPPPoESettingsToStruct(&poe_conf_conf,i);
        if(!ret)
        	WriteSettingsToFile(fp,&poe_conf_conf);
        fclose(fp);
        if(CONNECTION_SINGLE_PPPOE == system_conf.Connection)
        	return MKPOE_OK;    			
	}

    return MKPOE_OK;
}

/* 
 *  This function will read Single PPPoE settings to pConf.
 *  If char point to NULL , means no need set value in pppoe.conf.
 */
static int ReadSinglePPPoESettingsToStruct(PPPOE_CONF_CONF *pConf){
    
    ReadSinglePPPoESettings(&single_pppoe_conf);
    memset(pConf, 0 , sizeof(PPPOE_CONF_CONF));
    
    /* ETH */
    pConf->ETH = interface_str[ETH_DEFAULT];
    
    /* USER */
    pConf->USER = single_pppoe_conf.userName;
    
    /* SERVICENAME */
    pConf->SERVICENAME = SERVICENAME_DEFAULT;
    
    /* ACNAME */
    pConf->ACNAME = ACNAME_DEFAULT;

    /* DEMAND */
    if(DOD == single_pppoe_conf.dialMode){
        sprintf(pConf->DEMAND,"%d",single_pppoe_conf.DOD_idleTime*60);
    }
    else{
        sprintf(pConf->DEMAND,"%s",yes_no[YESNO_NO]);
    }

    /* IDLE_TIME (This option valid only when $DEMAND is "no") */
    if(MANUALLY == single_pppoe_conf.dialMode){
        sprintf(pConf->IDLE_TIME,"%d",single_pppoe_conf.Manual_idelTime*60);
    }
    else{
        sprintf(pConf->IDLE_TIME,"%d",0);
    }    
    
    /* DNSTYPE */
    if(TYPE_FIXED == single_pppoe_conf.dnsType){
        pConf->DNSTYPE = dnsType[DNSTYPE_SPECIFY];
    }
    else{
        pConf->DNSTYPE = dnsType[DNSTYPE_SERVER];
    }
    
    /* DNS1, DNS2 */
    if(TYPE_FIXED == single_pppoe_conf.dnsType){
        unsigned char *p1= (unsigned char *)&single_pppoe_conf.dns[0];
        unsigned char *p2= (unsigned char *)&single_pppoe_conf.dns[1];
        
        sprintf(pConf->DNS1,"%u.%u.%u.%u",*p1,*(p1+1),*(p1+2),*(p1+3));
        sprintf(pConf->DNS2,"%u.%u.%u.%u",*p2,*(p2+1),*(p2+2),*(p2+3));
    }
    else{
        *pConf->DNS1 = '\0';
        *pConf->DNS2 = '\0';
    }
    
    /* USERCTL */
    pConf->USERCTL = USERCTL_DEFAULT;
    
    /* PEERDNS */
    if(TYPE_FIXED == single_pppoe_conf.dnsType){
        pConf->PEERDNS = null_str;
    }
    else{
        pConf->PEERDNS = yes_no[YESNO_YES];
    }
    
    /* CONNECT_POLL */
    pConf->CONNECT_POLL = CONNECT_POLL_DEFAULT;
    
    /* CONNECT_TIMEOUT */
    pConf->CONNECT_TIMEOUT = CONNECT_TIMEOUT_DEFAULT;
    
    /* PING */
    pConf->PING = PING_DEFAULT;
    
    /* FORCEPING */
    pConf->FORCEPING = FORCEPING_DEFAULT;
    
    /* PIDFILE */
    sprintf(pid_file, PIDFILE_DEFAULT, 0);
    pConf->PIDFILE = pid_file;
    
    /* SYNCHRONOUS */
    pConf->SYNCHRONOUS = yes_no[SYNCHRONOUS_DEFAULT];
    
    /* CLAMPMSS */
    pConf->CLAMPMSS = CLAMPMSS_DEFAULT;
    
    /* LCP_INTERVAL */
    pConf->LCP_INTERVAL = LCP_INTERVAL_DEFAULT;
    
    /* LCP_FAILURE */
    pConf->LCP_FAILURE = LCP_FAILURE_DEFAULT;

    /* PPPOE_TIMEOUT */
    pConf->PPPOE_TIMEOUT = PPPOE_TIMEOUT_DEFAULT;

    /* FIREWALL */
    pConf->FIREWALL = firewall[FIREWALL_DEFAULT];
    
    /* PPPOE_EXTRA */
    pConf->PPPOE_EXTRA = PPPOE_EXTRA_DEFAULT;
    
    /* PPPD_EXTRA */
    pConf->PPPD_EXTRA = PPPD_EXTRA_DEFAULT;
    
    /* LINUX_PLUGIN */
    pConf->LINUX_PLUGIN = LINUX_PLUGIN_DEFAULT;
    
    /* LOCAL_IP */
    if(single_pppoe_conf.ipType != TYPE_DYNAMIC){
        unsigned char *p= (unsigned char *)&single_pppoe_conf.ipAddress;
        
        sprintf(pConf->LOCAL_IP,"%u.%u.%u.%u",*p,*(p+1),*(p+2),*(p+3));
    }
    else{
        *pConf->LOCAL_IP = '\0';
    }

    return 0;
}
/*
 * Write settings to file , we already had the write permission to fp.
 */

static int WriteSettingsToFile(FILE *fp,PPPOE_CONF_CONF *pConf){
    
    if(!fp || !pConf){
        return -1;
    }
    rewind(fp);
    
    /* ETH */
    fprintf(fp,"ETH=%s\n",pConf->ETH);
    
    /* USER */
    fprintf(fp,"USER=%s\n",pConf->USER);
    
    /* SERVICENAME */
    fprintf(fp,"SERVICENAME=%s\n",pConf->SERVICENAME);
    
    /* ACNAME */
    fprintf(fp,"ACNAME=%s\n",pConf->ACNAME);

    /* DEMAND */
    fprintf(fp,"DEMAND=%s\n",pConf->DEMAND);

    /* IDLE_TIME (This option valid only when $DEMAND is "no") */
    fprintf(fp,"IDLE_TIME=%s\n",pConf->IDLE_TIME);

    /* DNSTYPE */
    fprintf(fp,"DNSTYPE=%s\n",pConf->DNSTYPE);
    
    /* DNS1, DNS2 */
    fprintf(fp,"DNS1=%s\n",pConf->DNS1);
    fprintf(fp,"DNS2=%s\n",pConf->DNS2);
    
    /* USERCTL */
    fprintf(fp,"USERCTL=%s\n",pConf->USERCTL);
    
    /* PEERDNS */
    fprintf(fp,"PEERDNS=%s\n",pConf->PEERDNS);

    /* CONNECT_POLL */
    fprintf(fp,"CONNECT_POLL=%d\n",pConf->CONNECT_POLL);
    
    /* CONNECT_TIMEOUT */
    fprintf(fp,"CONNECT_TIMEOUT=%d\n",pConf->CONNECT_TIMEOUT);
    
    /* PING */
    fprintf(fp,"PING=\"%c\"\n",pConf->PING);
    
    /* FORCEPING */
    if(pConf->FORCEPING){
        fprintf(fp,"FORCEPING=\"%c\"\n",pConf->FORCEPING);
    }
    else{
        fprintf(fp,"FORCEPING=\n");
    }    
    
    /* PIDFILE */
    fprintf(fp,"PIDFILE=\"%s\"\n",pConf->PIDFILE);
    
    /* SYNCHRONOUS */
    fprintf(fp,"SYNCHRONOUS=%s\n",pConf->SYNCHRONOUS);
    
    /* CLAMPMSS */
    fprintf(fp,"CLAMPMSS=%d\n",pConf->CLAMPMSS);
    
    /* LCP_INTERVAL */
    fprintf(fp,"LCP_INTERVAL=%d\n",pConf->LCP_INTERVAL);
    
    /* LCP_FAILURE */
    fprintf(fp,"LCP_FAILURE=%d\n",pConf->LCP_FAILURE);

    /* PPPOE_TIMEOUT */
    fprintf(fp,"PPPOE_TIMEOUT=%d\n",pConf->PPPOE_TIMEOUT);

    /* FIREWALL */
    fprintf(fp,"FIREWALL=%s\n",pConf->FIREWALL);
    
    /* PPPOE_EXTRA */
    fprintf(fp,"PPPOE_EXTRA=%s\n",pConf->PPPOE_EXTRA);
    
    /* PPPD_EXTRA */
    fprintf(fp,"PPPD_EXTRA=%s\n",pConf->PPPD_EXTRA);
    
    /* LINUX_PLUGIN */
    fprintf(fp,"LINUX_PLUGIN=%s\n",pConf->LINUX_PLUGIN);
    
    /* LOCAL_IP */
    fprintf(fp,"LOCAL_IP=%s\n",pConf->LOCAL_IP);


    /* Append items. */
    WriteAppendItems(fp);
    return 0;
}

static int WriteAppendItems(FILE *fp){
    if(!fp){
        return -1;
    }
    
    /* DEFAULTROUTE */
    fprintf(fp,"DEFAULTROUTE=%s\n",yes_no[YESNO_YES]);
    
    /* RETRY_ON_FAILURE */
    fprintf(fp,"RETRY_ON_FAILURE=%s\n",yes_no[YESNO_NO]);
    
    return 0;
    
}

static int ReadMultiPPPoESettingsToStruct(PPPOE_CONF_CONF *pConf, int poe_session_index)
{
    PPPOE_CONF *pMpoeConf;

    ReadMultiPPPoESettings(&mpoe_conf);
    memset(pConf, 0 , sizeof(PPPOE_CONF_CONF));
    
    if(mpoe_conf.session[poe_session_index]==-1)
    	return 1;
    pMpoeConf=&mpoe_conf.account[mpoe_conf.session[poe_session_index]];
    /* ETH */
    pConf->ETH = interface_str[ETH_DEFAULT];
    
    /* USER */
    pConf->USER = pMpoeConf->poe_user;
    
    /* SERVICENAME */
    pConf->SERVICENAME = SERVICENAME_DEFAULT;
    
    /* ACNAME */
    pConf->ACNAME = ACNAME_DEFAULT;

    /* DEMAND */
    if(MPOE_CONN_DEMAND == pMpoeConf->conn_type){
        sprintf(pConf->DEMAND,"%d",pMpoeConf->poe_idle*60);
    }
    else{
        sprintf(pConf->DEMAND,"%s",yes_no[YESNO_NO]);
    }

    /* IDLE_TIME (This option valid only when $DEMAND is "no") */
    if(MPOE_CONN_NORMAL == pMpoeConf->conn_type){
        sprintf(pConf->IDLE_TIME,"%d",pMpoeConf->poe_idle*60);
    }
    else{
        sprintf(pConf->IDLE_TIME,"%d",0);
    }    
    
    /* DNSTYPE */
    if(MPOE_DNS_FIXED ==  pMpoeConf->poe_dns_type){
        pConf->DNSTYPE = dnsType[DNSTYPE_SPECIFY];
    }
    else{
        pConf->DNSTYPE = dnsType[DNSTYPE_SERVER];
    }
    
    /* DNS1, DNS2 */
    if(MPOE_DNS_FIXED == pMpoeConf->poe_dns_type){
        unsigned char *p1= (unsigned char *)&pMpoeConf->poe_dns_1;
        unsigned char *p2= (unsigned char *)&pMpoeConf->poe_dns_2;
        
        sprintf(pConf->DNS1,"%u.%u.%u.%u",*p1,*(p1+1),*(p1+2),*(p1+3));
        sprintf(pConf->DNS2,"%u.%u.%u.%u",*p2,*(p2+1),*(p2+2),*(p2+3));
    }
    else{
        *pConf->DNS1 = '\0';
        *pConf->DNS2 = '\0';
    }
    
    /* USERCTL */
    pConf->USERCTL = USERCTL_DEFAULT;
    
    /* PEERDNS */
    if(MPOE_DNS_FIXED == pMpoeConf->poe_dns_type){
        pConf->PEERDNS = null_str;
    }
    else{
        pConf->PEERDNS = yes_no[YESNO_YES];
    }
    
    /* CONNECT_POLL */
    pConf->CONNECT_POLL = CONNECT_POLL_DEFAULT;
    
    /* CONNECT_TIMEOUT */
    pConf->CONNECT_TIMEOUT = CONNECT_TIMEOUT_DEFAULT;
    
    /* PING */
    pConf->PING = PING_DEFAULT;
    
    /* FORCEPING */
    pConf->FORCEPING = FORCEPING_DEFAULT;
    
    /* PIDFILE */
    sprintf(pid_file, PIDFILE_DEFAULT, poe_session_index);
    pConf->PIDFILE = pid_file;
    
    /* SYNCHRONOUS */
    pConf->SYNCHRONOUS = yes_no[SYNCHRONOUS_DEFAULT];
    
    /* CLAMPMSS */
    pConf->CLAMPMSS = CLAMPMSS_DEFAULT;
    
    /* LCP_INTERVAL */
    pConf->LCP_INTERVAL = LCP_INTERVAL_DEFAULT;
    
    /* LCP_FAILURE */
    pConf->LCP_FAILURE = LCP_FAILURE_DEFAULT;

    /* PPPOE_TIMEOUT */
    pConf->PPPOE_TIMEOUT = PPPOE_TIMEOUT_DEFAULT;

    /* FIREWALL */
    pConf->FIREWALL = firewall[FIREWALL_DEFAULT];
    
    /* PPPOE_EXTRA */
    pConf->PPPOE_EXTRA = PPPOE_EXTRA_DEFAULT;
    
    /* PPPD_EXTRA */
    pConf->PPPD_EXTRA = PPPD_EXTRA_DEFAULT;
    
    /* LINUX_PLUGIN */
    pConf->LINUX_PLUGIN = LINUX_PLUGIN_DEFAULT;
    
    /* LOCAL_IP */
	*pConf->LOCAL_IP = '\0';
	
	return 0;	
}
