#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "flash.h"
#include "fw.h"
#include "sys_init.h"
#include "system.h"
#include "PRO_GetStr.h"
#include "led.h"
#include "email.h"
#include "wan.h"
#include "lan.h"
#include "dev_opt.h"
#include "schling.h"
#include "routing.h"
#include "wireless.h"
#include "mkcrontab.h"

#define	SYSCONF		"/var/system.conf"
extern int mkcrontab_main(void);

static int sys_conf_init(void)
{
	int conf_size;
	int ret=0;

	conf_size=flash_get_configf_size(MTD_CONFIG);
	if(conf_size <= 0){
        ret = restore();
	}
	else{
		ret=flash_read_config(MTD_CONFIG,SYSCONF,conf_size);
	}

	return ret;
}

static int sys_network_init(void)
{
	StartLAN(1);
	StartWan();
	return 0;
}

static int sys_time_init(void)
{
	Daytime();
	return 0;
}

static int sys_logd_init(void)
{
	system("/sbin/syslogd;/sbin/klogd");
	return 0;
}

static int sys_urlgw_init(void)
{
	system("/sbin/url_gw >/dev/null");
	return 0;
}

static int sys_ripd_init(void)
{
	FILE *fh;
	char ripd[2];

	fh=fopen(SYSCONF,"r");
	if(!fh)
		return -1;
	PRO_GetStr("ROUTING","rip_enable",ripd,2,fh);
	fclose(fh);
	if(ripd[0]=='1'){
		system("/sbin/routed >/dev/null 2>&1");
	}
	return 0;
}

static int sys_crond_init(void)
{
	mail_conf conf;
	int ret;
	FILE *fp;

	memset(&conf,0,sizeof(conf));
	ReadMailConf(&conf);
	fp = fopen(CRONTAB_FILE,"w");
	if(fp){
		fprintf(fp,"SHELL=/bin/sh\n");
		fprintf(fp,"PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/bin\n");
		fprintf(fp,"MAILTO=root\n");
		fprintf(fp,"HOME=/\n");
		fclose(fp);
	}
	mkcrontab_main();

	if(conf.mail_type==1){
		if(conf.out_enable){
			ret=SaveMailSchl(&conf,"/var/log/filter.log");
			if(ret)
				return ret;
		}

		if(conf.in_enable){
			if(conf.out_enable)
				Schl_Stop();
			ret=SaveMailSchl(&conf,"/var/log/access.log");
			if(ret)
				return ret;
		}
	}

    	return 0;
}

static int sys_route_init(void)
{
	int ret;

	ret=InitRoute();
	if(ret)
		return ret;
	return 0;
}

static int sys_firewall_init(void)
{
	int ret;
#ifdef _SH2C_
	system("/htdocs/sh_chain_init  ; /htdocs/sh_static_rules");
#else
	system("/etc/rc.d/chain_init.sh  ; /etc/rc.d/static-rules.sh ");
#endif
    WriteCommScript();
	ret=WriteScript();
	if(ret)
		return ret;
//	ret=StartFW();
    ret = FWInit();
	return ret;
}

static int sys_thttpd_passwd_init(void){

    return MakeHttpdPasswd();
}

static int sys_led_init(void){
    initLED(LED_POWER);
    initLED(LED_STATUS);

    turnOnLED(LED_POWER);
    turnOffLED(LED_STATUS);
    return 0;
}

static int sys_button_init(void){
    system("/sbin/button &");
    return 0;
}

static int sys_wireless_init(void)
{
	return SetWireless();
}

static int sys_dnsproxy_init(void)
{
	mkdproxyconf();
	system("/sbin/dproxy");
	return 0;
}

static int sys_reaim_init(void)
{
#ifdef _SH2C_
	system("/htdocs/sh_reaim >/dev/null 2>&1 &");
#else
	system("/etc/rc.d/reaim.sh >/dev/null 2>&1");
#endif
	system("/sbin/reaim >/dev/null 2>&1 ");
	return 0;
}

int sys_init_main(void)
{
	sys_conf_init();
	sys_led_init();
	sys_button_init();
	sys_thttpd_passwd_init();
	sys_firewall_init();
	sys_network_init();
	sys_dnsproxy_init();
	sys_time_init();
	sys_logd_init();
	sys_ripd_init();
	sys_crond_init();
	sys_urlgw_init(); 
	sys_route_init();
	sys_wireless_init();
	sys_reaim_init();
	return 0;
}
