#ifndef _W_WEP_C_
#define _W_WEP_C_
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "wireless.h"
#include "h_var.h"
#include "pppmd5.h"

static int errlog;
static char generate=0;
static unsigned char mykey64[20];
static BYTE mykey128[14];
static int which_submit;

enum{
    WEP_GENE=1,
    WEP_SAVE
};

enum{
    AU_OPENSYS,
    AU_SHAREKEY,
    AU_AUTOMATIC,
    AU_SIZE
};
static char *auth_db[AU_SIZE+1]={"Open System","Shared Key","Automatic",0};


static void wep_key_algorithm(CHAR *buf1,BYTE *buf2);
static void wep_key_md5(CHAR *buf1,BYTE *buf2);

/* 					Fuction generate_wepkey
		Generate Wireless Encrpytion Key, now support 64-bit/128-bit Key.
		
		Input :
			BYTE type : WEP_KEY_TYPE_64/WEP_KEY_TYPE_128
			CHAR *s : String to generate key. (ASCIIZ)
			BYTE *key : Caller must provide key buffer, 
						64-bit need 4*5=20 Bytes
						128-bit need 13 Bytes
		Output :
			BYTE *key : Four group 64-bit key/One group 128-bit key (HEX format)
*/
void generate_wepkey(const BYTE type,CHAR *s,BYTE * key)
{
	switch(type)
	{
		case WEP_KEY_TYPE_64 :
			wep_key_algorithm(s,key);
			break;
		case WEP_KEY_TYPE_128 :
			wep_key_md5(s,key);
			break;
		default :
			break;
	}
}

/*	generate four group of 64-bit keys */
static void wep_key_algorithm(CHAR *buf1,BYTE * buf2)
{
	DWORD  seed = 0;
	INT16 i,j;	
	BYTE * pseed;
	INT16 phraselen;

	phraselen=strlen(buf1);
    pseed = (BYTE *)&seed;
    for (i=0; i<phraselen; i++ ) {
#ifdef _LITTLE_ENDIAN_        
        pseed[i%4] ^= buf1[i];
#else        
		pseed[3-(i%4)] =pseed[3-(i%4)] ^ buf1[i];
#endif
	}
				
    for (j = 0; j < 4; j++) {
    	for (i=0; i<5 ;i++ )  {
			seed = 214013 * seed + 0x269ec3;
			buf2[j*5+i]=(BYTE)((seed >> 16) & 0x7fff);
        }
    }

    return;
}

/*	generate one group of 128-bit key */
static void wep_key_md5(CHAR * buf1,BYTE * buf2)
{
	INT16 i;
	CHAR *cp;
	static CHAR password_buf[65];
	DWORD phraselen;
	static MD5_CTX		MD;

	phraselen=strlen(buf1);

	MD5Init(&MD );
	cp=password_buf;

	for(i=0;i<64;i++)
		*cp++=buf1[i%phraselen];

	MD5Update(&MD,(BYTE*)password_buf, 64);
	MD5Final(&MD);

	memcpy(buf2,MD.digest, 13);

	return;
}


static int r2t_wep_auth(char buf[CGI_BUF_LEN],char **buf1){
    int i;
    
    errlog = 0;
    *buf = '\0';
    for(i=0;i<AU_SIZE;i++){	
        if(wireless_conf.pcmcia.Authentication==(i+1)){
			strcat(buf,option_sl);
		}		
		else{
			strcat(buf,option_s);
		}
		strcat(buf,auth_db[i]);
		strcat(buf,option_e);
	}
	return 0;
}



static int r2t_wep(char buf[CGI_BUF_LEN],char **buf1){
    if(!wireless_conf.pcmcia.Encryption){
        strcpy(buf,str_checked);
    }    
    else{
        *buf='\0';
    }
    return 0;
}   
    
static int r2t_wep_64bit(char buf[CGI_BUF_LEN],char **buf1){
    if(wireless_conf.pcmcia.Encryption  &&(wireless_conf.pcmcia.key_length==5)){
        strcpy(buf,str_checked);
    }    
    else{
        *buf='\0';
    }
    return 0;
}

#define r2t_keyx_y(x,y)                                             \
    static int r2t_key##x##_##y(char buf[CGI_BUF_LEN],char **buf1){ \
	    if(generate){                                               \
		    sprintf(buf,"%02x",(unsigned char)mykey64[(x-1)*5+(y-1)]);\
		}                                                           \
        else{                                                       \
            sprintf(buf,"%02x",(unsigned char)(wireless_conf.pcmcia.key_length==5)?wireless_conf.pcmcia.Keys[x-1][y-1]:0); \
		}                                                           \
	    return 0;                                                   \
	}

r2t_keyx_y(1,1);
r2t_keyx_y(1,2);
r2t_keyx_y(1,3);
r2t_keyx_y(1,4);
r2t_keyx_y(1,5);

r2t_keyx_y(2,1);
r2t_keyx_y(2,2);
r2t_keyx_y(2,3);
r2t_keyx_y(2,4);
r2t_keyx_y(2,5);

r2t_keyx_y(3,1);
r2t_keyx_y(3,2);
r2t_keyx_y(3,3);
r2t_keyx_y(3,4);
r2t_keyx_y(3,5);

r2t_keyx_y(4,1);
r2t_keyx_y(4,2);
r2t_keyx_y(4,3);
r2t_keyx_y(4,4);
r2t_keyx_y(4,5);


static int r2t_default_key64(char buf[CGI_BUF_LEN],char **buf1){
	int i;
	char str[3];

	*buf='\0';
	for(i=0;i<4;i++){
		if(wireless_conf.pcmcia.defaultKeyID==i){
        	strcat(buf,option_sl);
        }        
        else{
        	strcat(buf,option_s);
        }        
        sprintf(str,"%d ",i+1);
        strcat(buf,str);
        strcat(buf,option_e);
    }
    return 0;
}

static int r2t_wep_128bit(char buf[CGI_BUF_LEN],char **buf1){
    if(wireless_conf.pcmcia.Encryption  &&(wireless_conf.pcmcia.key_length==13)){
        strcpy(buf,str_checked);
    }    
    else{
        *buf='\0';
    }
    return 0;
}

#define r2t_k128_x_y(x,y)                                           \
static int r2t_k128_##x##_##y(char buf[CGI_BUF_LEN],char **buf1){   \
    *buf = '\0';                                                    \
    if(generate){                                                   \
        sprintf(buf,"%02x",mykey128[y-1]);                          \
	}                                                               \
	else{                                                           \
        sprintf(buf,"%02x",(unsigned char)(wireless_conf.pcmcia.key_length==13)?wireless_conf.pcmcia.Keys[x-1][y-1]:0);    \
	}	                                                            \
	buf[2]='\0';                                                    \
	return 0;                                                       \
}

r2t_k128_x_y(1,1);
r2t_k128_x_y(1,2);
r2t_k128_x_y(1,3);
r2t_k128_x_y(1,4);
r2t_k128_x_y(1,5);
r2t_k128_x_y(1,6);
r2t_k128_x_y(1,7);
r2t_k128_x_y(1,8);
r2t_k128_x_y(1,9);
r2t_k128_x_y(1,10);
r2t_k128_x_y(1,11);
r2t_k128_x_y(1,12);
r2t_k128_x_y(1,13);

static int r2t_passphrase(char buf[CGI_BUF_LEN],char **buf1){
    *buf = '\0';
    return 0;
}

static int r2t_message(char buf[CGI_BUF_LEN],char **buf1){
    set_err_msg(buf,buf1);
    return 0;
}
    

static char auth;
static int t2r_wep_auth(const char *value){
    auth=str_db_search(auth_db,value);
    return 0;
}

enum{
    WEP_DISABLE,
    WEP_ENABLED1,
    WEP_ENABLED2,
    WEP_NONE_,
    WEP_DB_SIZE
};
static char wep;
static char *wep_db[WEP_DB_SIZE]={"none","64bit","128bit",0};
static int t2r_wep(const char *value){
    wep=str_db_search(wep_db,value);
    return 0;
}

static unsigned char key[4][14];
#define t2r_key(x,y)                            \
static int t2r_key##x##_##y(const char *value){ \
    int lTemp;                                  \
    if(char2hex(value, 2, &lTemp)==-1){         \
        errlog = 1;                             \
    }                                           \
    key[x-1][y-1] = lTemp;                      \
    return 0;                                   \
}

t2r_key(1,1);
t2r_key(1,2);
t2r_key(1,3);
t2r_key(1,4);
t2r_key(1,5);

t2r_key(2,1);
t2r_key(2,2);
t2r_key(2,3);
t2r_key(2,4);
t2r_key(2,5);

t2r_key(3,1);
t2r_key(3,2);
t2r_key(3,3);
t2r_key(3,4);
t2r_key(3,5);

t2r_key(4,1);
t2r_key(4,2);
t2r_key(4,3);
t2r_key(4,4);
t2r_key(4,5);

static int default_value;
static int t2r_default_key64(const char *value){
    default_value=atoi(value)-1;
    return 0;
}

static unsigned char key128[14];

#define t2r_k128_x_y(x,y)                           \
static int t2r_k128_##x##_##y(const char *value){   \
    int lTemp;                                      \
    if(char2hex(value, 2, &lTemp)==-1){             \
        errlog = 1;                                 \
    }                                               \
    key128[y-1] = lTemp;                            \
    return 0;                                       \
}

t2r_k128_x_y(1,1);
t2r_k128_x_y(1,2);
t2r_k128_x_y(1,3);
t2r_k128_x_y(1,4);
t2r_k128_x_y(1,5);
t2r_k128_x_y(1,6);
t2r_k128_x_y(1,7);
t2r_k128_x_y(1,8);
t2r_k128_x_y(1,9);
t2r_k128_x_y(1,10);
t2r_k128_x_y(1,11);
t2r_k128_x_y(1,12);
t2r_k128_x_y(1,13);

static char gene_content[32+1];
static int t2r_passphrase(const char *value){
    strncpy(gene_content,value,32);
    gene_content[32]='\0';
    return 0;
}

static int t2r_keygen(const char *value){
    which_submit = WEP_GENE;
	generate = 1;
	return 0;
}

static int t2r_save(const char *value){
    which_submit = WEP_SAVE;
    generate = 0;
	return 0;
}

static int t2r_help(const char *value){
    return 0;
}

static int t2r_close(const char *value){
    return 0;
}


static int t2r_message(const char *value){
    return 0;
}

int w_wep_main(void){
	LIST *hd=NULL;
	WIRELESS_CONF *pConf=&wireless_conf;
	int ccode;
	int i;

    ReadWirelessSettings(pConf);
	if (strcmp(REQUEST_METHOD,POST) == 0){
		hd=cgi_input_parse();
		if(!hd){
			cgi_err("","");
		}		
		if(!do_cgi_t2r(hd,key_fun_tab)){
		    switch(which_submit){
		    case WEP_SAVE:
		        if(errlog){ /* Something error! */
                    char *msg = get_err_msg(MSG1_WEP);
                    fmt_err_msg(msg);
                }
                else{
                    /* No error , fill in conf */
                    pConf->pcmcia.Authentication=auth+1;
                    switch(wep){
                    case WEP_DISABLE:                    
			            pConf->pcmcia.Encryption =0;
		                break;
		            case WEP_ENABLED1:		           
			            pConf->pcmcia.Encryption =1;
				        pConf->pcmcia.key_length=5;
				        pConf->pcmcia.defaultKeyID=default_value;
				        for(i=0;i<4;i++){
				            memcpy(&pConf->pcmcia.Keys[i][0], &key[i][0], 13);
				        }
				        break;
				   case WEP_ENABLED2:
				        pConf->pcmcia.Encryption =1;
					    pConf->pcmcia.key_length=13;
					    pConf->pcmcia.defaultKeyID=0;
					    for(i=1;i<4;i++){
						    memset(&pConf->pcmcia.Keys[i][0], 0, 13);
					    }
        				memcpy(&pConf->pcmcia.Keys[0][0], &key128[0], 13);
        				break;
        			default:
        			    break;
        			}        			
                    ccode=SaveWirelessSettings(pConf);
			WriteConfToFlash();
			SetWirelessWep();

		}
                break;
            case WEP_GENE:           
			if(!gene_content[0])
				break;	
		    	pConf->pcmcia.Encryption =1;
			    switch (wep){
                case WEP_DISABLE:
				    pConf->pcmcia.Encryption =0;
				    break;
				case WEP_ENABLED1:
				    pConf->pcmcia.key_length=5;
					generate_wepkey(WEP_KEY_TYPE_64, gene_content, &mykey64[0]);
					break;
				case WEP_ENABLED2:
				    if (pConf->pcmcia.key_length!=13){
					    for(i=1;i<4;i++){
						    memset(&pConf->pcmcia.Keys[i][0], 0, 13);
						}
					}
					pConf->pcmcia.key_length=13;
					generate_wepkey(WEP_KEY_TYPE_128, gene_content, &mykey128[0]);
					break;
                default:
                    break;
                }
                break;
            default:
                break;
            }
		}
	}
   	return html_parser("w_wep.htm",key_fun_tab);
}

#endif /* _W_WEP_C_ */
