/* ============================================================================
 * Copyright (C) 1998-2001 Angus Mackay. All rights reserved;
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * ============================================================================
 */

/*
 * ez-ipupdate
 *
 * a very simple dynDNS client for the ez-ip dynamic dns service
 * (http://www.ez-ip.net).
 *
 * why this program when something like:
 *   curl -u user:pass http://www.ez-ip.net/members/update/?key=val&...
 * would do the trick? because there are nicer clients for other OSes and
 * I don't like to see UNIX get the short end of the stick.
 *
 * tested under Linux and Solaris.
 *
 */

/*
 *  1.Filter un-necessary code out
 *  2.Directly read configuration from system.conf
 *  3.Add check external ip from http://checkip.dyndns.org
 *  Sand Tu<sand_tu@sercomm.com>
 *  2002/09/23 16:35
 */

#include "common.h"
#include "PRO_file.h"
#include "PRO_GetStr.h"
#include "PRO_SetStr.h"
#include "PRO_GetInt.h"
#include "PRO_SetInt.h"

#define DEBUG 0

#if DEBUG
#include "errno.h"
#endif
#define DEFAULT_MSSAGE_FILE	"/var/ddns.message"
#define DEFAULT_MESSAGE "Failed to connect to dyndns server."

#define VERSION "3.0.11b7"
#define OS "i686-pc-linux-gnu"
#define DEF_SERVICE "dyndns"

#define DYNDNS_DEFAULT_SERVER "www.dyndns.org"
//#define DYNDNS_DEFAULT_SERVER "members.dyndns.org"
#define DYNDNS_DEFAULT_PORT "80"
#define DYNDNS_REQUEST "/nic/update"
#define DYNDNS_MAX_INTERVAL (25*24*3600)

#define DEFAULT_TIMEOUT 120
#define BUFFER_SIZE (1*1024-1) //reduce buffer size to about 1k

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <asm/sockios.h>
#include <time.h>


#define  snprintf(x, y, z...) sprintf(x, ## z)
#define ARRAY_LEN(x) (sizeof(x)/sizeof(x[0]))


// the max time we will wait if the server tells us to
#define MAX_WAITRESPONSE_WAIT (24*3600)

/**************************************************/

struct service_t
{
  char *title;
  char *names[3];
  int (*update_entry)(void);
  char *default_server;
  char *default_port;
  char *default_request;
};

enum {
  UPDATERES_OK = 0,
  UPDATERES_ERROR,
  UPDATERES_SHUTDOWN,
};

/**************************************************/
char *ip_address = NULL;
char *server = NULL;
char *port = NULL;
char user[256];
char auth[512];
char user_name[128];
char password[128];
char message[128]={0};
char *request = NULL;
int wildcard = 0;
char host[64];
struct timeval timeout;
char buf[BUFFER_SIZE+1];
static volatile int client_sockfd;

/* service objects for  sololy dyndns services */

int DYNDNS_update_entry(void);
void alarm_handle(int sig);

struct service_t service =
  { "dyndns",
    { "dyndns", 0, 0, },
    DYNDNS_update_entry,
    DYNDNS_DEFAULT_SERVER,
    DYNDNS_DEFAULT_PORT,
    DYNDNS_REQUEST
  };

//static struct service_t *service = NULL;

/**************************************************/
int do_connect(int *sock, char *host, char *port);
void base64Encode(char *intext, char *output);
int main(void);
/**************************************************/

/*
 * do_connect
 *
 * connect a socket and return the file descriptor
 *
 */
int do_connect(int *sock, char *host, char *port)
{
  struct sockaddr_in address;
  int len;
  int result;
  struct hostent *hostinfo;
  struct servent *servinfo;

  // set up the socket
  if((*sock=socket(AF_INET, SOCK_STREAM, 0)) == -1)
  {
    return(-1);
  }
  address.sin_family = AF_INET;

  // get the host address
  hostinfo = gethostbyname(host);
  if(!hostinfo)
  {
    close(*sock);
    return(-1);
  }
  address.sin_addr = *(struct in_addr *)*hostinfo -> h_addr_list;

  // get the host port
  servinfo = getservbyname(port, "tcp");
  if(servinfo)
  {
    address.sin_port = servinfo -> s_port;
  }
  else
  {
    address.sin_port = htons(atoi(port));
  }

  // connect the socket
  len = sizeof(address);
  if((result=connect(*sock, (struct sockaddr *)&address, len)) == -1)
  {
    close(*sock);
    return(-1);
  }

  return 0;
}

static char table64[]=
  "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
void base64Encode(char *intext, char *output)
{
  unsigned char ibuf[3];
  unsigned char obuf[4];
  int i;
  int inputparts;

  while(*intext) {
    for (i = inputparts = 0; i < 3; i++) {
      if(*intext) {
        inputparts++;
        ibuf[i] = *intext;
        intext++;
      }
      else
        ibuf[i] = 0;
    }

    obuf [0] = (ibuf [0] & 0xFC) >> 2;
    obuf [1] = ((ibuf [0] & 0x03) << 4) | ((ibuf [1] & 0xF0) >> 4);
    obuf [2] = ((ibuf [1] & 0x0F) << 2) | ((ibuf [2] & 0xC0) >> 6);
    obuf [3] = ibuf [2] & 0x3F;

    switch(inputparts) {
      case 1: /* only one byte read */
        sprintf(output, "%c%c==",
            table64[obuf[0]],
            table64[obuf[1]]);
        break;
      case 2: /* two bytes read */
        sprintf(output, "%c%c%c=",
            table64[obuf[0]],
            table64[obuf[1]],
            table64[obuf[2]]);
        break;
      default:
        sprintf(output, "%c%c%c%c",
            table64[obuf[0]],
            table64[obuf[1]],
            table64[obuf[2]],
            table64[obuf[3]] );
        break;
    }
    output += 4;
  }
  *output=0;
}


void output(void *buf)
{
  fd_set writefds;
  int max_fd;
  struct timeval tv;
  int ret;

#if DEBUG
  printf("I say: %s\n", (char *)buf);
#endif
  // set up our fdset and timeout
  FD_ZERO(&writefds);
  FD_SET(client_sockfd, &writefds);
  max_fd = client_sockfd;
  memcpy(&tv, &timeout, sizeof(struct timeval));

  ret = select(max_fd + 1, NULL, &writefds, NULL, &tv);

  if(ret == -1)
  {
#if DEBUG
    printf("select:\n");
#endif
  }
  else if(ret == 0)
  {
#if DEBUG
    printf("timeout\n");
#endif
  }
  else
  {
    /* if we woke up on client_sockfd do the data passing */
    if(FD_ISSET(client_sockfd, &writefds))
    {
      if(send(client_sockfd, buf, strlen(buf), 0) == -1)
      {
#if DEBUG
        printf("error send()ing request");
#endif
      }
    }
    else
    {
#if DEBUG
      printf("error: case not handled.");
#endif
    }
  }
}

int read_input(char *buf, int len)
{
  fd_set readfds;
  int max_fd;
  struct timeval tv;
  int ret;
  int bread = -1;

  // set up our fdset and timeout
  FD_ZERO(&readfds);
  FD_SET(client_sockfd, &readfds);
  max_fd = client_sockfd;
  memcpy(&tv, &timeout, sizeof(struct timeval));

  ret = select(max_fd + 1, &readfds, NULL, NULL, &tv);

#if DEBUG
  printf("ret: %d\n", ret);
#endif

  if(ret == -1)
  {
#if DEBUG
//    printf("select:\n");
	printf("select() ERROR: %s\n",strerror(errno));
#endif
  }
  else if(ret == 0)
  {
#if DEBUG
    printf("timeout\n");
#endif
  }
  else
  {
    /* if we woke up on client_sockfd do the data passing */
    if(FD_ISSET(client_sockfd, &readfds))
    {
		bread = recv(client_sockfd, buf, len-1, 0);
		buf[bread] = '\0';
#if DEBUG
      printf("got: %s\n", buf);
#endif
      if(bread == -1)
      {
#if DEBUG
        printf("error recv()ing reply");
#endif
      }
    }
    else
    {
#if DEBUG
      printf("error: case not handled.");
#endif
    }
  }

  return(bread);
}

char *DYNDNS_check_ip(void)
{
  char *bp = buf;
  int bytes;
  int btot;
  char *server1="checkip.dyndns.org";
//  char *server1="216.177.17.147";
  int ret=0;
	// buffer initialize
  buf[BUFFER_SIZE] = '\0';

	// here connect to checkip.dyndns.org:80
  ret=do_connect((int*)&client_sockfd, server1, port);
  if(ret){
	exit(1);
  }
  snprintf(buf, BUFFER_SIZE, "GET / HTTP/1.0\015\012");
  output(buf);

  snprintf(buf, BUFFER_SIZE, "Accept : */*\015\012");
  output(buf);

  snprintf(buf, BUFFER_SIZE, "User-Agent: %s %s [%s] (%s)\015\012",
      "ez-update", OS, "daemon" , "by Angus Mackay");
  output(buf);

  snprintf(buf, BUFFER_SIZE, "Host: %s\015\012", server1);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "\015\012");
  output(buf);

  bp = buf;
  bytes = 0;
  btot = 0;
  while((bytes=read_input(bp, BUFFER_SIZE-btot)) > 0)
  {
    bp += bytes;
    btot += bytes;
  }
  close(client_sockfd);
  buf[btot] = '\0';

#if DEBUG
  printf("server output: %s\n", buf);
#endif

	// grep the outer ip address from buf
  ip_address=strstr(buf,"Current IP Address:");
  strtok(ip_address,":\n");
  ip_address=strtok(NULL,":\n");

	// remove the space in front of the string
  while(*ip_address==' ')
	ip_address++;

  return(ip_address);
}

int DYNDNS_update_entry(void)
{
  char *bp = buf;
  int bytes;
  int btot;
  int ret;
  int retval = UPDATERES_OK;
  buf[BUFFER_SIZE] = '\0';

  if(do_connect((int*)&client_sockfd, server, port) != 0)
  {
    return(UPDATERES_ERROR);
  }

  snprintf(buf, BUFFER_SIZE, "GET %s?", request);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "%s=%s&", "hostname", host);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "%s=%s&", "myip", ip_address);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "%s=%s&", "wildcard", wildcard ? "ON" : "OFF");
  output(buf);
  snprintf(buf, BUFFER_SIZE, " HTTP/1.0\015\012");
  output(buf);
  snprintf(buf, BUFFER_SIZE, "Authorization: Basic %s\015\012", auth);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "User-Agent: %s- %s [%s] (%s)\015\012",
      "ez-update", OS, "daemon" , "by Angus Mackay");
  output(buf);
  snprintf(buf, BUFFER_SIZE, "Host: %s\015\012", server);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "\015\012");
  output(buf);
  bp = buf;
  bytes = 0;
  btot = 0;

  while((bytes=read_input(bp, BUFFER_SIZE-btot)) > 0)
  {
    bp += bytes;
    btot += bytes;
#if DEBUG
    printf("btot: %d\n", btot);
#endif
  }
  close(client_sockfd);
  buf[btot] = '\0';

#if DEBUG
  printf("server output: %s\n", buf);
#endif
  if(sscanf(buf, " HTTP/1.%*c %3d", &ret) != 1)
  {
    ret = -1;
  }

  switch(ret)
  {
    case -1:
      retval = UPDATERES_ERROR;
      break;

    case 200:
      if(strstr(buf, "\ngood ") != NULL)
      {
      	strcpy(message,"Ip address updated successfully!");
      }
      else if(strstr(buf, "\nnochg") != NULL)
      {
        strcpy(message,"Ip address need not change.");
      }
      else
      {
        if(strstr(buf, "\nnohost") != NULL)
        {
          strcpy(message,"No such host!");
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nnotfqdn") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\n!yours") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nabuse") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nbadauth") != NULL)
        {
        strcpy(message,"Authorization failed!");
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nbadsys") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nbadagent") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nnumhost") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\ndnserr") != NULL)
        {
          retval = UPDATERES_ERROR;
        }
        else if(strstr(buf, "\n911") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\n999") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\n!donator") != NULL)
        {
          retval = UPDATERES_OK;
        }

        // this one should be last as it is a stupid string to signify waits
        // with as it is so short
        else if(strstr(buf, "\nw") != NULL)
        {
          int howlong = 0;
          char *p = strstr(buf, "\nw");
          char reason[256];
          char mult;

          // get time and reason
          if(strlen(p) >= 2)
          {
            sscanf(p, "%d%c %255[^\r\n]", &howlong, &mult, reason);
            if(mult == 'h')
            {
              howlong *= 3600;
            }
            else if(mult == 'm')
            {
              howlong *= 60;
            }
            if(howlong > MAX_WAITRESPONSE_WAIT)
            {
              howlong = MAX_WAITRESPONSE_WAIT;
            };
          }
          else
          {
            sprintf(reason, "problem parsing reason for wait response");
          }

          sleep(howlong);
          retval = UPDATERES_ERROR;
        }
        else
        {
          retval = UPDATERES_ERROR;
        }
      }
      break;

    case 401:
      strcpy(message,"Authorization failed!");
      retval = UPDATERES_SHUTDOWN;
      break;

    default:
      retval = UPDATERES_ERROR;
      break;
  }

  return(retval);
}

int main(void)
{
  int retval = 1;
  FILE *fd;

  *user = '\0';
  *user_name = '\0';
  *password = '\0';

  timeout.tv_sec = DEFAULT_TIMEOUT;
  timeout.tv_usec = 0;

	(void) fclose( stdin );
	(void) fclose( stdout );
	(void) fclose( stderr );
	if ( daemon( 1, 1 ) < 0 )
	{
		exit( 1 );
	}
/****Add by Sure The connection time is too long...****/
  signal(SIGALRM,alarm_handle);	

  alarm(60);

  fd =  fopen(DDNS_SYSTEM_CONF,"rt");
  if(fd==NULL)
	return DDNS_ERROR;

  if(PRO_GetStr(DDNS_SECTION, DDNS_ACCOUNT , user_name, GENERIC_STRING_LEN, fd) != DDNS_OK)
	return DDNS_ERROR;
  if(PRO_GetStr(DDNS_SECTION, DDNS_PASSWORD , password, GENERIC_STRING_LEN, fd) != DDNS_OK)
	return DDNS_ERROR;
  if(PRO_GetStr(DDNS_SECTION, DDNS_HOSTNAME , host, DDNS_MAX_HOSTNAME_LEN, fd) != DDNS_OK)
	return DDNS_ERROR;

  fclose(fd);

  server = strdup(service.default_server);
  port = strdup(service.default_port);
/***2003-07-10 Sure  Save default message first****/
/*  strcpy(message,DEFAULT_MESSAGE);
  fd=fopen(DEFAULT_MSSAGE_FILE,"w");
  if(!fd)
 	 return DDNS_ERROR;
  fprintf(fd,"%s",message);
  fclose(fd);*/

	// obtain external ip address
  DYNDNS_check_ip();

	// process auth word and request command
  sprintf(user, "%s:%s", user_name, password);

  base64Encode(user, auth);  // to generate the authentication word
	// request == GET /nic/update
  request = strdup(service.default_request);
	if(service.update_entry() == UPDATERES_OK)
          retval = 0;

	 fd=fopen(DEFAULT_MSSAGE_FILE,"w");
 	if(!fd)
 		return DDNS_ERROR;
 	fprintf(fd,"%s",message);
	 fclose(fd);

//	if(ip_address) { free(ip_address); }
	if(port) { free(port); }
	if(request) { free(request); }
	if(server) { free(server); }

  return(retval);
}

void alarm_handle(int sig)
{
	FILE *fp;

	strcpy(message,DEFAULT_MESSAGE);
	fp=fopen(DEFAULT_MSSAGE_FILE,"w");
	if(!fp)
		exit(0);
	fprintf(fp,"%s",message);
	fclose(fp);

//	if(ip_address) { free(ip_address); }
	if(port) { free(port); }
	if(request) { free(request); }
	if(server) { free(server); }
	exit(0);
}
