
#define EXPORT_SYMTAB

#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
#define MODEVERSIONS
#endif

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/module.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/wait.h>
#include <asm/uaccess.h>
#include "led.h"

/* GPIO pin types,from /include/asm-arm/arch-ixp425/gpio.h */
#define IXP425_GPIO_OUT 		1
#define IXP425_GPIO_IN  		2

/* GPIO signal types */
#define IXP425_GPIO_LOW			0
#define IXP425_GPIO_HIGH		1

/* GPIO APIs from /arch/arm/mach-ixp425/gpio.c */
void gpio_line_isr_clear(u8 line);
void gpio_line_get(u8 line, int *value);
void gpio_line_set(u8 line, int value);
void gpio_line_config(u8 line, u32 style);

#ifdef DEBUG
#define PRINTK(format,argument...) printk(format,##argument)
#else
#define PRINTK(format,argument...)
#endif

MODULE_LICENSE("GPL");

static struct led_data_s led_data[] = {
	{GPIO_TEST,0,-1,0,2,0},
	{GPIO_WLAN,0,-1,0,2,1},
	{0,0,0,0,0,0}
};

static struct semaphore led_sem;
static spinlock_t       device_lock;

static ssize_t proc_read_led_fops(struct file *filp,
                                  char *buf,size_t count , loff_t *offp)
{
	char tmp[512]="";
	int len=0,i=0;
	if(*offp!=0)
		return 0;
	for(i=0;led_data[i].logic!=0;i++){
        int value;
        gpio_line_get(led_data[i].gpio_pin,&value);
		len+=sprintf(tmp+len,"GPIO%02d %d\n",i,value);
	}
	
	len=strlen(tmp)+1;
	copy_to_user(buf,tmp,len);
	PRINTK("<1>read...\n");
	*offp = len;
	return len;
}


static int led_atoi( char *name)
{
	int val = 0;
    for(;;name++)
	{
		switch(*name)
		{
			case '0'...'9':
				val = val*10+(*name - '0');
				break;
			default:
				return val;
		}
	}
}

static void led_on(unsigned long gpio,int logic)
{
    int value;

    value = (logic > 0)?IXP425_GPIO_HIGH:IXP425_GPIO_LOW;
    
    gpio_line_set(gpio,value);
}

static void led_off(unsigned long gpio,int logic)
{
    int value;

    value = (logic > 0)?IXP425_GPIO_LOW:IXP425_GPIO_HIGH;
    
    gpio_line_set(gpio,value);    
}

static void led_init(unsigned long gpio,int init,int logic)
{
    gpio_line_config(gpio,IXP425_GPIO_OUT);

	if(init)
		led_on(gpio,logic);
	else
		led_off(gpio,logic);
}

static void led_reverse(unsigned long gpio)
{
    int value;
    
    gpio_line_get(gpio,&value);
    gpio_line_set(gpio,!value);
}


static struct timer_list led_slow_timer;
static struct timer_list led_fast_timer;

static int led_fast_timer_running=0;
static int led_slow_timer_running=0;

static void led_blink_timer(unsigned long speed)
{
	struct led_data_s *led;
	int need_timer=0;
	
	for(led=led_data;led->logic!=0;led++){
		if(led->count > 0 && led->speed==speed ){
			led_reverse(led->gpio_pin);
			led->count--;
			
			if(led->count>0)
				need_timer=1;
			else{
				/* last stat is off*/
				if(led->last_stat==0)
					led_off(led->gpio_pin,led->logic);
				/* last stat is on */
				else if(led->last_stat==1)
					led_on(led->gpio_pin,led->logic);
			}
		}
	}
	
	if(need_timer){
		if(speed==LED_FAST){
			led_fast_timer.data=speed;
            led_fast_timer.expires=jiffies+FAST_INTERVAL;
            add_timer(&led_fast_timer);
		}else{
			led_slow_timer.data=speed;
            led_slow_timer.expires=jiffies+SLOW_INTERVAL;
            add_timer(&led_slow_timer);
		}
	}else{
		if(speed==LED_FAST)
			led_fast_timer_running=0;
		else
			led_slow_timer_running=0;
			
	}

}

void led_blink(unsigned long gpio,int count ,int speed,int last_stat)
{
	struct led_data_s *led;
	unsigned int irqflags = 0x00;
	
	spin_lock_irqsave(&device_lock, irqflags);
	for(led=led_data;led->logic!=0;led++){
		if(led->gpio_pin == gpio ){
			led->speed=speed;
			led->count=2*count;
			led->last_stat=last_stat;
		}
	}
		
	if(speed==LED_FAST && led_fast_timer_running==0){
		led_fast_timer_running=1;
		led_fast_timer.data=speed;
        led_fast_timer.expires=jiffies+FAST_INTERVAL;
        add_timer(&led_fast_timer);
	}else if(speed==LED_SLOW && led_slow_timer_running==0){
		led_slow_timer_running=1;
		led_slow_timer.data=speed;
        led_slow_timer.expires=jiffies+SLOW_INTERVAL;
        add_timer(&led_slow_timer);
	}
    
	spin_unlock_irqrestore(&device_lock, irqflags);	
}


static ssize_t proc_write_led_fops(struct file *filp,const char *buffer,
                                   size_t count , loff_t *offp)
{
	char line[10];
	unsigned int irqflags = 0x00;
	
	copy_from_user(line,buffer,count);
	line[count]=0x00;
	
	PRINTK("<0>echo in==%s\n",line);
	
	if(count<1) 
		return count;
		
	down_interruptible(&led_sem);	
	
	spin_lock_irqsave(&device_lock, irqflags);
	switch(line[0]){
		/* upgrade */
		case 'u':
			led_off(GPIO_TEST,-1);
			led_on(GPIO_POWER,-1);
			led_blink(GPIO_TEST,led_atoi(&line[1]),LED_SLOW,0);
			led_blink(GPIO_POWER,led_atoi(&line[1]),LED_SLOW,1);
			break;
            /* test */
		case 't':
			led_on(GPIO_TEST,-1);		
			mdelay(300);
			led_off(GPIO_TEST,-1);
			break;
            /* reboot */
		case 'r':
			led_on(GPIO_TEST,-1);
			led_on(GPIO_POWER,-1);
			mdelay(1000);
			led_on(GPIO_RESET,-1);
			led_off(GPIO_TEST,-1);
			break;
            /* write flash */
		case 'w':
			led_off(GPIO_TEST,-1);
			led_off(GPIO_POWER,-1);
			led_blink(GPIO_TEST,led_atoi(&line[1]),LED_FAST,0);
			led_blink(GPIO_POWER,led_atoi(&line[1]),LED_FAST,1);
			break;
            /* adsl */
		case 'a':
			if(line[1]=='1' && line[2]=='\0')
				led_on(GPIO_ADSL_ACT,-1);
			else if(line[1]=='0')
				led_off(GPIO_ADSL_ACT,-1);
			else
				led_blink(GPIO_ADSL_ACT,led_atoi(&line[1]),LED_FAST,1);
			break;
            /* sync */
		case 's':
			led_blink(GPIO_ADSL_DOWN,led_atoi(&line[1]),LED_SLOW,0);
			break;	
            /* restore to default */
		case 'd':
			led_blink(GPIO_TEST,led_atoi(&line[1]),LED_SLOW,0);            
			break;		
		/* upgrade */
		case 'l':
			led_off(GPIO_WLAN,-1);
			led_blink(GPIO_WLAN,led_atoi(&line[1]),LED_FAST,0);
			break;
            /* test */			
	}
	
	spin_unlock_irqrestore(&device_lock, irqflags);	

	up(&led_sem);	

	return count;
}

static struct file_operations led_fops = {
    read: proc_read_led_fops,
    write: proc_write_led_fops,
};

static struct proc_dir_entry *led_file;

static int __init led_init_module(void)
{
	int i=0;

	PRINTK("init...\n");
	led_file=create_proc_entry("led",0666,&proc_root);
	led_file->owner = THIS_MODULE;
	led_file->proc_fops = &led_fops;

	led_init(GPIO_RESET,0,-1);	
	for(i=0;led_data[i].logic!=0;i++){	
		led_init(led_data[i].gpio_pin,led_data[i].init,led_data[i].logic);	
	}
	
	init_timer(&led_slow_timer);
	led_slow_timer.function=led_blink_timer;
	init_timer(&led_fast_timer);
	led_fast_timer.function=led_blink_timer;
	
	sema_init(&led_sem,1);
	
	return 0;
}
static void __exit led_cleanup_module(void)
{
	remove_proc_entry("led",&proc_root);
	del_timer(&led_fast_timer);
	del_timer(&led_slow_timer);
	PRINTK("<0>cleanup...\n");
}

module_init(led_init_module);
module_exit(led_cleanup_module);
