
#define EXPORT_SYMTAB

#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
#define MODEVERSIONS
#endif

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/module.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/wait.h>
#include <linux/netdevice.h>
#include <asm/uaccess.h>

/* GPIO pin types,from /include/asm-arm/arch-ixp425/gpio.h */
#define IXP425_GPIO_OUT 		1
#define IXP425_GPIO_IN  		2

/* GPIO signal types */
#define IXP425_GPIO_LOW			0
#define IXP425_GPIO_HIGH		1

/* GPIO APIs from /arch/arm/mach-ixp425/gpio.c */
void gpio_line_isr_clear(u8 line);
void gpio_line_get(u8 line, int *value);
void gpio_line_set(u8 line, int value);
void gpio_line_config(u8 line, u32 style);

#ifdef DEBUG
#define PRINTK(format,argument...) printk(format,##argument)
#else
#define PRINTK(format,argument...)
#endif

MODULE_LICENSE("GPL");

static struct semaphore push_button_sem;
static struct timer_list push_button_timer;
static char MyBuf[32];
static int test_led_on=0;


#define GPIO_TEST       0
#define GPIO_PUSHBUTTON 4

#define LED_BLINK_INTERVAL 200

/* polling every 0.5 sec */
#define SCAN_RATE        HZ/3
/* if press button then polling every 0.1 sec , test time is 10*0.1 = 1 sec */
#define TEST_THRESHOLD   10
/* reboot time is > 1 sec and < 4 sec */
#define REBOOT_THRESHOLD 40

/*******************************************************************************
 *
 * turnOnLED - turn LED on associated with GPIO pin
 *
 * Input :  gpioPin used as LED (pin is set to output
 * 
 ********************************************************************************/

static void turnOnLED (int gpioPin)
{
    gpio_line_set(gpioPin,0);
}

/*******************************************************************************
 *
 * turnOffLED - turn LED off associated with GPIO pin
 *
 * Input :  gpioPin used as LED (pin is set to output
 * 
 ********************************************************************************/
static void turnOffLED (int gpioPin)
{
    gpio_line_set(gpioPin,1);
}

static void initLED (int gpioPin)
{
    gpio_line_config(gpioPin,IXP425_GPIO_OUT);
}

static void initPushButton (void)
{
    gpio_line_config(GPIO_PUSHBUTTON,IXP425_GPIO_IN);
}

static int isButtonPushed(void)
{
    int pushed;
    
    gpio_line_get(GPIO_PUSHBUTTON,&pushed);

    if(!pushed) return 1;
        
    return 0;
}

static void push_button_polling(unsigned long count)
{
	unsigned int interval=SCAN_RATE;
	
	PRINTK("<0>count==%ld\n",count);
	//turnOffLED(GPIO_TEST);
	if(count >= REBOOT_THRESHOLD && test_led_on==0){
		test_led_on=1;
	    turnOnLED(GPIO_TEST);
	}

	if((!isButtonPushed()) && count >= TEST_THRESHOLD){	
		if ( count > REBOOT_THRESHOLD ){
			strcpy(MyBuf,"RESTORE");
		}else if ( count > TEST_THRESHOLD){
			strcpy(MyBuf,"REBOOT");
			
		}else if( count == TEST_THRESHOLD){
			strcpy(MyBuf,"TEST");
		}
		
		up(&push_button_sem);

        /* if press GPIO == 0 */
	}
	else 
	{

		/* press */
        if(isButtonPushed()){
			if(count>=TEST_THRESHOLD && count<= REBOOT_THRESHOLD)
			{
			turnOnLED(GPIO_TEST);
			//turnOffLED(GPIO_TEST);	
			}
			push_button_timer.data=count+1;
			interval=HZ/10;
            /* not press or press less 1 sec */	
		}else{
			push_button_timer.data=0;
            interval=SCAN_RATE;
		}
		
        push_button_timer.expires=jiffies+interval;
		push_button_timer.function=push_button_polling;
        add_timer(&push_button_timer);
		
	}

	
}

#ifdef MAC_PROC
static ssize_t proc_read_macaddr_fops(struct file *filp,
                                      char *buf,size_t count , loff_t *offp)
{
    int len=0;
	unsigned char mac[MAX_ADDR_LEN];
	char macaddr[18]="";

	memset(mac,0,MAX_ADDR_LEN);
    if(*offp!=0)
        return 0;
	
	kerSysGetMacAddress(&mac,2);
	len=sprintf(macaddr,"%02X:%02X:%02X:%02X:%02X:%02X\n",mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);		
	macaddr[len]='\0';	
	copy_to_user(buf,macaddr,len);
	*offp = len;
    return len;
}
#endif

static ssize_t proc_read_push_button_fops(struct file *filp,
                                          char *buf,size_t count , loff_t *offp)
{
    int len=0;
    
    if(*offp!=0)
        return 0;
	
	/* start polling */
	push_button_timer.data=0;
    push_button_timer.expires=jiffies+SCAN_RATE;
	push_button_timer.function=push_button_polling;
    add_timer(&push_button_timer);	

	down_interruptible(&push_button_sem);
	
	len=strlen(MyBuf)+1;
	copy_to_user(buf,MyBuf,len);
	*offp = len;
    return len;
}

static ssize_t proc_write_push_button_fops(struct file *filp,const char *buffer,
                                           size_t count , loff_t *offp)
{
	up(&push_button_sem);	
	return count;
}

struct file_operations push_button_fops = {
    read: proc_read_push_button_fops,
    write: proc_write_push_button_fops,
};

static struct proc_dir_entry *push_button_file;

#ifdef MAC_PROC
struct file_operations get_macaddr_fops = {
    read: proc_read_macaddr_fops,
};

static struct proc_dir_entry *proc_macaddr;
#endif

static int __init push_button_file_init_module(void)
{
	PRINTK("init...\n");
	push_button_file=create_proc_entry("push_button",0666,&proc_root);
	push_button_file->owner = THIS_MODULE;
	push_button_file->proc_fops = &push_button_fops;
#ifdef MAC_PROC
	proc_macaddr=create_proc_entry("mac",0666,&proc_root);
	proc_macaddr->owner = THIS_MODULE;
	proc_macaddr->proc_fops = &get_macaddr_fops;
#endif    
	/* init gpio */
    initLED(GPIO_TEST);
    initPushButton();

	init_timer(&push_button_timer);
    
	sema_init(&push_button_sem,0);
	
	return 0;
}

static void __exit push_button_file_cleanup_module(void)
{
	remove_proc_entry("push_button",&proc_root);
#ifdef MAC_PROC    
	remove_proc_entry("mac",&proc_root);
#endif    
	del_timer(&push_button_timer);
	PRINTK("<0>cleanup...\n");
}

module_init(push_button_file_init_module);
module_exit(push_button_file_cleanup_module);
	
