#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "PRO_GetInt.h"
#include "PRO_SetInt.h"
#include "PRO_GetStr.h"
#include "PRO_SetStr.h"
#include "PRO_SetMultiItemStr.h"
#include "schling.h"
#include "email.h"
#include "h_var.h"

#define SYSCONF 	"/var/system.conf"
#define FILTERLOG 	"/var/log/filter.log"
#define ACCESSLOG 	"/var/log/access.log"
#define DOSLOG 		"/var/log/dos.log"

int ReadMailConf(mail_conf *conf)
{
	FILE *fp;
	char buf[16]={0}, ipbuf[16]={0};

	fp=fopen(SYSCONF,"r");
	if(!fp)
		return -1;
	PRO_GetInt(Section,DOS_EN,&(conf->dos_enable),fp);
	PRO_GetInt(Section,OUT,&(conf->out_enable),fp);
	PRO_GetInt(Section,IN,&(conf->in_enable),fp);
	PRO_GetInt(Section,MAIL,&(conf->mail_type),fp);
	PRO_GetStr(Section,TIME,buf,16,fp);
	PRO_GetStr(Section,RECV,conf->receiver,MaxEmailLen,fp);
	PRO_GetStr(Section,SUBJ,conf->subject,MaxEmailLen,fp);
	PRO_GetChar(Section,SMTPTYPE,(char *)&(conf->smtp_type),fp);
	PRO_GetStr(Section,SMTPSRV,conf->smtp_server,MaxEmailLen,fp);
	PRO_GetInt(Section,SMTPPORT,&(conf->smtp_port),fp);
	PRO_GetStr(Section,SMTPIP,ipbuf,16,fp);

	fclose(fp);

	sscanf(buf,"%d,%d,%d",&(conf->day),&(conf->hour),&(conf->ampm));
	extract_ip(ipbuf,(unsigned char *)&conf->smtp_serverip[0]);

	return 0;
}

int CheckMailConf(mail_conf *conf)
{
	if ((conf->dos_enable !=0) && (conf->dos_enable !=1))
		return INVALID_DOS_ENABLE;

	if ((conf->out_enable !=0) && (conf->out_enable !=1))
		return INVALID_OUT_ENABLE;

	if ((conf->in_enable !=0) && (conf->in_enable !=1))
		return INVALID_IN_ENABLE;

	if ((conf->mail_type !=0) && (conf->mail_type !=1))
		return INVALID_MAIL_TYPE;

	if ((conf->day <0) || (conf->day >8))
		return INVALID_DAY;

	if ((conf->hour <1) || (conf->hour >12))
		return INVALID_HOUR;

	if ((conf->ampm !=0) && (conf->ampm !=1))
		return INVALID_AMPM;

	if(strlen(conf->receiver)>MaxEmailLen)
		return INVALID_RECEIVER;

	if(strlen(conf->subject)>MaxEmailLen)
		return INVALID_SUBJECT;
	return 0;
}

int x12to24(Schl_Act *mail_schl, mail_conf *conf)
{

	if(!mail_schl || !conf)
		return -1;

	if(conf->hour==12){
		if(conf->ampm==0)
			mail_schl->hour=0;
	}
	else{
		if(conf->ampm && conf->hour < 12)
			mail_schl->hour = conf->hour + 12;
		else
			mail_schl->hour = conf->hour;
	}

	if(conf->day==0)
		mail_schl->date=7;
	else if(conf->day>6)
		mail_schl->date=conf->day + 1;
	else
		mail_schl->date=conf->day;

	return 0;
}

int SaveMailSchl(mail_conf *conf,char fname[32])
{
	Schl_Act mail_schl;
	char buf[128],subj[34]={0};
	int ret=0;

	if(conf->out_enable || conf->in_enable){
		memset(&mail_schl,'\0',sizeof(Schl_Act));
		if(conf->subject)
			sprintf(subj,"-s \"%s\"",conf->subject);
		sprintf(buf,"%s -m %s %s",subj,fname,conf->receiver);
		mail_schl.task=SCHL_EMAIL;
		mail_schl.tsk_para=buf;
		ret=x12to24(&mail_schl,conf);
		if(ret)
			return -1;
		mail_schl.min=0;
		Schl_Stop();
		ret=Schl_Add(&mail_schl, SCHL_TYP_WEEKLY);
		Schl_Start();
		return ret;
	}
	return 0;
}

int SaveMailConf(mail_conf *conf)
{
	multi_items all_items[11];
	char dos[2],out[2],in[2],mail[2],time[16]={0};
	char fname[32], smtptype[2], smtpsrvip[16], smtpport[6];
	int ret;

	ret=CheckMailConf(conf);
	if(ret)
		return ret;

	strcpy(all_items[0].key,DOS_EN);
	sprintf(dos,"%d",conf->dos_enable);
	all_items[0].value=dos;

	strcpy(all_items[1].key,OUT);
	sprintf(out,"%d",conf->out_enable);
	all_items[1].value=out;

	strcpy(all_items[2].key,IN);
	sprintf(in,"%d",conf->in_enable);
	all_items[2].value=in;

	strcpy(all_items[3].key,MAIL);
	sprintf(mail,"%d",conf->mail_type);
	all_items[3].value=mail;

	strcpy(all_items[4].key,TIME);
	sprintf(time,"%d,%d,%d",conf->day,conf->hour,conf->ampm);
	all_items[4].value=time;

	strcpy(all_items[5].key,RECV);
	all_items[5].value=conf->receiver;

	strcpy(all_items[6].key,SUBJ);
	all_items[6].value=conf->subject;

	strcpy(all_items[7].key,SMTPTYPE);
	sprintf(smtptype,"%d",conf->smtp_type);
	all_items[7].value=smtptype;

	strcpy(all_items[8].key,SMTPSRV);
	if (conf->smtp_type!=0)
	    strcpy(conf->smtp_server, "");
	all_items[8].value=conf->smtp_server;

	strcpy(all_items[9].key,SMTPIP);
	if (conf->smtp_type!=1)
        sprintf(smtpsrvip,"0.0.0.0");
    else
	    sprintf(smtpsrvip,"%d.%d.%d.%d",conf->smtp_serverip[0],conf->smtp_serverip[1],conf->smtp_serverip[2],conf->smtp_serverip[3]);
	all_items[9].value=smtpsrvip;

	strcpy(all_items[10].key,SMTPPORT);
	sprintf(smtpport,"%d",conf->smtp_port);
	all_items[10].value=smtpport;

	ret=PRO_SetMultiItemStr(Section,all_items,11,SYSCONF);
	if(ret)
		return ret;

	ret=Schl_Del(SCHL_EMAIL);
	if(ret)
		return -1;
//	Schl_Stop();
	if(conf->mail_type==1){
		if(conf->out_enable){
			strcpy(fname,FILTERLOG);
			ret=SaveMailSchl(conf,fname);
			if(ret)
				return ret;
		}
		if(conf->in_enable){
			strcpy(fname,ACCESSLOG);
			ret=SaveMailSchl(conf,fname);
			if(ret)
				return ret;
		}
	}

	return 0;
}
