#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "groups.h"
#include "serv.h"
#include "PRO_GetStrLen.h"
#include "PRO_SetStr.h"
#include "PRO_GetStr.h"

#define SYS_CONF	"/var/system.conf"

char BLOCK_NONE_STR[]   =       "BLOCK NONE";
char BLOCK_SELECT_STR[] =       "BLOCK SELECTED";
char BLOCK_ALL_STR[]    =       "BLOCK ALL";
char ALWAYS_STR[]       =       "Always";
char SCHEDULE_STR[]     =       "Schedule";

#define SAFEFREE(x)	do{	\
				if((x)!=NULL){		\
					free(x);	\
					(x)=NULL;	\
				}	\
			}while(0)


int ReadGroup(int gid,group_conf *group)
{
	FILE *fp;
	int num;
	char key[8];
	char *buf,*p,*q,*alloc_p;
	char **list;
	int len=0;

	if(gid<0 || gid>5)
		return FW_GROUP_GID_ERR;

	sprintf(key,"Group%1d",gid);

	fp=fopen(SYS_CONF,"r");
	if(!fp)
		return FW_FILE_NOT_EXIST;

	len=PRO_GetStrLen(GROUPS_SEC,key,fp);

	buf=(char *)malloc(len+1);
	alloc_p = buf;
	if(!alloc_p){
	    return FW_MEM_ERR;
	}

	memset(buf,0,len+1);
	if(PRO_GetStr(GROUPS_SEC,key,buf,len,fp)){
		fclose(fp);
		return FW_GROUP_READ_ERR;
	}
	fclose(fp);

    if( !memcmp(buf,SCHEDULE_STR,sizeof(SCHEDULE_STR)-1) ){
        group->schedule = GROUP_BASE_SCHEDULE;
        buf += sizeof(SCHEDULE_STR);
    }
    else{
        group->schedule = GROUP_ALWAYS;
        buf += sizeof(ALWAYS_STR);
    }


    if( !memcmp(buf, BLOCK_NONE_STR,sizeof(BLOCK_NONE_STR)-1) ){
        group->block = BLOCK_NONE;
    }
    else if ( !memcmp(buf,BLOCK_SELECT_STR,sizeof(BLOCK_SELECT_STR)-1) ) {
        group->block = BLOCK_SELECT;
    }
    else group->block=BLOCK_ALL;


	if(group->block==BLOCK_SELECT){

		num=(buf[ sizeof(BLOCK_SELECT_STR) ]-'0')*10+(buf[ sizeof(BLOCK_SELECT_STR) + 1 ]-'0');

		group->appname=(char **)malloc((num+1)*4);
		memset(group->appname,0,(num+1)*4);

		list=group->appname;
                   /* BLOCK SELECTED| |xx| | */
		p=&buf[sizeof(BLOCK_SELECT_STR) + 2 + 1 ];
		while(p && *p!='\0'){
			*list=strdup(p);
			list++;
			q=strchr(p,' ');
			if(q)
				*q=0;
			else
				break;
			p=q+1;
		}
	}

	if(alloc_p){
		SAFEFREE(alloc_p);
	}
	return 0;
}

void FreeGroup(group_conf *group)
{
	char **list;

	if(group){
		if(group->appname){
			list=group->appname;
			while(*list){
				SAFEFREE(*list);
				list++;
			}
			SAFEFREE(group->appname);
		}
	}

}

static int CheckGroup(group_conf *group)
{
	if(!group)
		return FW_GROUP_NOT_EXIST;
    if(group->schedule != GROUP_BASE_SCHEDULE && group->schedule != GROUP_ALWAYS)
        return FW_GROUP_SCHEDULE_ERR;
	if(group->block<BLOCK_NONE||group->block>BLOCK_ALL)
		return FW_GROUP_BLOCK_ERR;
	if(group->block==BLOCK_SELECT && group->appname==NULL)
		return FW_GROUP_APP_ERR;

	return 0;
}

int SaveGroup(int gid,group_conf *group,int num)
{
	char key[8];
	char *buf=NULL;
	char **list=NULL;
	int ret,i;
	char schedule_block[50];

	if(gid < 0 || gid >= GROUP_NUM)
		return FW_GROUP_GID_ERR;

	ret=CheckGroup(group);
	if(ret)
		return ret;

	sprintf(key,"Group%1d",gid);

	switch(group->block){
		case BLOCK_NONE:
		case BLOCK_ALL:
		    sprintf(schedule_block,"%s %s",
		        GROUP_BASE_SCHEDULE == group->schedule ? SCHEDULE_STR : ALWAYS_STR,
		        BLOCK_NONE == group->block ? BLOCK_NONE_STR : BLOCK_ALL_STR
		    );
		    PRO_SetStr(GROUPS_SEC,key,schedule_block,SYS_CONF);
			break;
		case BLOCK_SELECT:
			sprintf(schedule_block, "%s", GROUP_BASE_SCHEDULE == group->schedule ? SCHEDULE_STR : ALWAYS_STR);
			buf=(char *)malloc( strlen(schedule_block) + sizeof(BLOCK_SELECT_STR) + 5 +num*EntryNameLen );
			if(!buf){
			    break;
			}
			memset(buf, 0, strlen(schedule_block) + sizeof(BLOCK_SELECT_STR) + 5 +num*EntryNameLen);
			sprintf(buf,"%s %s 00 ", schedule_block, BLOCK_SELECT_STR);
			buf[strlen(schedule_block) + sizeof(BLOCK_SELECT_STR) + 1]=num/10+'0';
			buf[strlen(schedule_block) + sizeof(BLOCK_SELECT_STR) + 2]=num%10+'0';

			list=group->appname;
			for(i=0;i<num;i++){
				strcat(buf,*(list+i));
				strcat(buf," ");
			}
			PRO_SetStr(GROUPS_SEC,key,buf,SYS_CONF);
			free(buf);
			break;
		default:
			return -1;
	}
	WriteScript();
//	StartFW();
    Service_modify(gid);
	return 0;
}

int serv_in_group(char *name,int gid)
{
	FILE *fp;
	char key[20];
	char *buf;
	int len=0;

	if(!name)
		return 0;

	if(gid<0 || gid>5)
		return 0;

	sprintf(key,"Group%1d",gid);

	fp=fopen(SYS_CONF,"r");
	if(!fp)
		return 0;

	len=PRO_GetStrLen(GROUPS_SEC,key,fp);
	buf=(char *)malloc(len+1);

	memset(buf,0,len+1);
	PRO_GetStr(GROUPS_SEC,key,buf,len+1,fp);
	fclose(fp);

    if( !memcmp(buf,BLOCK_NONE_STR,sizeof(BLOCK_NONE_STR)-1) ) { /* BLOCK NONE */
		SAFEFREE(buf);
		return 0;
	}
	else if( !memcmp(buf,BLOCK_ALL_STR,sizeof(BLOCK_ALL_STR)-1) ){
		SAFEFREE(buf);
		return 1;
	}

	sprintf(key," %s ",name);

	if(strstr(&buf[ sizeof(BLOCK_SELECT_STR) + 2 ],key)){
		SAFEFREE(buf);
		return 1;
	}
	else{
		SAFEFREE(buf);
		return 0;
	}
}

/* Recorder group index in groups.htm after change group index. And this value
 * will be used in grp_mem.htm and sch_def.htm
 *                                                      Argon 2003-7-14
 */
#define GROUP_INDEX_FILE        "/var/groupIndex"

int RecorderGrpIndex(int groupIndex){
    FILE *fp = fopen(GROUP_INDEX_FILE, "w");

    if(!fp){
        return -1;
    }

    fprintf(fp,"%d\n",groupIndex);
    fclose(fp);
    return 0;
}

int GetGrpIndex(void){
    FILE *fp = fopen(GROUP_INDEX_FILE,"r");
    int grpIndex = 0;

    if(!fp){
        return -1;
    }
    fscanf(fp,"%d\n",&grpIndex);
    fclose(fp);
    return grpIndex;
}

int ServiceInit(void){
    int i,len=0;
    char cmd[128];

    len += sprintf(cmd+len, "%s", CMD_ACC_CTRL_SH);
        
    for(i=0; i<GROUP_NUM; i++){
        len += sprintf(cmd+len, " %d", i);
    }
    
    debug(cmd);
    
    return system(cmd);
}

int Service_modify(int groupIndex){
    char cmd[128];
    
    sprintf(cmd,"%s %d", CMD_ACC_CTRL_SH, groupIndex);
    
    debug(cmd);
    
    return system(cmd);
}


int gid_modify(struct in_addr ip, int prev_gid, int curr_gid){
    char cmd[128];
    char *add = "add";
    char *del = "del";
    
    sprintf(cmd,"%s %s %s %d %s %s %d", 
        CMD_GID_MODIFY_SH, del, inet_ntoa(ip), prev_gid, add, inet_ntoa(ip), curr_gid);
    
    debug(cmd);
    
    return system(cmd);
}
