#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "sch_def.h"
#include "PRO_GetStr.h"
#include "PRO_SetMultiItemStr.h"
#include "groups.h"


#define SYS_CONF	"/var/system.conf"
#define GROUP_SCH   "Group%d_%s"

int ReadSchDef(sch_def_conf *sch, int groupIndex)
{
	FILE *fp;
    char groupStr[20];

	fp=fopen(SYS_CONF,"r");
	if(!fp){
		return FW_FILE_NOT_EXIST;
	}

    if( groupIndex < 0 || groupIndex >= GROUP_NUM){
        return FW_GROUP_GID_ERR;
    }


    sprintf(groupStr, GROUP_SCH, groupIndex, MON);
	PRO_GetStr(SCH_DEF_SEC,groupStr,sch->weekday[MONDAY],MAX_SCH_LEN,fp);

    sprintf(groupStr, GROUP_SCH, groupIndex, TUE);
	PRO_GetStr(SCH_DEF_SEC,groupStr,sch->weekday[TUESDAY],MAX_SCH_LEN,fp);

    sprintf(groupStr, GROUP_SCH, groupIndex, WED);
	PRO_GetStr(SCH_DEF_SEC,groupStr,sch->weekday[WEDNESDAY],MAX_SCH_LEN,fp);


    sprintf(groupStr, GROUP_SCH, groupIndex, THU);
	PRO_GetStr(SCH_DEF_SEC,groupStr, sch->weekday[THURSDAY],MAX_SCH_LEN,fp);

    sprintf(groupStr, GROUP_SCH, groupIndex, FRI);
    PRO_GetStr(SCH_DEF_SEC,groupStr, sch->weekday[FRIDAY],MAX_SCH_LEN,fp);

    sprintf(groupStr, GROUP_SCH, groupIndex, SAT);
	PRO_GetStr(SCH_DEF_SEC,groupStr, sch->weekday[SATURDAY],MAX_SCH_LEN,fp);

	sprintf(groupStr, GROUP_SCH, groupIndex, SUN);
	PRO_GetStr(SCH_DEF_SEC,groupStr, sch->weekday[SUNDAY],MAX_SCH_LEN,fp);

	fclose(fp);

	return 0;
}


static int CheckSchDef(sch_def_conf *sch)
{
	int len_full=MAX_SCH_LEN-1;
	int len_half=MAX_SCH_LEN/2-1;
	int len_zero=0;
    int i;

#define	LEN_MATCH(x)	(strlen(x)==len_full || strlen(x)==len_half || strlen(x)==len_zero)


	if(!sch)
		return FW_SCH_NOT_EXIST;
	for(i=0; i<WEEK_DAY; i++){
	    if(!LEN_MATCH(sch->weekday[i])){
		    return FW_SCH_MON_ERR;
		}
	}
#undef 	LEN_MATCH

	return 0;
}

int SaveSchDef(sch_def_conf *sch, int groupIndex)
{
	multi_items mi[7];
	int ret;

	ret=CheckSchDef(sch);
	if(ret){
		return ret;
	}

    if( groupIndex < 0 || groupIndex >= GROUP_NUM){
        return FW_GROUP_GID_ERR;
    }

    /* Key Max length is 24. */
    sprintf(mi[0].key, GROUP_SCH, groupIndex, MON);
	sprintf(mi[1].key, GROUP_SCH, groupIndex, TUE);
	sprintf(mi[2].key, GROUP_SCH, groupIndex, WED);
	sprintf(mi[3].key, GROUP_SCH, groupIndex, THU);
	sprintf(mi[4].key, GROUP_SCH, groupIndex, FRI);
	sprintf(mi[5].key, GROUP_SCH, groupIndex, SAT);
	sprintf(mi[6].key, GROUP_SCH, groupIndex, SUN);

	mi[0].value=sch->weekday[MONDAY];
	mi[1].value=sch->weekday[TUESDAY];
	mi[2].value=sch->weekday[WEDNESDAY];
	mi[3].value=sch->weekday[THURSDAY];
	mi[4].value=sch->weekday[FRIDAY];
	mi[5].value=sch->weekday[SATURDAY];
	mi[6].value=sch->weekday[SUNDAY];

	if(PRO_SetMultiItemStr(SCH_DEF_SEC,mi,7,SYS_CONF))
		return FW_SCH_SAVE_ERR;

	return 0;
}


/*
 *  inSchedule -- This function will read system.conf and judge if time is in schedule.
 *
 */
int inSchedule(int groupIndex, time_t givenTime){
    struct tm *timep;
    sch_def_conf	schConf;
    char hs1[3]={0};
    char ms1[3]={0};
    char hf1[3]={0};
    char mf1[3]={0};

    char hs2[3]={0};
    char ms2[3]={0};
    char hf2[3]={0};
    char mf2[3]={0};

    int start1,finish1,start2,finish2;
    int now;
    int strLen;

    timep = localtime((const time_t *)&givenTime);
    ReadSchDef(&schConf,groupIndex);

    strLen = strlen(schConf.weekday[timep->tm_wday]);
    if( !strLen){ /* String is null , always false. */
        return 0;
    }

    if(strLen == 11){
		sscanf(schConf.weekday[timep->tm_wday],SCH_HALF_FORMAT,hs1,ms1,hf1,mf1);
	}
	else if(strLen==23){
		sscanf(schConf.weekday[timep->tm_wday],SCH_FULL_FORMAT,hs1,ms1,hf1,mf1,hs2,ms2,hf2,mf2);
	}

    now = timep->tm_hour * 60 + timep->tm_min;

    start1 = atoi(hs1) * 60 + atoi(ms1);
    finish1 = atoi(hf1) * 60 + atoi(mf1);
debug("strLen is %d, givenTime is %lu now is %d start1 is %d finish1 is %d. timep->tm_wday is %d",strLen,givenTime,now,start1,finish1,timep->tm_wday);
	if( strLen == 11 ){
        if(now >= start1 && now <= finish1){
            return 1;
        }
        else{
            return 0;
        }
    }
    else if ( strLen == 23 ){
        start2 = atoi(hs2) * 60 + atoi(ms2);
        finish2 = atoi(hf2) * 60 + atoi(mf2);

        if((now >= start1 && now <= finish1) || (now >= start2 && now <= finish2)){
            return 1;
        }
        else{
            return 0;
        }
    }
    return 0;
}

int sch_modify(int gid , time_t givenTime){
    char cmd[128];
    int in_sch;
    
    if(inSchedule(gid,givenTime)){
        in_sch = 1;
    }
    else{
        in_sch = 0;
    }
    
    sprintf(cmd,"%s %s %d", CMD_SCH_SH, in_sch?"start":"stop", gid);
    
    debug(cmd);
    
    return system(cmd);
}




