#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "serv.h"
#include "PRO_DelKey.h"
#include "PRO_SetStr.h"
#include "PRO_2sect.h"
#include "wan.h"
#include "system.h"

#define SYS_CONF	"/var/system.conf"

#define SAFEFREE(x)	do{	\
				if((x)!=NULL){		\
					free(x);	\
					(x)=NULL;	\
				}	\
			}while(0)

int ReadSERV(serv_conf **pconf,int *num, int poe_acc_index)
{
	serv_conf *head=NULL,*aitem=NULL,*pitem=NULL;
	FILE *fp;
	char buf[128];
	char *p,*q;
	int n=0;
	char sec_name[10];
	fp=fopen(SYS_CONF,"r");
	if(!fp)
		return FW_FILE_NOT_EXIST;
		
	if(poe_acc_index>=0&&poe_acc_index<MAX_PPPOE_ACCOUNT)
		sprintf(sec_name,"%s%d",SERV_SEC,poe_acc_index+1);	
	else
		sprintf(sec_name,"%s",SERV_SEC);	
			
	if(PRO_2sect(sec_name,fp)){
		fclose(fp);
		return FW_SERV_READ_ERR;
	}
		
	while(fgets(buf,128,fp)){
		if(buf[0]=='[')
			break;
		p = strstr(buf,"\n");
		if (p)
			*p = 0; // remove '\n'
		
		p=strchr(buf,'=');
		if(!p)
			continue;
		else
			*p=0;
			
		aitem=malloc(sizeof(struct serv_conf));

		strcpy(aitem->name,buf);
		p++;

		if(*p=='1')
			aitem->reserver=RESERVERED;
		else
			aitem->reserver=UNRESERVERED;
			
		p+=2;
		q=strchr(p,':');
		if(q)
			*q=0;
		strcpy(aitem->protocol,p);

                q++;
                strcpy(aitem->port_info,q);
		aitem->next=NULL;
		
		n++;
		
		if(head==NULL){
			head=aitem;
			pitem=head;
                }
                else{
			pitem->next=aitem;
			pitem=pitem->next;
		}
		
	}
	fclose(fp);
	if(head)
		*pconf=head;
	if(num)
		*num=n;
	
	return 0;
}

void FreeSERV(serv_conf *p)
{
	serv_conf *q;
	
	while(p){
		q=p->next;
		SAFEFREE(p);
		p=q;
	}
}

int CheckSERV(serv_conf *p)
{
	if(!p)
		return FW_SERV_NOT_EXIST;
	if(strlen(p->name)==0 || strlen(p->name)>=EntryNameLen)
		return FW_SERV_NAME_ERR;
	if(p->reserver!=RESERVERED && p->reserver!=UNRESERVERED)
		return FW_SERV_RESV_ERR;
	if(strlen(p->protocol)==0)
		return FW_SERV_PRO_ERR;
	if(strlen(p->port_info)==0)
		return FW_SERV_PORT_ERR;
		
	return 0;
	
}

int AddSERV(serv_conf *p, int poe_acc_index)
{
	char buf[128];
	int ret;
	char sec_name[10];
	
	ret=CheckSERV(p);
	if(ret)	
		return ret;
		
	if(poe_acc_index>=0&&poe_acc_index<MAX_PPPOE_ACCOUNT)
		sprintf(sec_name,"%s%d",SERV_SEC,poe_acc_index+1);	
	else
		sprintf(sec_name,"%s",SERV_SEC);	
			
	sprintf(buf,"%d,%s:%s",UNRESERVERED,p->protocol,p->port_info);

	ret=PRO_SetStr(sec_name,p->name,buf,SYS_CONF);
	if(ret)
		return FW_SERV_ADD_ERR;
//	WriteScript();
//	StartFW();
	return 0;
}

int DelSERV(char **name,int num, int poe_acc_index)
{
	int i;
	int ret=0;
	char sec_name[10];
	
	if(!name||num==0)
		return 0;
		
	if(poe_acc_index>=0&&poe_acc_index<MAX_PPPOE_ACCOUNT)
		sprintf(sec_name,"%s%d",SERV_SEC,poe_acc_index+1);	
	else
		sprintf(sec_name,"%s",SERV_SEC);
		
	for(i=0;i<num;i++)
		ret+=PRO_DelKey(sec_name,*(name+i),SYS_CONF);
	if(ret)
		return FW_SERV_DEL_ERR;
//	WriteScript();
//	StartFW();

	return 0;
}
