#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#include "vs.h"
#include "PRO_DelKey.h"
#include "PRO_2sect.h"
#include "PRO_SetStr.h"
#include "PRO_GetStr.h"
#include "wan.h"

#define SYS_CONF	"/var/system.conf"

#define SAFEFREE(x)	do{	\
				if((x)!=NULL){		\
					free(x);	\
					(x)=NULL;	\
				}	\
			}while(0)

int ReadVS(vs_conf **pconf,int *num, int poe_acc_index)
{
	vs_conf *head=NULL,*aitem=NULL,*pitem=NULL;
	FILE *fp;
	char buf[128];
	char *p;
	int n=0;
	char ip[16];
	char sec_name[10];
	
	fp=fopen(SYS_CONF,"r");
	if(!fp)
		return FW_FILE_NOT_EXIST;
		
	if(poe_acc_index>=0&&poe_acc_index<MAX_PPPOE_ACCOUNT)
		sprintf(sec_name,"%s%d",VS_SEC,poe_acc_index+1);	
	else
		sprintf(sec_name,"%s",VS_SEC);	
			
	if(PRO_2sect(sec_name,fp)){
		fclose(fp);
		return FW_VS_READ_ERR;
	}
	
	while(fgets(buf,128,fp)){
		if(buf[0]=='[')
			break;
		p = strstr(buf,"\n");
		if (p)
			*p = 0; // remove '\n'

		p=strchr(buf,'=');
		if(!p)
			continue;
		else
			*p=0;

		aitem=malloc(sizeof(struct vs_conf));
		memset(aitem,0,sizeof(struct vs_conf));

		strcpy(aitem->name,buf);

		p++;

		aitem->reserver=*p-'0';

		p+=2;

		aitem->enable=*p-'0';

		p+=2;

		sscanf(p,"%s %s %s %s %s %s",ip, aitem->protocol, aitem->in_p, aitem->in_endp, aitem->ext_p, aitem->ext_endp);
		inet_aton(ip,&aitem->pc_ip);

		n++;

		if(head==NULL){
			head=aitem;
			pitem=head;
                }
                else{
			pitem->next=aitem;
			pitem=pitem->next;
		}

	}
	fclose(fp);
	if(head)
		*pconf=head;
	if(num)
		*num=n;
	
	return 0;
}

void FreeVS(vs_conf *p)
{
	vs_conf *q;

	while(p){
		q=p->next;
		SAFEFREE(p);
		p=q;
	}
}

static int CheckVS(vs_conf *p)
{
	int in_port,ext_port;
	int in_endport,ext_endport;

	if(!p)
		return FW_VS_NOT_EXIST;
	if(strlen(p->name)==0)
		return FW_VS_NAME_ERR;
	if(p->reserver!=RESERVERED && p->reserver!=UNRESERVERED)
		return FW_VS_RESV_ERR;
	if(p->enable!=ENABLE && p->enable!=DISABLE)
		return FW_VS_ENABLE_ERR;
//	if(inet_addr(p->pc_ip) == INADDR_NONE )
//		return FW_VS_PC_ERR;

	in_port=atoi(p->in_p);
	ext_port=atoi(p->ext_p);
	if(in_port<=0 || in_port>=65536)
		return FW_VS_INPORT_ERR;
	if(ext_port<=0 || ext_port>=65536)
		return FW_VS_EXTPORT_ERR;

	in_endport=atoi(p->in_endp);
	ext_endport=atoi(p->ext_endp);
// 	cgi_err(p->in_endp,p->ext_endp);

	if(in_endport<=0 || in_endport>=65536)
		return FW_VS_INPORT_ERR;
	if(ext_endport<=0 || ext_endport>=65536)
		return FW_VS_EXTPORT_ERR;

	return 0;
}

int AddVS(vs_conf *p, int poe_acc_index)
{
	int ret;
	char buf[128];
	char sec_name[10];

	if(!p)
		return 0;
	ret=CheckVS(p);
	if(ret)
		return ret;

	sprintf(buf,"%d %d %s %s %s %s %s %s",UNRESERVERED,p->enable,inet_ntoa(p->pc_ip),p->protocol,p->in_p,p->in_endp,p->ext_p,p->ext_endp);

	if(poe_acc_index>=0&&poe_acc_index<MAX_PPPOE_ACCOUNT)
		sprintf(sec_name,"%s%d",VS_SEC,poe_acc_index+1);	
	else
		sprintf(sec_name,"%s",VS_SEC);	
		
	ret=PRO_SetStr(sec_name,p->name,buf,SYS_CONF);
	if(ret)
		return FW_VS_ADD_ERR;
//	WriteScript();
//	StartFW();
    vs_modify(p,ADD_VS);

	return 0;
}

int DelVS(char *name, int poe_acc_index)
{
	vs_conf *pConf;
	char sec_name[10];	
	if(!name){
		return 0;
	}
	    
    pConf = GetVSByName(name,poe_acc_index);
    if(!pConf){
        return 0;
    }
    
	if(poe_acc_index>=0&&poe_acc_index<MAX_PPPOE_ACCOUNT)
		sprintf(sec_name,"%s%d",VS_SEC,poe_acc_index+1);	
	else
		sprintf(sec_name,"%s",VS_SEC);
				
	if(PRO_DelKey(sec_name,name,SYS_CONF))
		return FW_VS_DEL_ERR;
    
//    WriteCommScript();
//	WriteScript();
//	StartFW();
    vs_modify(pConf,DEL_VS);

	return 0;
}


int UpdateVS(char *name,vs_conf *p, int poe_acc_index)
{
	int ret;
	FILE *fp;
	char buf[128],*q;
	vs_conf *pConf;
	char sec_name[10];	
	
	if(!p || !name){
		return 0;
	}

    debug("%s:%s():[%d]: time is %lu.",__FILE__,__FUNCTION__,__LINE__,time(0));
    
	if(poe_acc_index>=0&&poe_acc_index<MAX_PPPOE_ACCOUNT)
		sprintf(sec_name,"%s%d",VS_SEC,poe_acc_index+1);	
	else
		sprintf(sec_name,"%s",VS_SEC);	
		
	pConf = GetVSByName(name,poe_acc_index);
	ret=CheckVS(p);
	if(ret){
		return ret;
	}
	
	fp=fopen(SYS_CONF,"r");
	if(!fp){
		return FW_FILE_NOT_EXIST;
	}
	PRO_GetStr(sec_name,name,buf,sizeof(buf),fp);
	fclose(fp);
    debug("%s:%s():[%d]: time is %lu.",__FILE__,__FUNCTION__,__LINE__,time(0));

	if(strcmp(name,p->name))
		PRO_DelKey(sec_name,name,SYS_CONF);
	q=&buf[2];

	memset(q,0,strlen(q));
	sprintf(q,"%d %s %s %s %s %s %s",p->enable,inet_ntoa(p->pc_ip),p->protocol,p->in_p,p->in_endp,p->ext_p,p->ext_endp);
	ret=PRO_SetStr(sec_name,p->name,buf,SYS_CONF);
	if(ret)
		return FW_VS_UPDATE_ERR;
    debug("%s:%s():[%d]: time is %lu.",__FILE__,__FUNCTION__,__LINE__,time(0));

//    WriteCommScript();
//	WriteScript();
//	StartFW();

    debug("%s:%s():[%d]: time is %lu.",__FILE__,__FUNCTION__,__LINE__,time(0));
    vs_modify(pConf,DEL_VS);
    debug("%s:%s():[%d]: time is %lu.",__FILE__,__FUNCTION__,__LINE__,time(0));
    //ada----2003-20-9----
//  vs_modify(p,ADD_VS);
    if(p->enable){
    vs_modify(p,ADD_VS);
    }
    //ada
    debug("%s:%s():[%d]: time is %lu.",__FILE__,__FUNCTION__,__LINE__,time(0));
    
	return 0;

}


int ResetVS(int poe_acc_index)
{
	int ret;
	struct stat st;
	FILE *fp;
	char *buf,*p,*q,*r;
	char key[30];
	int i=0;
	char sec_name[10];
	
	ret=stat(SYS_CONF,&st);
	if(ret<0)
		return -1;
		
	if(poe_acc_index>=0&&poe_acc_index<MAX_PPPOE_ACCOUNT)
		sprintf(sec_name,"%s%d",VS_SEC,poe_acc_index+1);	
	else
		sprintf(sec_name,"%s",VS_SEC);	
		
	buf=(char *)alloca(st.st_size+1);
	if(!buf)
		return -2;
	memset(buf,0,st.st_size+1);
	fp=fopen(SYS_CONF,"r");
	ret=fread(buf,1,st.st_size,fp);
	fclose(fp);
	if(ret!=st.st_size)
		return -3;
	sprintf(key,"[%s]",sec_name);

	if(strncmp(buf,key,strlen(key))!=0){
		//The first section did not match
		sprintf(key,"\n[%s]",sec_name);
		p=strstr(buf,key);
		if(!p)
			return -4;
	}
	else
		p=buf;
	//Do not go to next section

	r=strstr(p+1,"\n[");

	while(i<RESERVERED_NUM){

		q=strchr(p,'=');
		if(q)
			i++;
		q=q+3;
		*q='0';
		q=q+2;
		*q='*';
		p=q;
	}
	q=strchr(p,'\n');
	if(q)
		*(q+1)=0;
	fp=fopen(SYS_CONF,"w");
	fwrite(buf,strlen(buf),1,fp);
	if(r)
		fwrite(r+1,strlen(r+1),1,fp);

	fflush(fp);
	fclose(fp);

//	WriteScript();
//	StartFW();
    vs_clear();
	return 0;
}


int DisableAllVS(int poe_acc_index)
{
	int ret;
	struct stat st;
	FILE *fp;
	char *buf,*p,*q,*r;
	char key[30];
	char sec_name[10];
		
	if(poe_acc_index>=0&&poe_acc_index<MAX_PPPOE_ACCOUNT)
		sprintf(sec_name,"%s%d",VS_SEC,poe_acc_index+1);	
	else
		sprintf(sec_name,"%s",VS_SEC);	
		
	ret=stat(SYS_CONF,&st);
	if(ret<0)
		return -1;

	buf=(char *)alloca(st.st_size+1);
	if(!buf)
		return -2;
	memset(buf,0,st.st_size+1);
	fp=fopen(SYS_CONF,"r");
	ret=fread(buf,1,st.st_size,fp);
	fclose(fp);
	if(ret!=st.st_size)
		return -3;
	sprintf(key,"[%s]",sec_name);

	if(strncmp(buf,key,strlen(key))!=0){
		//The first section did not match
		sprintf(key,"\n[%s]",sec_name);
		p=strstr(buf,key);
		if(!p)
			return -4;
	}
	else
		p=buf;
	//Do not go to next section
	r=strstr(p+1,"\n[");

	q=strchr(p,'=');
	while(q){
		if(r!=NULL && q>r)
			break;
		p=q+3;
		*p='0';
		q=strchr(p,'=');
	}
	// OK now.
	fp=fopen(SYS_CONF,"w");
	fwrite(buf,st.st_size,1,fp);
	fflush(fp);
	fclose(fp);

//	WriteScript();
//	StartFW();
    vs_clear();

	return 0;
}

int vs_modify(vs_conf *p,int add){
    char cmd[128];
    char protocol[10];
    char *q;
    
    if(!p->pc_ip.s_addr){ /* server IP is zero! */
        return 0;
    }
    
    strcpy(protocol, p->protocol);
    q = strchr(protocol,'/');
    if(q){
        *q = ' ';
    }
    
    sprintf(cmd, "%s %s %s \"%s\" %s:%s %s:%s >/dev/null 2>&1", 
        VS_MODIFY,
        add?"add":"del",
        inet_ntoa(p->pc_ip),
        protocol,
        p->ext_p,p->ext_endp,
        p->in_p,p->in_endp
    );
    debug(cmd);
    return system(cmd);    
}

int vs_clear(){
    system("/sbin/iptables -t nat -F NAT_VS_DNAT_CHAIN");
    system("/sbin/iptables -t nat -F NAT_VS_SNAT_CHAIN");
    return 0;
}

vs_conf *GetVSByName(char *name, int poe_acc_index){
	FILE *fh;
	char key[32];
	char buf[128];
	static vs_conf conf;
	char ip[16];
	char *p;
	char sec_name[10];
		
	if(!name){
		return NULL;
	}

    fh=fopen(SYS_CONF,"rt");
    
	if(!fh){
	    return NULL;
	}
	
	rewind(fh);
	
	if(poe_acc_index>=0&&poe_acc_index<MAX_PPPOE_ACCOUNT)
		sprintf(sec_name,"%s%d",VS_SEC,poe_acc_index+1);	
	else
		sprintf(sec_name,"%s",VS_SEC);	
			
	strcpy(key,name);
	PRO_GetStr(sec_name, key, buf, sizeof(buf), fh);
    p = buf;
    
    conf.reserver=*p-'0';

    p+=2;

    conf.enable=*p-'0';

    p+=2;

    sscanf(p, "%s %s %s %s %s %s", ip, conf.protocol, conf.in_p, conf.in_endp, conf.ext_p, conf.ext_endp);
    inet_aton(ip,&conf.pc_ip);

    fclose(fh);
    	
    return &conf;
}

int VSChainInit(void){
    vs_conf *pConf, *p;
    int VsNum;
    
    ReadVS(&pConf,&VsNum,-1);
    p=pConf;
	while(p){
	    if(p->enable){
	        vs_modify(p, ADD_VS);
	    }	    
		p=p->next;
	}
	FreeVS(pConf);
	return 0;
}

