#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <arpa/inet.h>
#include <unistd.h>

#include "PRO_file.h"
#include "PRO_GetStr.h"
#include "PRO_GetInt.h"
#include "PRO_SetStr.h"
#include "PRO_GetChar.h"
#include "PRO_GetShort.h"
#include "PRO_GetLong.h"
#include "PRO_DelKey.h"
#include "PRO_DelSEC.h"

#include "PRO_SetMultiItemStr.h"

#include "lan.h"
#include "h_var.h"
#include "h_const.h"
#include "dev_opt.h"

char lan_sec[] = "LAN";
char pc_db_sec[]="PC_DB";

int group_index=0; /* THis value will be used by grp_mdm_cgi.c */

LAN_CONF lan_conf;

extern int mkdhcpdconf_main(void);

int ReadLanSettings(LAN_CONF *pConf)
{

	char key[32];
	char ip[16];
	FILE *fh;

	if(!pConf)
	    return LAN_ReadConfErr;

        fh=fopen(SYSCONF,"rt");
	if(!fh)
	    return LAN_ReadConfErr;
	rewind(fh);

	sprintf(key,"Lan_Ip");
	PRO_GetStr(lan_sec,key,ip,16,fh);
	extract_ip(ip,(unsigned char *)&pConf->LanIPAddr);

	sprintf(key,"Lan_Mask");
	PRO_GetStr(lan_sec,key,ip,16,fh);
	extract_ip(ip,(unsigned char *)&pConf->NetMaskIPAddr);

	sprintf(key,"Lan_Ip2"); /* For Unnumbered IP. */
	PRO_GetStr(lan_sec,key,ip,16,fh);
	extract_ip(ip,(unsigned char *)&pConf->LanIPAddr2);

	sprintf(key,"Lan_Mask2"); /* For Unnumbered IP. */
	PRO_GetStr(lan_sec,key,ip,16,fh);
	extract_ip(ip,(unsigned char *)&pConf->NetMaskIPAddr2);

	sprintf(key,"Dhcp");
	PRO_GetChar(lan_sec,key,(char *)&pConf->DhcpEnable,fh);

    sprintf(key,"StartIp");
	PRO_GetStr(lan_sec,key,ip,16,fh);
	extract_ip(ip,(unsigned char *)&pConf->StartIp);

	sprintf(key,"FinishIp");
	PRO_GetStr(lan_sec,key,ip,16,fh);
	extract_ip(ip,(unsigned char *)&pConf->FinishIp);

	fclose(fh);

	return 0;
}

int SaveLanSettings(LAN_CONF *pConf)
{
	char ip[16],mask[16],ip2[16],mask2[16],dhcp[2],startip[16],finiship[16];
	multi_items	all_items[LAN_ITEM_NUMBER];
	int ret;

	sprintf(all_items[LAN_ITEM_LAN_IP].key,"Lan_Ip");
	sprintf(ip,"%u.%u.%u.%u",*((unsigned char*)&pConf->LanIPAddr),*((unsigned char*)&pConf->LanIPAddr+1),*((unsigned char*)&pConf->LanIPAddr+2),*((unsigned char*)&pConf->LanIPAddr+3) );
	all_items[LAN_ITEM_LAN_IP].value=ip;

	sprintf(all_items[LAN_ITEM_LAN_MASK].key,"Lan_Mask");
	sprintf(mask,"%u.%u.%u.%u",*((unsigned char*)&pConf->NetMaskIPAddr),*((unsigned char*)&pConf->NetMaskIPAddr+1),*((unsigned char*)&pConf->NetMaskIPAddr+2),*((unsigned char*)&pConf->NetMaskIPAddr+3) );
	all_items[LAN_ITEM_LAN_MASK].value=mask;

	sprintf(all_items[LAN_ITEM_LAN_IP2].key,"Lan_Ip2");  /* For Unnumbered IP. */
	sprintf(ip2,"%u.%u.%u.%u",*((unsigned char*)&pConf->LanIPAddr2),*((unsigned char*)&pConf->LanIPAddr2+1),*((unsigned char*)&pConf->LanIPAddr2+2),*((unsigned char*)&pConf->LanIPAddr2+3) );
	all_items[LAN_ITEM_LAN_IP2].value=ip2;

	sprintf(all_items[LAN_ITEM_LAN_MASK2].key,"Lan_Mask2");  /* For Unnumbered IP. */
	sprintf(mask2,"%u.%u.%u.%u",*((unsigned char*)&pConf->NetMaskIPAddr2),*((unsigned char*)&pConf->NetMaskIPAddr2+1),*((unsigned char*)&pConf->NetMaskIPAddr2+2),*((unsigned char*)&pConf->NetMaskIPAddr2+3) );
	all_items[LAN_ITEM_LAN_MASK2].value=mask2;

	sprintf(all_items[LAN_ITEM_DHCP].key,"Dhcp");
	sprintf(dhcp,"%d",pConf->DhcpEnable);
	all_items[LAN_ITEM_DHCP].value=dhcp;

	sprintf(all_items[LAN_ITEM_STARTIP].key,"StartIp");
	sprintf(startip,"%u.%u.%u.%u",*((unsigned char*)&pConf->StartIp),*((unsigned char*)&pConf->StartIp+1),*((unsigned char*)&pConf->StartIp+2),*((unsigned char*)&pConf->StartIp+3) );
	all_items[LAN_ITEM_STARTIP].value=startip;

	sprintf(all_items[LAN_ITEM_FINISHIP].key,"FinishIp");
	sprintf(finiship,"%u.%u.%u.%u",*((unsigned char*)&pConf->FinishIp),*((unsigned char*)&pConf->FinishIp+1),*((unsigned char*)&pConf->FinishIp+2),*((unsigned char*)&pConf->FinishIp+3) );
	all_items[LAN_ITEM_FINISHIP].value=finiship;

	ret=PRO_SetMultiItemStr(lan_sec,all_items,LAN_ITEM_NUMBER, SYSCONF);
	if(ret){
		return LAN_SaveConfErr;
	}
	return LAN_OK;
}

int RestoreLANSettings(LAN_CONF *pConf){

    pConf->LanIPAddr        = htonl(LAN_IP_DEFAULT_HEX);
    pConf->NetMaskIPAddr    = htonl(LAN_MASK_DEFAULT_HEX);
    pConf->DhcpEnable       = LAN_DHCPS_DEFAULT_HEX;
    pConf->StartIp          = htonl(LAN_START_IP_DEFAULT_HEX);
    pConf->FinishIp         = htonl(LAN_FINISH_IP_DEFAULT_HEX);

    pConf->LanIPAddr2       = htonl(LAN_IP2_DEFAULT_HEX);
    pConf->NetMaskIPAddr2   = htonl(LAN_MASK2_DEFAULT_HEX);

    return 0;
}

/* PC_DB begin */
PC_DB_CONF_s pc_db[WK_MAX_NUM];
PC_DB_CONF pc_db_conf;
/************************************************************************
 * FUNCTION    : pc_db_link_init()                                      *
 *                                                                      *
 * INPUT       : NULL                                                   *
 *                                                                      *
 * OUTPUT      : NULL                                                   *
 *                                                                      *
 * DESCRIPTION : Init PC database link.                                 *
 *               After this function , PC database should be like this: *
 *----------------------------------------------------------------------*
 *      +-------+    +-------+    +-------+                  +-------+  *
 *      |   0   |--> |   1   |--> |   2   |-->.............. | MAX-1 |  *
 *      +-------+    +-------+    +-------+                  +-------+  *
 ***********************************************************************/
void pc_db_link_init(void)
{
    short i;

    pc_db_conf.number = 0;
    pc_db_conf.p_database = &pc_db[0];

    memset(&pc_db[WK_MAX_NUM-1], 0, sizeof(PC_DB_CONF_s));
    for (i=WK_MAX_NUM-2; i>=0; i--)
    {
        memset(&pc_db[i], 0, sizeof(PC_DB_CONF_s));
        pc_db[i].next = &pc_db[i+1];
    }
}

/************************************************************************
 * FUNCTION    : PCDBGetStr()                                           *
 *                                                                      *
 * INPUT       : p          ==> Char point to source string.            *
 *               ostr       ==> Char point to out put string.           *
 *               separate   ==> Seperate character.                     *
 *                                                                      *
 * OUTPUT      : Next item in p                                         *
 *                                                                      *
 * DESCRIPTION : Get item from the whole line                           *
 *                                                                      *
 * **NOTE**    : PC name should not have 'separate'!!!!                 *
 *----------------------------------------------------------------------*
 * Before      :  bobby 11-22-33-44-55-66 192.168.1.3 1 1 0 0 0         *
 *               ^                                                      *
 *               p                                                      *
 * After       :  bobby 11-22-33-44-55-66 192.168.1.3 1 1 0 0 0         *
 *                     ^                                                *
 *                     q                                                *
 *             :  ostr is "bobby"                                       *
 ***********************************************************************/
static char *PCDBGetStr(char *p, char *ostr, char separate)
{
    char *q;

    if(!p){
        return NULL;
    }
    while (*p == separate)
        p++;

    if (!*p)
        return NULL;

    for (q=p; (*q != separate) && (*q); q++);
    strncpy(ostr, p, (q-p));
    ostr[q-p] = '\0';

    return q;

}
/************************************************************************
 * FUNCTION    : PCDBExtractInfo()                                      *
 *                                                                      *
 * INPUT       : str        ==> Char point to entry begin               *
 *               p_Conf     ==> PC_DB_CONF_s point will contain entry   *
 *                                                                      *
 * OUTPUT      : 0  ==> success                                         *
 *               -1 ==> fail                                            *
 *                                                                      *
 * DESCRIPTION : Get struct from the whole line(str)                    *
 *----------------------------------------------------------------------*
 * Before      :  bobby 11-22-33-44-55-66 192.168.1.3 1 1 0 0 0         *
 *               ^                                                      *
 *               str                                                    *
 ***********************************************************************/
int PCDBExtractInfo(char *str, PC_DB_CONF_s *p_Conf)
{
    char separate = ' ';
    char *p=str, tmp[20];

    if(!p){
        return -1;
    }
    if ((p = PCDBGetStr(p, tmp, separate)) == NULL)
        return -1;
    strcpy(p_Conf->name, tmp);

    if ((p = PCDBGetStr(p, tmp, separate)) == NULL)
        return -1;
    extract_mac((const char *)tmp, p_Conf->mac_addr);

    if ((p = PCDBGetStr(p, tmp, separate)) == NULL)
        return -1;
    extract_ip(tmp, p_Conf->ip_addr);


    if ((p = PCDBGetStr(p, tmp, separate)) == NULL)
        return -1;
    p_Conf->reserve=atoi(tmp);

    if ((p = PCDBGetStr(p, tmp, separate)) == NULL)
        return -1;
    p_Conf->group=atoi(tmp);

    if ((p = PCDBGetStr(p, tmp, separate)) == NULL)
        return -1;
    p_Conf->mac_type=atoi(tmp);

    if ((p = PCDBGetStr(p, tmp, separate)) == NULL)
        return -1;
    p_Conf->access_type=atoi(tmp);

    if ((p = PCDBGetStr(p, tmp, separate)) == NULL)
        return -1;
    p_Conf->iface=atoi(tmp);

    return 0;
}

int ReadPCDBSettings(PC_DB_CONF *p_Conf, FILE *fh)
{
	char key[32], pc_info[100];
    short i;

    PC_DB_CONF_s *p_entry;

	if(!p_Conf || !fh)
		return PCDB_ReadConfErr;
    rewind(fh);

    /* Init pc db link first. */
    pc_db_link_init();

	strcpy(key, "number");
	PRO_GetChar(pc_db_sec, key, (unsigned char *)&p_Conf->number, fh);


    if (p_Conf->number > WK_MAX_NUM)
        return PCDB_ReadConfErr;

#if 0
    if (p_Conf->number == 0)     /* No PC in database, read successful */
    {
        p_Conf->p_database = NULL;
        return 0;
    }
#endif /* #if 0 */

    for (i=0, p_entry=p_Conf->p_database; i<p_Conf->number && p_entry; i++, p_entry = p_entry->next)
    {
	    sprintf(key,"%d", i);
	    PRO_GetStr(pc_db_sec, key, pc_info, 100, fh);

	    if (PCDBExtractInfo(pc_info, p_entry))
	        return -1;
    }

    if (i != p_Conf->number)
        return  PCDB_ReadConfErr;
	return 0;
}
int ReadPCDBConf(PC_DB_CONF *p_Conf) /* caller provide buf chain to contain info */
{
	FILE *fh;
	int ret=0;

	if(!p_Conf)
		return PCDB_ReadConfErr;

	fh=fopen(SYSCONF,"rt");
	if(!fh)
		return PCDB_ReadConfErr;

	ret=ReadPCDBSettings(p_Conf, fh);
	if(ret)
		goto err;
	fclose(fh);
	return 0;
err:
	fclose(fh);
	return ret;
}

int PCDBCheckItem(PC_DB_CONF_s *pConf)
{
	return 0;
}

/* Check conf for exist PC, return index if exist
    Note:  -1 means not exist, index from 0 to .. */
int PCNameExist(char *name, PC_DB_CONF *p_chain)
{
	short i;
    PC_DB_CONF_s *p_entry;

    for (p_entry=p_chain->p_database, i=0; p_entry; p_entry=p_entry->next, i++)
    {
#ifdef _BB_DEBUG_
            sprintf(bb_buf, "<%s><%d>, PCNameExist, i<%d>,name<%s>",
                __FILE__, __LINE__, i, p_entry->name);
            bb_trace(bb_buf);
#endif
        if (!strcmp(name, p_entry->name)){
            return i;
        }

    }

    return -1;
}

/************************************************************************
 * FUNCTION    : PCDBSaveItem()                                         *
 *                                                                      *
 * INPUT       : p_Conf  ==> PC_DB_CONF_s handle , contain entry data   *
 *               fh      ==> conf file handle                           *
 *               index_l ==> entry index in conf file (key)             *
 *                                                                      *
 * OUTPUT      : LAN_OK  ==> success                                    *
 *               Other   ==> fail                                       *
 *                                                                      *
 * DESCRIPTION : Save one PC entry to config file, key is index_l       *
 ***********************************************************************/
int PCDBSaveItem(PC_DB_CONF_s *p_Conf, FILE *fh, unsigned char index_l)
{
	multi_items	all_items[1];
	int ret=0;

    char mac_addr[18], ip_addr[16], pc_info[100];

    if(index_l > WK_MAX_NUM){
        return PCDB_SaveConfErr;
    }
	if(!fh){
		return PCDB_ReadConfErr;
	}
    rewind(fh);

    printf_haddr(mac_addr, p_Conf->mac_addr, '-');
    sprintf(ip_addr,"%u.%u.%u.%u", p_Conf->ip_addr[0], p_Conf->ip_addr[1], p_Conf->ip_addr[2], p_Conf->ip_addr[3]);

    sprintf(pc_info, "%s %s %s %d %d %d %d %d",
        p_Conf->name, mac_addr, ip_addr, p_Conf->reserve, p_Conf->group, p_Conf->mac_type, p_Conf->access_type, p_Conf->iface);


	sprintf(all_items[0].key,"%d", index_l);
	all_items[0].value=pc_info;


	ret=PRO_SetMultiItemStr(pc_db_sec, all_items, 1, SYSCONF);
	if(ret){
        fclose(fh);
		return PCDB_SaveConfErr;
	}
	return LAN_OK;

}
/************************************************************************
 * FUNCTION    : SavePCDBSetting()                                      *
 *                                                                      *
 * INPUT       : p_Conf  ==> PC_DB_CONF handle , contain PC database    *
 *               fh      ==> conf file handle                           *
 *                                                                      *
 * OUTPUT      : LAN_OK  ==> success                                    *
 *               Other   ==> fail                                       *
 *                                                                      *
 * DESCRIPTION : Save all PC database to config file , after this func  *
 *               config file will be SAME as p_Conf , no more , no less.*
 ***********************************************************************/

int SavePCDBSetting(PC_DB_CONF *p_Conf, FILE *fh)
{
    short i;
    PC_DB_CONF_s *p_entry = p_Conf->p_database;
	multi_items	all_items[1];
    char number[4], key[4];
	int ret=0;


	if(!fh)
		return PCDB_SaveConfErr;
    rewind(fh);

#if 0
    /* No need to save */
    if (p_Conf->number == 0)
        return 0;
#endif /* #if 0 */

    /* 1. Clear PC database entries. */
    for (i=0; i<WK_MAX_NUM-1; i++)
    {
#ifdef _BB_DEBUG_
            sprintf(bb_buf, "<%s><%d>, i<%d>,", __FILE__, __LINE__, i);
            bb_trace(bb_buf);
#endif
        sprintf(key, "%d", i);
        PRO_DelKey(pc_db_sec, key, SYSCONF);
    }

    /* 2. Set number to file. */
    sprintf(all_items[0].key, "number");
	sprintf(number,"%d", p_Conf->number);
	all_items[0].value=number;

	ret=PRO_SetMultiItemStr(pc_db_sec, all_items, 1, SYSCONF);
	if(ret){
        fclose(fh);
		return PCDB_SaveConfErr;
	}

#ifdef _BB_DEBUG_
            sprintf(bb_buf, "<%s><%d>, p_Conf->number<%d>", __FILE__, __LINE__, p_Conf->number);
            bb_trace(bb_buf);
#endif

    /* 3. Set all PC entries to file. */
    for (i=0; i<p_Conf->number && p_entry; i++, p_entry = p_entry->next)
    {
        if (PCDBSaveItem(p_entry, fh, i))
            return PCDB_SaveConfErr;
    }

    return LAN_OK;
}

int SavePCDBConf(PC_DB_CONF *p_Conf) /* caller provide buf chain to contain info */
{
	FILE *fh;
	int ret=0;

	if(!p_Conf)
		return PCDB_SaveConfErr;

    fh=fopen(SYSCONF,"rt");
	if(!fh)
		return PCDB_SaveConfErr;

	ret=SavePCDBSetting(p_Conf, fh);
	if(ret)
		goto err;
	fclose(fh);
   return 0;
err:
	fclose(fh);
	return ret;
}

/****************************************************************************
 * FUNCTION    : PCDBAdd()                                                  *
 *                                                                          *
 * INPUT       : p_chain  ==> PC_DB_CONF handle , contain PC database       *
 *               p_add    ==> PC_DB_CONF_s handle                           *
 *                                                                          *
 * OUTPUT      : LAN_OK  ==> success                                        *
 *               Other   ==> fail                                           *
 *                                                                          *
 * DESCRIPTION : Add one entry to config file                               *
 *--------------------------------------------------------------------------*
 * (1)  +-------+    +-------+    +-------+   +-------+      +-------+      *
 *      |   0   |--> |   1   |--> |   2   |-->|  ""   |..... | MAX-1 |-->0  *
 *      +-------+    +-------+    +-------+   +-------+      +-------+      *
 *                                                                          *
 * (2)  +-------+    +-------+    +-------+   +-------+      +-------+      *
 *      |   0   |--> |   1   |--> |   2   |-+ |  ""   |..... | MAX-1 |-->0  *
 *      +-------+    +-------+    +-------+ | +-------+      +-------+      *
 *                                          |                               *
 *                                          |  +-------+                    *
 *                                          +->|   3   |--+                 *
 *                                             +-------+  |                 *
 *                                                  ^     v                 *
 *                                                p_add  NULL               *
 * (3)                                                                      *
 *                                                                          *
 *    Write p_chain to config file                                          *
 *                                                                          *
 ***************************************************************************/
int PCDBAdd(PC_DB_CONF *p_chain, PC_DB_CONF_s *p_add)
{
    PC_DB_CONF_s *p_entry;
  	FILE *fh;

    if (PCDBCheckItem(p_add))
        return PCDB_AddConfErr;

    if (p_chain->number>=WK_MAX_NUM)
        return PCDB_AddConfErr;

 	fh=fopen(SYSCONF,"rt");
	if(!fh)
		return PCDB_AddConfErr;

    if(PCNameExist(p_add->name, p_chain)>=0){
    		fclose(fh);
		return PCDB_AddConfErr;
	}

    p_chain->number++;
    if (p_chain->number == 1)
    {
        p_chain->p_database = p_add;
        p_add->next = NULL;
    }
    else
    {
        for (p_entry=p_chain->p_database; (p_entry->next) && (strcmp(p_entry->next->name,"")); p_entry=p_entry->next)
        {
#ifdef _BB_DEBUG_
            sprintf(bb_buf, "<%s><%d>, name<%s>", __FILE__, __LINE__, p_entry->name);
            bb_trace(bb_buf);
#endif
        }
        p_entry->next=p_add;
        p_add->next=NULL;
    }
#ifdef _BB_DEBUG_
            sprintf(bb_buf, "<%s><%d>, name<%s>", __FILE__, __LINE__, p_entry->next->name);
            bb_trace(bb_buf);
#endif
    if (SavePCDBSetting(p_chain, fh)){
    		fclose(fh);
        return PCDB_AddConfErr;
	}
    fclose(fh);
    return LAN_OK;
}

/****************************************************************************
 * FUNCTION    : PCDBDel()                                                  *
 *                                                                          *
 * INPUT       : p_chain  ==> PC_DB_CONF handle , contain PC database       *
 *               name     ==> pc name (ex:Argon)                            *
 *                                                                          *
 * OUTPUT      : LAN_OK  ==> success                                        *
 *               Other   ==> fail                                           *
 *                                                                          *
 * DESCRIPTION : Add one entry to config file                               *
 *--------------------------------------------------------------------------*
 * (1)          +-------+    +-------+    +-------+         +-------+       *
 *  p_chain --> |   0   |--> |"Argon"|--> |   2   |-->..... | MAX-1 |-->0   *
 *              +-------+    +-------+    +-------+         +-------+       *
 *                                                                          *
 * (2)          +-------+    +-------+    +-------+         +-------+       *
 *  p_chain --> |   0   |--+ |"Argon"|--> |   2   |--> .....| MAX-1 |-->0   *
 *              +-------+  | +-------+    +-------+         +-------+       *
 *                         |                 ^                              *
 *                         |                 |                              *
 *                         +-----------------+                              *
 *                                                                          *
 * (3)                                                                      *
 *                                                                          *
 *    Write p_chain to config file                                          *
 *                                                                          *
 ***************************************************************************/
int PCDBDel(PC_DB_CONF *p_chain, char *name)
{
  	FILE *fh;
    char i, j;
    PC_DB_CONF_s *p_entry=p_chain->p_database;

    if((i=PCNameExist(name, p_chain)) < 0)
		return PCDB_DelConfErr;

	fh=fopen(SYSCONF,"rt");
	if(!fh)
		return PCDB_DelConfErr;

    p_chain->number--;
//    debug("%s:%s()[%d]: i is %s chain number is %d.",__FILE__,__FUNCTION__,__LINE__,i,p_chain->number);

#ifdef _BB_DEBUG_
            sprintf(bb_buf, "<%s><%d>, i<%d>,", __FILE__, __LINE__, i);
            bb_trace(bb_buf);
#endif
    if (i==0)
    {
        p_chain->p_database=p_chain->p_database->next;
    }
    else
    {

        for (j=0; j<i-1; j++, p_entry=p_entry->next);
        p_entry->next = p_entry->next->next;
    }
    if (SavePCDBSetting(p_chain, fh)){
    	fclose(fh);
        return PCDB_DelConfErr;
	}
    fclose(fh);
    return LAN_OK;
}

/****************************************************************************
 * FUNCTION    : PCDBUpdate()                                               *
 *                                                                          *
 * INPUT       : p_chain  ==> PC_DB_CONF handle , contain PC database       *
 *               p_update ==> PC_DB_CONF_s handle                           *
 *                                                                          *
 * OUTPUT      : LAN_OK  ==> success                                        *
 *               Other   ==> fail                                           *
 *                                                                          *
 * DESCRIPTION : Add one entry to config file                               *
 *--------------------------------------------------------------------------*
 * (1)          +-------+    +-------+    +-------+         +-------+       *
 *  p_chain --> |   0   |--> |"Argon"|--> |   2   |-->..... | MAX-1 |-->0   *
 *              +-------+    +-------+    +-------+         +-------+       *
 *              +-------+                                                   *
 * p_update --> |"Argon"|                                                   *
 *              +-------+                                                   *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 * (2)          +-------+    +-------+    +-------+         +-------+       *
 *  p_chain --> |   0   |--> |"Argon"|--> |   2   |--> .....| MAX-1 |-->0   *
 *              +-------+    +-------+    +-------+         +-------+       *
 *                                                                          *
 * (3)                                                                      *
 *                                                                          *
 *    Write p_chain to config file                                          *
 *                                                                          *
 ***************************************************************************/

int PCDBUpdate(PC_DB_CONF *p_chain, PC_DB_CONF_s *p_update)
{
  	FILE *fh;
    char i, j;
    PC_DB_CONF_s *p_entry;

    if (PCDBCheckItem(p_update))
        return PCDB_SaveConfErr;

    if((i=PCNameExist(p_update->name, p_chain)) < 0)
		return PCDB_SaveConfErr;

	fh=fopen(SYSCONF,"rt");
	if(!fh)
		return PCDB_SaveConfErr;

    for (j=0, p_entry=p_chain->p_database; j<i; j++, p_entry=p_entry->next);
    p_update->next = p_entry->next;
    memcpy(p_entry, p_update, sizeof(PC_DB_CONF_s));

    if (SavePCDBSetting(p_chain, fh)){
    	fclose(fh);
        return PCDB_SaveConfErr;
    }
        fclose(fh);
    return 0;
}


int PCDBGetIndexByName(char *name)
{
    return (PCNameExist(name,  &pc_db_conf));
}

int GetIndexByNameInWholePC(char *name){
    int pcIndex = PCNameExist(name,  &pc_db_conf);

    if(pcIndex < 0){
        /* Do not in PC_DB , search dynamic pc list .*/
    	FILE *fp;
    	int i = pc_db_conf.number ;
    	char file[255] = "/var/udhcpd.leases";
    	struct lease_t lease;

    	if (!(fp = fopen(file, "r"))) {
    		return -1;
    	}

    	while ( fread(&lease, sizeof(lease), 1, fp) ) {

    		if( !strcmp(name,lease.hostname) ){
    		    fclose(fp);
    		    return i;
    		}
    		else{
    		    i++;
    		}
    	}

    	/* Can not find in dynamic pc list ! */
    	fclose(fp);

    	return -1;

    }
    else{
        return pcIndex;
    }
}


char *PCDBGetNameByIndex(unsigned char index_l)
{
    PC_DB_CONF_s *p_entry;
    short i;

    for (i=0, p_entry=pc_db_conf.p_database; i<index_l && p_entry; i++, p_entry=p_entry->next);
    return (i==index_l)?p_entry->name:NULL;
}
int PCDBGetIndexByIP(unsigned char *ip)
{
    PC_DB_CONF_s *p_entry;
    int i;
    for (i=0, p_entry=pc_db_conf.p_database; i<pc_db_conf.number && p_entry; i++, p_entry=p_entry->next)
    {
        if (!memcmp(p_entry->ip_addr, ip, 4))
            return i;
    }
    return -1;
}

int GetIndexByIPInWholePC(unsigned char *ip){
    int pcIndex = PCDBGetIndexByIP(ip);
    if(pcIndex < 0){
        /* Do not in PC_DB , search dynamic pc list .*/
    	FILE *fp;
    	int i = pc_db_conf.number ;
    	char file[255] = "/var/udhcpd.leases";
    	struct lease_t lease;

    	if (!(fp = fopen(file, "r"))) {
    		return -1;
    	}

    	while ( fread(&lease, sizeof(lease), 1, fp) ) {
			if( PCDBGetIndexByIP((unsigned char *)&lease.yiaddr) >= 0){
                continue;
            }

    		if( !memcmp(ip, (unsigned char *)&lease.yiaddr,4) ){
    		    fclose(fp);
    		    return i;
    		}
    		else{
    		    i++;
    		}
    	}

    	/* Can not find in dynamic pc list ! */
    	fclose(fp);

    	return -1;

    }
    else{
        return pcIndex;
    }
}

unsigned char *PCDBGetIPByIndex(unsigned char index_l)
{
    PC_DB_CONF_s *p_entry;
    short i;

    for (i=0, p_entry=pc_db_conf.p_database; i<index_l && p_entry; i++, p_entry=p_entry->next);
    return (i==index_l)?p_entry->ip_addr:NULL;
}

unsigned char *GetIPByIndexInDynPC(int index_l){
    FILE *fp;
	int i = 0 ;
	char file[255] = "/var/udhcpd.leases";
	struct lease_t lease;
	static struct in_addr addr;

	if (!(fp = fopen(file, "r"))) {
		return NULL;
	}

	while ( fread(&lease, sizeof(lease), 1, fp) ) {
        if( i != index_l){
            continue;
        }

        addr.s_addr = lease.yiaddr;
        fclose(fp);
        return (unsigned char *)&addr.s_addr;
	}
	fclose(fp);
	return NULL;
}

unsigned char *GetIPByIndexInWholePC(int index_l){

    if(index_l > pc_db_conf.number - 1){
        /* Not in PC_DB. */
        return GetIPByIndexInDynPC(index_l - pc_db_conf.number);
    }
    else{
        return PCDBGetIPByIndex(index_l);
    }
}


unsigned char *PCDBGetIPByName(char *name)
{
    unsigned char index_l;
    index_l = PCNameExist(name,  &pc_db_conf);
    return PCDBGetIPByIndex(index_l);
}

unsigned char *GetIPByNameInWholePC(char *name){
    int index_l;

    index_l = GetIndexByNameInWholePC(name);

    return GetIPByIndexInWholePC(index_l);
}


char *PCDBGetNameByIP(unsigned char *ip)
{
    int index_l;
    index_l = PCDBGetIndexByIP(ip);

    return PCDBGetNameByIndex(index_l);
}

char *GetNameByIPInWholePC(unsigned char *ip){

    char *pcName = PCDBGetNameByIP(ip);
    if( !pcName ){
        /* Do not in PC_DB , search dynamic pc list .*/
    	FILE *fp;
    	int i = pc_db_conf.number ;
    	char file[255] = "/var/udhcpd.leases";
    	static struct lease_t lease;


    	if ((!(fp = fopen(file, "r")))) {
    		return NULL;
    	}

    	while ( fread(&lease, sizeof(lease), 1, fp) ) {

    		if( !memcmp(ip, (unsigned char *)&lease.yiaddr,4) ){
    		    fclose(fp);
    		    return lease.hostname;
    		}
    		else{
    		    i++;
    		}
    	}

    	/* Can not find in dynamic pc list ! */
    	fclose(fp);

    	return NULL;

    }
    else{
        return pcName;
    }
}

PC_DB_CONF_s *PCDBGetEntry(char *name)
{
    short i;
    PC_DB_CONF_s *p_entry;
    char index_l;

    if ((index_l = PCDBGetIndexByName(name))<0)
        return NULL;
#ifdef _BB_DEBUG_
            sprintf(bb_buf, "<%s><%d>, PCDBGetEntry, index_l<%d>", __FILE__, __LINE__, index_l);
            bb_trace(bb_buf);
#endif
    for (i=0, p_entry=pc_db_conf.p_database; i<index_l && p_entry; i++, p_entry=p_entry->next);
#ifdef _BB_DEBUG_
            sprintf(bb_buf, "<%s><%d>, PCDBGetEntry, i<%d>", __FILE__, __LINE__, i);
            bb_trace(bb_buf);
#endif

    return (i==index_l)?p_entry:NULL;
}


/* *******************************************************************
 * this funcition is called by cgi ,
 *   obuf : out buf for cgi,
 *   type : 0 for pc_db, 1 for pc_dba 2 for vs and other list pc page
 * ****************************************************************** */
int GetDynamicPCInfo (char *obuf, char type, char *name) {
	FILE *fp;
	int i;
	char file[255] = "/var/udhcpd.leases";
	struct lease_t lease;
	struct in_addr addr;
	char hostname[16];
	unsigned char mac_addr[18];
    short len=0, tmplen=0;

    *obuf = 0;
	if (!(fp = fopen(file, "r"))) {
//        fprintf(stderr,"can not open %s.",file);
		//perror("could not open input file");
		return 0;
	}

	/*     "00:00:00:00:00:00 255.255.255.255 Wed Jun 30 21:49:08 1993" */
	while (fread(&lease, sizeof(lease), 1, fp)) {

        tmplen = 0;
		for (i = 0; i < 6; i++) {
			tmplen += sprintf(mac_addr+tmplen, "%02x", lease.chaddr[i]);
			if (i != 5)
			    tmplen += sprintf(mac_addr+tmplen, "-");
		}
		memcpy(hostname,lease.hostname,16);

//		fprintf(stderr,"host name is %s.",hostname);
		if (PCDBGetIndexByName(hostname)>=0)
		    continue;

//		fprintf(stderr,"will input buffer .");

		addr.s_addr = lease.yiaddr;
		if(!name || strcmp(hostname,name) ){
            len += sprintf(obuf+len, "%s", option_s);
        }
        else{
            len += sprintf(obuf+len, "%s", option_sl);
        }
        len += sprintf(obuf+len, "%s", hostname);
        len += sprintf(obuf+len, " (%s) ", inet_ntoa(addr));
        if (1 == type){
            len += sprintf(obuf+len, " (%s) ", mac_addr);
        }
        if( 2 != type ){
            len += sprintf(obuf+len, " (%s) ", str_pctype_db(0));
            len += sprintf(obuf+len, " (%s) ", str_pcreserve_db(0));
        }
        len += sprintf(obuf+len, "%s", option_e);


	}
	fclose(fp);

	return 0;
}

int get_pc_list(char **buf1){
	unsigned char i;
	char *none="None in PC Database";
	short len=0;
  	PC_DB_CONF *p_pcdb_conf=&pc_db_conf;
    PC_DB_CONF_s *p_entry;



	*buf1=(char *)malloc(255*(WK_NAME_LEN+2+16));

	if(!buf1){
	    return 0; /* Fail to allocate memory. */
	}

	memset( *buf1,0,255*(WK_NAME_LEN+2+16+/*sizeof(option_s)+sizeof(option_e)*/20) );

	/* Fix IP entries */
	len += sprintf(*buf1+len, "%s%s%s", option_s,none,option_e);

    for (i=0, p_entry=p_pcdb_conf->p_database; i<p_pcdb_conf->number && p_entry; i++){
        len += sprintf(*buf1+len, "%s", option_s);

        len += sprintf(*buf1+len, "%s", p_entry->name);
        len += sprintf(*buf1+len, " (%u.%u.%u.%u) ", p_entry->ip_addr[0], p_entry->ip_addr[1], p_entry->ip_addr[2], p_entry->ip_addr[3]);

        len += sprintf(*buf1+len, "%s", option_e);
        p_entry = p_entry->next;
    }

	/* Dynamic IP entries */

    GetDynamicPCInfo(*buf1+len, 2,NULL);


	return 0;
}

struct lease_t *GetDynPCInfo(char *name){
	FILE *fp;
	char file[255] = "/var/udhcpd.leases";
	static struct lease_t lease;

	if (!(fp = fopen(file, "r"))) {
		return 0;
	}

	while (fread(&lease, sizeof(lease), 1, fp)) {

		if( !strcmp(name,lease.hostname) ){
		    fclose(fp);
		    return &lease;
		}
	}
	fclose(fp);

	return NULL;
}

/*delete from fix at first. then dynamic pcs.*/
int delPCInWholePC(PC_DB_CONF *p_chain, char *name, unsigned int ip){
    FILE *fp;

    if(PCDBDel(p_chain, name) == 0){ /* delete in fix pc ok. */
        return 0;
    }

    /* Make the mac file. */
    fp = fopen(DEL_IP_FILE,"w");
    if(!fp){
        return -1;
    }

    fwrite(&ip, sizeof(unsigned int), 1, fp);
    fclose(fp);

    /* send signal to it. */
    system("/sbin/del-dyn-ip > /dev/null");
    return 0;
}

#define BUF_SIZE	100
char *GetMACByIP(unsigned char *ip){
	FILE *fp = fopen("/proc/net/arp", "r");
	static char mac_str[18];
	char buff[BUF_SIZE];
	char ip_str[16];
	int ip_len;

	if(!fp){
		return NULL;
	}
	sprintf(ip_str,"%u.%u.%u.%u", ip[0],ip[1],ip[2],ip[3]);
	ip_len = strlen(ip_str);

	while( fgets(buff, BUF_SIZE-1, fp) ){
		if( !memcmp(buff, ip_str, ip_len) ){ /* find the ip's entry. */
			fclose(fp);
			sscanf(buff,"%*s %*s %*s %s", mac_str);
			return mac_str;
		}
	}
	fclose(fp);
	return NULL;
}


#define UDHCPD_CONF	"/var/udhcpd.conf"

int StartLAN(int first)
{
	int ret=0;
	DWORD broadcast;
	char cmd[128]={0},lan_ip[16]={0},lan_mask[16]={0},lan_broad[16]={0};
	BYTE *p=NULL;
	LAN_CONF conf;

	ret=ReadLanSettings(&conf);
	if(ret)
		return ret;
		
#ifdef _LAN_BRIDGE_
	if(first){
	sprintf(cmd,"/sbin/brctl addbr %s >/dev/null 2>&1",BRG_LAN);
	system(cmd);
	sprintf(cmd,"/sbin/brctl stp %s off >/dev/null 2>&1",BRG_LAN);
	system(cmd);
	sprintf(cmd,"/sbin/ifconfig  %s 0.0.0.0 up promisc >/dev/null 2>&1",ETH_LAN);
	system(cmd);
	sprintf(cmd,"/sbin/ifconfig  %s 0.0.0.0 up promisc >/dev/null 2>&1",ISL_LAN);
	system(cmd);	
	}	
#endif

	sprintf(cmd,"/sbin/ifconfig %s 0.0.0.0 down >/dev/null 2>&1",LAN_ITF);
	system(cmd);

	sprintf(cmd,"/sbin/ifconfig %s 0.0.0.0 down >/dev/null 2>&1",LAN_ITF2);
	system(cmd);

    p = (BYTE *)&conf.LanIPAddr;
    sprintf(lan_ip,"%u.%u.%u.%u",*(p),*(p+1),*(p+2),*(p+3));

    p = (BYTE *)&conf.NetMaskIPAddr;
    sprintf(lan_mask,"%u.%u.%u.%u",*(p),*(p+1),*(p+2),*(p+3));

    broadcast = conf.LanIPAddr | (~conf.NetMaskIPAddr);
    p = (BYTE *)&broadcast;
    sprintf(lan_broad,"%u.%u.%u.%u",*(p),*(p+1),*(p+2),*(p+3));

    /*First LAN interface*/
    if(strcmp(lan_ip, "0.0.0.0") && strcmp(lan_mask, "0.0.0.0")){
		sprintf(cmd,"/sbin/ifconfig %s %s netmask %s broadcast %s >/dev/null 2>&1",LAN_ITF,lan_ip,lan_mask,lan_broad);
		system(cmd);
	}

	
	sprintf(cmd,"/sbin/route add -net 239.0.0.0 netmask 255.0.0.0 %s >/dev/null 2>&1",LAN_ITF);
	system(cmd);


    p = (BYTE *)&conf.LanIPAddr2;
    sprintf(lan_ip,"%u.%u.%u.%u",*(p),*(p+1),*(p+2),*(p+3));

    p = (BYTE *)&conf.NetMaskIPAddr2;
    sprintf(lan_mask,"%u.%u.%u.%u",*(p),*(p+1),*(p+2),*(p+3));

	/*Second LAN interface*/
	if(strcmp(lan_ip, "0.0.0.0") && strcmp(lan_mask, "0.0.0.0")){
		sprintf(cmd,"/sbin/ifconfig %s %s netmask %s >/dev/null 2>&1",LAN_ITF2,lan_ip,lan_mask);
		system(cmd);
	}


#ifdef _LAN_BRIDGE_
	if(first){
	sprintf(cmd,"/sbin/brctl addif %s %s >/dev/null 2>&1",BRG_LAN,ETH_LAN);
	system(cmd);
	sprintf(cmd,"/sbin/brctl addif %s %s >/dev/null 2>&1",BRG_LAN,ISL_LAN);
	system(cmd);
	}
#endif
	if(!first)
	system("/bin/killall udhcpd >/dev/null 2>&1");
	remove(UDHCPD_CONF);
	if(conf.DhcpEnable){
		mkdhcpdconf_main();
		system("/sbin/udhcpd >/dev/null 2>&1");
	}
	StartUPnP();
	return 0;
}


int StopLAN(void)
{
	char cmd[128]={0};
	StopUPnP();
#ifdef _LAN_BRIDGE_
	sprintf(cmd,"/sbin/ifconfig  %s 0.0.0.0 down >/dev/null 2>&1",ETH_LAN);
	system(cmd);
	sprintf(cmd,"/sbin/ifconfig  %s 0.0.0.0 down>/dev/null 2>&1",ISL_LAN);
	system(cmd);	
#endif

	sprintf(cmd,"/sbin/ifconfig %s 0.0.0.0 down>/dev/null 2>&1",LAN_ITF);
	system(cmd);

	sprintf(cmd,"/sbin/route add -net 239.0.0.0 netmask 255.0.0.0 %s >/dev/null 2>&1",LAN_ITF);
	system(cmd);

#ifdef _LAN_BRIDGE_
	sprintf(cmd,"/sbin/brctl delbr %s >/dev/null 2>&1",BRG_LAN);
	system(cmd);
#endif

	system("/bin/killall udhcpd >/dev/null 2>&1");
	return 0;
}


