#ifndef __LAN_H__
#define __LAN_H__

#include "portable.h"
#include "h_const.h"

typedef struct{
   DWORD   LanIPAddr;
   DWORD   NetMaskIPAddr;
    DWORD   LanIPAddr2;         /* For Unnumbered IP. */
    DWORD   NetMaskIPAddr2;     /* For Unnumbered IP. */
   BYTE    DhcpEnable;
   DWORD   StartIp;
   DWORD   FinishIp;
} LAN_CONF;

extern LAN_CONF lan_conf;
extern int group_index;

#define DHCP_ENABLE  1
#define DHCP_DISABLE 0

#define	LAN_OK					0			//Succeed
#define	LAN_SaveConfErr		    -1			//Fail to save LAN settings.
#define	LAN_ReadConfErr		    -2			//Fail to read LAN settings.
#define PCDB_SaveConfErr    -3          //Fail to save PC database.
#define PCDB_ReadConfErr    -4          //Fail to read PC database.
#define PCDB_NameExistErr   -5          //When add, found dulplicate PC
#define PCDB_DelConfErr     -6          //del err
#define PCDB_AddConfErr     -7          //add err

#define LAN_IP_DEFAULT_HEX          0xc0a80101
#define LAN_MASK_DEFAULT_HEX        0xffffff00
#define LAN_START_IP_DEFAULT_HEX    0xc0a8010a
#define LAN_FINISH_IP_DEFAULT_HEX   0xc0a8013c
#define LAN_DHCPS_DEFAULT_HEX       1
#define LAN_IP2_DEFAULT_HEX         0
#define LAN_MASK2_DEFAULT_HEX       0



#define PCDB_ENTRY_MAX_LEN      100
enum{
    LAN_ITEM_LAN_IP,
    LAN_ITEM_LAN_MASK,
    LAN_ITEM_LAN_IP2,
    LAN_ITEM_LAN_MASK2,
    LAN_ITEM_DHCP,
    LAN_ITEM_STARTIP,
    LAN_ITEM_FINISHIP,
    LAN_ITEM_NUMBER
};
#define WK_NAME_LEN     17
#define WK_MAX_NUM      10              //10 for temp use

enum{
    PCDB_ITEM_MAC,
    PCDB_ITEM_NAME,
    PCDB_ITEM_IP,
    PCDB_ITEM_RESERVE,
    PCDB_ITEM_GROUP,
    PCDB_ITEM_MAC_TYPE,
    PCDB_ITEM_ACCESS_TYPE,
    PCDB_ITEM_IFACE,
    PCDB_ITEM_NUMBER
};
typedef struct _PC_DB_CONF_s{
    unsigned char   mac_addr[6];
    char            name[WK_NAME_LEN];
    unsigned char   ip_addr[4];
    char            reserve;     // DHCP server reserve it 0-->DHCP client 1-->fixed ip 2-->DHCP server reserved 3-->fix name and mac
    unsigned char   group;          /* group belong */
    unsigned char	mac_type;//0-->auto 1-->fixed
    char	        access_type;
    char	        iface;
    struct _PC_DB_CONF_s    *next;
}PC_DB_CONF_s;

typedef struct {
    unsigned char   number;
    PC_DB_CONF_s    *p_database;
}PC_DB_CONF;

#define DEL_IP_FILE    "/var/udhcpd.delip"

/* ********************************************************************
    The following part is used for read dynamic IP pc allocated from
    server, read "udhcpd.lease" under /var
********************************************************************* */
struct lease_t {
	unsigned char chaddr[16];
	unsigned long yiaddr;
	unsigned long expires;
	char hostname[16];
};

extern PC_DB_CONF_s pc_db[WK_MAX_NUM];
extern PC_DB_CONF pc_db_conf;

int ReadLanSettings(LAN_CONF *pConf);
int SaveLanSettings(LAN_CONF *pConf);
int RestoreLANSettings(LAN_CONF *pConf);

void pc_db_link_init(void);
int ReadPCDBConf(PC_DB_CONF *pConf);
int SavePCDBConf(PC_DB_CONF *p_Conf);
int PCDBAdd(PC_DB_CONF *p_chain, PC_DB_CONF_s *p_add);
int PCDBDel(PC_DB_CONF *p_chain, char *name);
int PCDBUpdate(PC_DB_CONF *p_chain, PC_DB_CONF_s *p_update);
PC_DB_CONF_s *PCDBGetEntry(char *name);

int AddPCDBConf(PC_DB_CONF_s *p_Conf);
int DeletePCDBConf(char *name);
int UpdatePCDBConf(PC_DB_CONF_s *p_Conf);

int PCDBGetIndexByName(char *name);
int GetIndexByNameInWholePC(char *name);
char *PCDBGetNameByIndex(unsigned char index_l);
int PCDBGetIndexByIP(unsigned char *ip);
unsigned char *PCDBGetIPByIndex(unsigned char index_l);
unsigned char *PCDBGetIPByName(char *name);
char *PCDBGetNameByIP(unsigned char *ip);
int GetDynamicPCInfo (char *obuf, char type, char *name);
int GetIndexByIPInWholePC(unsigned char *ip);
char *GetNameByIPInWholePC(unsigned char *ip);
char *GetMACByIP(unsigned char *ip);

int get_pc_list(char **buf1);

unsigned char *GetIPByNameInWholePC(char *name);
struct lease_t *GetDynPCInfo(char *name);

int delPCInWholePC(PC_DB_CONF *p_chain, char *name, unsigned int ip);

int StartLAN(int first);
int StopLAN(void);



int openRawSocket (char *IfName, int *s, u_short type);
int peekfd(int s,time_t tv_usec);
int arpCheck(char *IfName,unsigned char *RequestHwAddr,  unsigned char *RequestProAddr);
#endif /* __LAN_H__ */
