#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "PRO_file.h"
#include "PRO_GetStr.h"
#include "PRO_SetStr.h"
#include "PRO_GetInt.h"
#include "PRO_SetInt.h"
#include "routing.h"

#define SYSCONF "/var/system.conf"

int GetRoute(char ***route,int *routenum)
{
	FILE *fh;
	char buf[128]={0};
	char **list=NULL;
	char *p=NULL;
	int num=0;
	
	fh=fopen(SYSCONF,"r");
	if(!fh)
		return ReadRoutingConfErr;	
	list=(char **)malloc(sizeof(list)*MAX_ROUTE_NUM);
	memset(list,0,sizeof(list)*MAX_ROUTE_NUM);
	while(fgets(buf,sizeof(buf),fh)){
		if(num>MAX_ROUTE_NUM)
				break;
		if(strstr(buf,RTLIST_KEY)==buf){
			*(list+num)=malloc(MAX_ROUTE_LEN);
			p=strchr(buf,'\n');
			if(p)
				*p=0;
			strcpy(*(list+num),buf+strlen(RTLIST_KEY));
			num++;
		}
	}
	*(list+num)=NULL;
	*route=list;
	*routenum=num;
	fclose(fh);
	
	return 0;
}

void FreeRtList(rt_table *table)
{
	rt_table *p;
	
	p=table;
	while(p){
		table=table->next;
		free(p);
		p=table;
	}
}	

int GetRtList(rt_table **table,int *routenum)
{
	char **buf=NULL,*list;
	int ccode=0,num=0;
	rt_table *p=NULL,*q=NULL;
	
	if(!table)
		return ReadRoutingConfErr;
	*table=NULL;
	
	ccode=GetRoute(&buf,&num);
	if(ccode)
		return GetRtListErr;
	if(num==0)
		return 0;
	*routenum=num;
	while(*buf){
		p=(rt_table *)malloc(sizeof(rt_table));
		if(!p){
			ccode=MallocMemErr;
			FreeRtList(*table);
			return MallocMemErr;
		}
		p->next=NULL;
		list=strrchr(*buf,',');
		if(!list||list==*buf)
			return ReadRoutingConfErr;
		p->metric=atoi(list+1);
		*list=0;
		list=strrchr(*buf,',');
		if(!list||list==*buf)
			return ReadRoutingConfErr;
		strncpy(p->gateway,list+1,MaxIPLen);
		*list=0;
		list=strrchr(*buf,',');
		if(!list||list==*buf)
			return ReadRoutingConfErr;
		strncpy(p->netmask,list+1,MaxIPLen);
		*list=0;
		strncpy(p->ipaddr,*buf,MaxIPLen);
		if(*table==NULL){
			*table=q=p;
		}
		else{
			q->next=p;
			q=p;
		}
		buf++;
	}

	return ccode;
}

int ReadRipEnable(int *rip_enable)
{
	FILE *fp;
	
	fp=fopen(SYSCONF,"r");
	if(!fp)
		return ReadRoutingConfErr;
	PRO_GetInt(Route_Section,RIP_ENABLE,rip_enable,fp);
	fclose(fp);
	return 0;
}
	
int SaveRipEnable(int rip_enable)
{
	PRO_SetInt(Route_Section,"rip_enable",rip_enable,SYSCONF);

	system("/bin/killall routed >/dev/null 2>&1");

	if(rip_enable==1){
		system("/sbin/routed & >/dev/null 2>&1");
	}
	return 0;
}

int AddRtList(char *addr,char *mask,char *gateway,int metric,int n)
{
	FILE *fh,*fp;
	int fd,ret,ccode=0,i=1,mt=0;
	char *p=NULL,*ip1=NULL,*ip2=NULL,*ip3=NULL,*route=NULL;
	char buf[128]={0},tmp[128]={0},buf1[128]={0};
	char tempf[]="/var/tempfile.XXXXXX";
	
	if(!addr || !mask || !gateway || !metric)
		return AddRtListErr;
	
	sprintf(buf1,"/sbin/route add -net %s netmask %s gw %s metric %d >/dev/null 2>&1",addr,mask,gateway,metric);
	ret=system(buf1);
	if(ret){
		return AddRtListErr;
	}
	route=malloc(MAX_ROUTE_LEN);
	if (!route){
		return AddRtListErr;
	}	
	memset(route,0,MAX_ROUTE_LEN);
	sprintf(route,"%s,%s,%s,%d",addr,mask,gateway,metric);
	fd=mkstemp(tempf);
	if(fd==-1){
		ccode=AddRtListErr;
		goto err;
	}
	close(fd);
	fp=fopen(tempf,"w");
	if(!fp){
		ccode=AddRtListErr;
		goto err;
	}
	fh=fopen(SYSCONF,"r");
	if(!fh){
		fclose(fp);
		remove(tempf); 
		ccode=AddRtListErr;
		goto err;
	}	
	while(fgets(buf,sizeof(buf),fh)){
		if(strstr(buf,RIP_ENABLE)&&((n==0)||strstr(tmp,Route_Section))){
			fprintf(fp,"%s%s\n",RTLIST_KEY,route);
		}
		if(strstr(buf,RTLIST_KEY)==buf){
			strcpy(tmp,buf);
			p=strchr(tmp,'\n');
			if(p)
				*p=0;
			if(strcmp(tmp+strlen(RTLIST_KEY),route)==0)
				continue;
			if(i==n){
				ip1=malloc(MaxIPLen);
				if(!ip1){
					ccode=AddRtListErr;
					goto err;
				}
				ip2=malloc(MaxIPLen);
				if(!ip2){
					ccode=AddRtListErr;
					goto err;
				}
				ip3=malloc(MaxIPLen);
				if(!ip3){
					ccode=AddRtListErr;
					goto err;
				}
				p=strrchr(tmp+strlen(RTLIST_KEY),',');
				mt=atoi(p+1);
				*p=0;
				p=strrchr(tmp+strlen(RTLIST_KEY),',');
				strncpy(ip3,p+1,MaxIPLen);
				*p=0;
				p=strrchr(tmp+strlen(RTLIST_KEY),',');
				strncpy(ip2,p+1,MaxIPLen);
				*p=0;
				strncpy(ip1,tmp+strlen(RTLIST_KEY),MaxIPLen);
				fprintf(fp,"%s%s\n",RTLIST_KEY,route);
				i++;
				continue;
			}
			i++;
		}
		fputs(buf,fp);
	}
	fclose(fh);
	fclose(fp);
	rename(tempf,SYSCONF);
	chown(SYSCONF,99,99);
	if(ip1 && ip2 && ip3 && metric){
		sprintf(buf1,"/sbin/route del -net %s netmask %s gw %s metric %d  &> /dev/null",ip1,ip2,ip3,mt);
		system(buf1);
	}
	
err:

	if(route)
		free(route);
	if(ip1)
		free(ip1);
	if(ip2)
		free(ip2);
	if(ip3)
		free(ip3);
		
//	sprintf(buf1,"/sbin/route add -net %s netmask %s gw %s metric %d &> /dev/null",addr,mask,gateway,metric);
//	system(buf1);
	
	return ccode;
}
	
int DelRtList(char *addr,char *mask,char *gateway,int metric)
{
	FILE *fh,*fp;
	int fd;
	char *p=NULL;
	char route[128],buf[128]={0},tmp[128]={0},buf1[128];
	char tempf[]="/var/tempfile.XXXXXX";
	
	if(!addr || !mask || !gateway || !metric)
		return DelRtListErr;
	sprintf(route,"%s,%s,%s,%d",addr,mask,gateway,metric);
	fd=mkstemp(tempf);
	if(fd==-1)
		return DelRtListErr;
	close(fd);
	fp=fopen(tempf,"w");
	if(!fp)
		return DelRtListErr;
	fh=fopen(SYSCONF,"r");
	if(!fh){
		fclose(fp);
		remove(tempf); 
		return DelRtListErr;
	}	
	while(fgets(buf,sizeof(buf),fh)){
		if(strstr(buf,RTLIST_KEY)==buf){
			strcpy(tmp,buf);
			p=strchr(tmp,'\n');
			if(p)
				*p=0;
			if(!strcmp(tmp+strlen(RTLIST_KEY),route))
				continue;
		}
		fputs(buf,fp);
	}
	fclose(fh);
	fclose(fp);
	rename(tempf,SYSCONF);
	chown(SYSCONF,99,99);
	
	sprintf(buf1,"/sbin/route del -net %s netmask %s gw %s metric %d &> /dev/null",addr,mask,gateway,metric);
	system(buf1);
	
	return 0;
}

int GetAllRoute(rt_table **rttable,int *rtnum)
{
	FILE *fp;
	char buf[128]={0},metric[2];
	rt_table *p=NULL,*q=NULL;
	int ccode=0,num=0;

	*rttable=NULL;
	system("/sbin/route -n >/var/rt_list");
	fp=fopen(RT_LIST,"r");
	if(!fp)
		return -1;
	while(fgets(buf,sizeof(buf),fp)){
		if(strstr(buf,"Kernel")||strstr(buf,"Destination"))
			continue;
		p=(rt_table *)malloc(sizeof(rt_table));
		if(!p){
			ccode=MallocMemErr;
			FreeRtList(*rttable);
			return MallocMemErr;
		}
		memset(p,0,sizeof(rt_table));
		strncpy(p->ipaddr,buf,MaxIPLen-1);
		p->ipaddr[MaxIPLen-1]='\0';
		
		strncpy(p->gateway,buf+16,MaxIPLen-1);
		p->gateway[MaxIPLen-1]='\0';
		
		strncpy(p->netmask,buf+32,MaxIPLen-1);
		p->netmask[MaxIPLen-1]='\0';
		
		strncpy(metric,buf+54,2);
		metric[2]='\0';
		p->metric=atoi(metric);
		
		if(*rttable==NULL){
			*rttable=q=p;
		}
		else{
			q->next=p;
			q=p;
		}
		num++;
	}
	fclose(fp);
	remove(RT_LIST);
	*rtnum=num;
	
	return ccode;
}

int InitRoute(void)
{
	int num=0,i,ret;
	char buf[128];
	rt_table *sysroute=NULL;
	
	ret=GetRtList(&sysroute,&num);
	if(ret)
		return ret;
	if(num==0)
		return 0;
	for(i=0;i<num;i++){
		sprintf(buf,"/sbin/route add -net %s netmask %s gw %s metric %d >/dev/null 2>&1",sysroute->ipaddr,sysroute->netmask,sysroute->gateway,sysroute->metric);
		system(buf);
		sysroute=sysroute->next;
	}
	if(sysroute)
		FreeRtList(sysroute);
	return 0;
}
