#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <net/if.h>
#include <time.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "PRO_file.h"
#include "PRO_GetStr.h"
#include "PRO_GetInt.h"
#include "PRO_SetStr.h"
#include "PRO_SetMultiItemStr.h"

#include "flash.h"
#include "led.h"
#include "h_var.h"
#include "system.h"
#include "wan.h"
#include "dev_opt.h"
#include "lan.h"

SYSTEM_CONF system_conf;
IFACE_STATUS if_status;

extern char *crypt(const char *key, const char *setting);

/* From local_passwd.c (C) Regents of Univ. of California blah blah */
static unsigned char itoa64[] =         /* 0 ... 63 => ascii - 64 */
        "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

static void to64(register char *s, register long v, register int n) {
    while (--n >= 0) {
        *s++ = itoa64[v&0x3f];
        v >>= 6;
    }
}

char *crypt_password(const char *psw_org){
    char salt[3];
    char *cpw;

    (void) srandom( (int) time( (time_t*) 0 ) );
    to64( &salt[0], random(), 2 );
    cpw = crypt( psw_org, salt );
    return cpw;
}

int CheckPasswd(const char *pass,const char*crypt_pass){
    return strcmp( crypt( pass, crypt_pass ), crypt_pass );
}


static char system_sec[] = "System";
/*===================================================================*
 * FUNCTION    : ReadSystemSettings                                  *
 *                                                                   *
 * INPUT       : System conf point                                   *
 *                                                                   *
 * OUTPUT      : System_OK ==> Successful                            *
 *               Other  ==> Fail                                     *
 *                                                                   *
 * Description : Read System settings to pConf                       *
 *                                                                   *
 *===================================================================*/
int ReadSystemSettings(SYSTEM_CONF *pConf){
    char key[32];
    char connection[40];
    FILE *fh;

	if(!pConf)
		return System_ReadConfErr;

	fh=fopen(SYSCONF,"rt");

	if(!fh)
		return System_ReadConfErr;
	rewind(fh);

	sprintf(key,"UserName");
	PRO_GetStr(system_sec,key,pConf->UserName,SYS_USER_LEN,fh);

	sprintf(key,"Password");
	PRO_GetStr(system_sec,key,pConf->Password,SYS_CRYPT_LEN,fh);

	sprintf(key,"Connection");
	PRO_GetStr(system_sec,key,connection,sizeof(connection),fh);
	pConf->Connection = str_db_search(Connection_db,connection);

    sprintf(key,"CUIUserName");
    PRO_GetStr(system_sec,key,pConf->cuiUserName,CUI_USER_LEN,fh);

    sprintf(key,"CUIPassword");
    PRO_GetStr(system_sec,key,pConf->cuiPassword,CUI_PASS_LEN,fh);

	fclose(fh);

	return System_OK;
}

int CheckSystemSettings(SYSTEM_CONF *pConf){

	return System_OK;

}

int SaveSystemSettings(SYSTEM_CONF *pConf){

    multi_items	all_items[SYS_ITEM_NUMBER];
    int ret;

	ret=CheckSystemSettings(pConf);
	if(ret){
		return ret;
	}

	sprintf(all_items[SYS_ITEM_USERNAME].key,"UserName");
	all_items[SYS_ITEM_USERNAME].value=pConf->UserName;

	sprintf(all_items[SYS_ITEM_PASSWD].key,"Password");
	all_items[SYS_ITEM_PASSWD].value=pConf->Password;

	sprintf(all_items[SYS_ITEM_CONNECTION].key,"Connection");
	all_items[SYS_ITEM_CONNECTION].value=Connection_db[(BYTE)pConf->Connection];

	sprintf(all_items[SYS_ITEM_CUIUSERNAME].key,"CUIUserName");
	all_items[SYS_ITEM_CUIUSERNAME].value=pConf->cuiUserName;

	sprintf(all_items[SYS_ITEM_CUIPASSWD].key,"CUIPassword");
	all_items[SYS_ITEM_CUIPASSWD].value=pConf->cuiPassword;

    ret=PRO_SetMultiItemStr(system_sec,all_items,SYS_ITEM_NUMBER, SYSCONF);
	if(ret){
		return System_WriteConfErr;
	}
	return System_OK;
}

int ChangePasswd(SYSTEM_CONF *pConf){
    FILE *fp;

    fp = fopen(PASSWD_FILE,"w");

    if(!fp){
        return System_WriteConfErr;
    }

    fprintf(fp,"%s:%s",pConf->UserName,pConf->Password);
    fclose(fp);

    return System_OK;
}

/****************************************************************
 *  Function    :  MakeHttpdPasswd()                            *
 *                                                              *
 *  Input       :  void                                         *
 *                                                              *
 *  Output      :  System_ReadConfErr                           *
 *                 System_WriteConfErr                          *
 *                 System_OK                                    *
 *                                                              *
 *  Description :  This function will change thttpd's password  *
 *                 file (/htdocs/.htpasswd) according to config *
 *                 file (/var/system.conf)                      *
 ****************************************************************/
int MakeHttpdPasswd(void){
    SYSTEM_CONF *pConf = &system_conf;

    if(ReadSystemSettings(pConf) != System_OK){
        return System_ReadConfErr;
    }

    if(ChangePasswd(pConf) != System_OK){
        return System_WriteConfErr;
    }

    return System_OK;
}

/********************************************************************************
 *  GetIfaceStatus()        Get interface status and put the data to *pStatus   *
 *                                                                              *
 *  0                   ==>     Successed                                       *
 *  other               ==>     Fail                                            *
 *                                                                              *
 ********************************************************************************/
int GetIfaceStatus(char *if_name, IFACE_STATUS *pStatus){
    struct ifreq ifr;
    int sockd;
    struct sockaddr_in *mysockaddr;

    memset(pStatus,0,sizeof(IFACE_STATUS));
    sockd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockd == -1) { /* Open socket error. */
		return WAN_GetStatusErr;
	}
	/* Init iface name. */
	strcpy(ifr.ifr_name, if_name);

    /* Try to get HW address. */
    if(ioctl(sockd, SIOCGIFHWADDR, &ifr) < 0) {
    		close(sockd);
		return WAN_GetStatusErr;
	}
	memcpy(pStatus->mac,ifr.ifr_hwaddr.sa_data,6);

    /* Try to get IP address. */
    if(ioctl(sockd, SIOCGIFADDR, &ifr) < 0) {
    		close(sockd);
		return WAN_GetStatusErr;
	}
	mysockaddr = (struct sockaddr_in *)&(ifr.ifr_addr);
	pStatus->ip = mysockaddr->sin_addr;

    /* Try to get netmask. */
    if(ioctl(sockd, SIOCGIFNETMASK, &ifr) < 0) {
    		close(sockd);
		return WAN_GetStatusErr;
	}
	mysockaddr = (struct sockaddr_in *)&(ifr.ifr_netmask);
	pStatus->mask = mysockaddr->sin_addr;

#if 0
    /* Try to get gw. */
    if(ioctl(sockd, SIOCGIFDSTADDR, &ifr) < 0) {

		return WAN_GetStatusErr;
	}
	mysockaddr = (struct sockaddr_in *)&(ifr.ifr_addr);
	pStatus->gw = mysockaddr->sin_addr;
#else
{
    FILE *fp;
    char sys_cmd[128];
    #define ROUTE_RESULT    "/var/routeResult"

    sprintf(sys_cmd,"/sbin/route -n | grep \"^0.0.0.0\" > %s",ROUTE_RESULT);
    system(sys_cmd);

    fp = fopen(ROUTE_RESULT,"r");
    if(fp){

        char line[128];
        int i;
        while(fgets(line,sizeof(line),fp) != NULL){
            for(i=16;line[i] != ' ';i++);
            line[i] = '\0';
            inet_aton(&line[16],&pStatus->gw);
        }
        fclose(fp);
    }
    remove(ROUTE_RESULT);
}
#endif /* #if 0 */

    /* Try to get flag. */
    if(ioctl(sockd, SIOCGIFFLAGS, &ifr) < 0) {
    		close(sockd);
		return WAN_GetStatusErr;
	}
	pStatus->flag = ifr.ifr_flags;
    close(sockd);

    return WAN_GetStatusOK;
}
#if 0
/****************************************************************
 *  Function    :  connectPPPoE()                               *
 *                                                              *
 *  Input       :  interface number                             *
 *                  Note: only 0 ,not implement Multi-PPPoE now *
 *                                                              *
 *  Output      :  VOID                                         *
 ****************************************************************/
void connectPPPoE(int interface){
    char if_name[5];

    /* Check system connection type. */
    ReadSystemSettings(&system_conf);

    if( ( system_conf.Connection != CONNECTION_SINGLE_PPPOE )
     && ( system_conf.Connection != CONNECTION_MULTI_PPPOE)
     ){
        /* Not PPPoE , return. */
        return;
    }

    /* Get Iface name. */
    sprintf(if_name,"ppp%d",interface);

    /* Get Iface status. */
    GetIfaceStatus(if_name,&if_status);

    if(if_status.flag&IFF_UP){
        /* This interface already UP. */
        return ;
    }

    system("/usr/sbin/adsl-start >/dev/null 2>&1");
    return;
}


/****************************************************************
 *  Function    :  disconnectPPPoE()                            *
 *                                                              *
 *  Input       :  interface number                             *
 *                  Note: only 0 ,not implement Multi-PPPoE now *
 *                                                              *
 *  Output      :  VOID                                         *
 ****************************************************************/
void disconnectPPPoE(int interface){
    char if_name[5];

    /* Check system connection type. */
    ReadSystemSettings(&system_conf);

    if( ( system_conf.Connection != CONNECTION_SINGLE_PPPOE )
     && ( system_conf.Connection != CONNECTION_MULTI_PPPOE)
     ){
        /* Not PPPoE , return. */
        return;
    }

    /* Get Iface name. */
    sprintf(if_name,"ppp%d",interface);

    /* Get Iface status. */
    GetIfaceStatus(if_name,&if_status);

    if((if_status.flag&IFF_UP) ==0 ){
        /* This interface already DOWN. */
        return ;
    }

    system("/usr/sbin/adsl-stop >/dev/null 2>&1");
    return;
}
#endif

/****************************************************************
 *  Function    :  GetConnectionStatus()                        *
 *                                                              *
 *  Input       :  Connection Method get form system conf       *
 *                                                              *
 *  Output      :  WAN_GetStatusOK                              *
 *                 WAN_GetStatusErr                             *
 *  Note        :  Will fill in status in pStatus.              *
 ****************************************************************/
int GetConnectionStatus(IFACE_STATUS *pStatus){
    char if_name[6];
    char mac[6];

    /* Try to get connection Type. */
    ReadSystemSettings(&system_conf);

    memset(pStatus,0,sizeof(IFACE_STATUS));

	/* For I can not get ppp iface's MAC ,get ethx MAC first.*/
	GetIfaceStatus(ETH_WAN,pStatus);
	memcpy(mac,pStatus->mac,6);

    switch(system_conf.Connection){
    case CONNECTION_DHCP: /* DHCP */
    case CONNECTION_LOCAL_OFFICE: /* Local Office */
        strcpy(if_name, ETH_WAN);
        break;
    case CONNECTION_SINGLE_PPPOE: /* Single PPPoE */
    case CONNECTION_MULTI_PPPOE:  /*  Multi-PPPoE */
#ifdef _WAN_MODEM_
    case CONNECTION_MODEM:        /*  Modem */
#endif
	strcpy(if_name,"ppp0");
        break;
    default:
        return WAN_GetStatusErr;
    }

    GetIfaceStatus(if_name,pStatus);
    /* Then copy MAC back. */
    memcpy(pStatus->mac,mac,6);

    return WAN_GetStatusOK;
}

/* Global pid struct , should be inited at boot time. */
static PID  pid;

int Get_Version(BYTE *buf)
{
	VCI_TABLE Pid;
	unsigned	short * dwTemp=NULL;
	int fd1,ret=0;

	fd1=open("/dev/mtd0",O_RDONLY);
	if(fd1<0)
		return -1;
	ret=lseek(fd1,64*1024-63,SEEK_SET);
	if(ret!=64*1024-63){
		close(fd1);
		return -1;
	}
	if(read(fd1,(BYTE *)&Pid,sizeof(VCI_TABLE))!=sizeof(VCI_TABLE)){
		close(fd1);
		return -1;
	}
	close(fd1);
	dwTemp = (WORD *)((BYTE *)&Pid+38);
	Pid.ProdID = *dwTemp;
	dwTemp = (WORD *)((BYTE *)&Pid+40);
	Pid.ProtID = *dwTemp;
	dwTemp = (WORD *)((BYTE *)&Pid+50);
	Pid.Fver = *dwTemp;
	memcpy(buf,(BYTE *)&Pid.VerControl,VCI_LEN);
	return VCI_LEN;
}

PID *ReadProgramID(void){
    return &pid;
}

int get_device_info(DEVICE_INFO *device){
    PID  *pid_p=NULL;
	BYTE dbData[512];

	memset(dbData,'\0',512);
    Get_Version((VOID *)dbData);
    pid_p = (PID *)dbData;
//    memset(&device_info,0,sizeof(DEVICE_INFO));
    memcpy(device->dbHWID, &(pid_p->hw), 6);
//    sprintf(device->FWVersion, "Version %d.%d", ((pid_p->wFwVersion & 0xf000)>>12), ((pid_p->wFwVersion & 0x0f00)>>8));
//    sprintf(device->FWVersion, "Version %d.%d0", 1, 0);
    sprintf(device->FWVersion, "R1.00");
    sprintf(device->FullFWVersion, "Version %d.%d Release %02d", ((pid_p->wFwVersion & 0xf000)>>12), ((pid_p->wFwVersion & 0x0f00)>>8), (pid_p->wFwVersion & 0xff));
    sprintf(device->device_name,"%s%02x%02x%02x",MAC_PREFIX,device->dbHWID[3],device->dbHWID[4],device->dbHWID[5]);

    return 0;
}

static int LockFile(void)
{
	int fd;

	fd = open("/var/lock/flash.lck",O_WRONLY|O_CREAT|O_EXCL,0666);
	if(fd == -1)
		return -1;
	close(fd);
	return 0;
}

static void UnLockFile(void)
{
	unlink("/var/lock/flash.lck");
}

int WriteConfToFlash(void)
{
 	int ret=0;
 	
	ret=LockFile();
	if(ret)
		return -1;
#ifndef _PC_TEST_
    initLED(LED_STATUS);
    turnOnLED(LED_STATUS);
    ret=flash_write_config(SYSCONF,MTD_CONFIG);
    turnOffLED(LED_STATUS);
#endif /* _PC_TEST_ */ 
    UnLockFile();
    
    return ret;
}

void doSomethingBeforeReboot(void){
    return;
}


/*
 *  do_reboot()  - This function will reboot the device.
 *                 Before reboot will do something like disconnect PPPoE.
 */
void do_reboot(void){
    doSomethingBeforeReboot();
    system("/bin/reboot");
    return;
}

/*
 * restore()    - This function will copy default config file (/htdocs/system.conf) to be system config
 *                (/var/system.conf) then write this config file to flash.
 */

int restore(void){
    char cmd[100];

    printf("Restore configuration...");
    sprintf(cmd,"cp %s %s -f",DEFCONF,SYSCONF);
    system(cmd);
    return WriteConfToFlash();
}

/*
 *  FactoryRestore()  - This function will restore the device's config to default.
 *                      Before restore will do something like disconnect PPPoE.
 *                      Then will do reboot.
 */

void FactoryRestore(void){
    doSomethingBeforeReboot();

    restore();

    system("/bin/reboot");
    return;
}

/****************************************************************
 *  Function    :  GetDNSIPFromFile()                           *
 *                                                              *
 *  Input       :  char *filename                               *
 *                                                              *
 *  Output      :  struct in_addr *                             *
 *                                                              *
 *  Note        :  This function will read filename, and        *
 *                 parse dns IP to struct.                      *
 *                 Max dns number is MAX_DNS_NUM                *
 *                 file format must be same as /etc/resolv.conf *
 ****************************************************************/

struct in_addr *GetDNSIPFromFile(char *filename){
    FILE *fp;
    char line[128];
    char space[128];
    char ip_str[16];
    static struct in_addr dns[MAX_DNS_NUM];
    int i=0,re;

    memset(dns,0,sizeof(dns));
    fp = fopen(filename,"r");
    if(!fp){
        return dns;
    }
    while(fgets(line,sizeof(line),fp)){
        if(sscanf(line,"nameserver%[ \t]%s",space,ip_str) != 2){
            continue;
        }
        if(inet_addr(ip_str)==INADDR_NONE){
            continue;
        }
        re=inet_aton(ip_str,&dns[i++]);
        if(i >= MAX_DNS_NUM){
            break;
        }
    }
    fclose(fp);
    return dns;
}

/****************************************************************
 *  Function    :  ModifyResolvConf()                           *
 *                                                              *
 *  Input       :  location                                     *
 *                     HEAD ==> Add dns from head               *
 *                     Tail ==> Add dns from tail               *
 *                 dns ==> dns IP you want to add               *
 *                 mustMod ==> if mustMod is set, the dns will  *
 *                             be the first/last nameserver.    *
 *                                                              *
 *  Output      :  System_OK                                    *
 *                 System_WriteConfErr                          *
 *                                                              *
 ****************************************************************/
int ModifyResolvConf(int location,struct in_addr dns,int mustMod){
    struct in_addr *dns_p = GetDNSIPFromFile(RESOLV_FILE);
    struct in_addr new_dns[MAX_DNS_NUM];
    int i;
    FILE *fp;

    if(!dns.s_addr){
        return System_OK;
    }
    for(i=0; i < MAX_DNS_NUM; i++){
        if(dns.s_addr == dns_p[i].s_addr){
            /* Already in resolv.conf , no need to modidy. */
            if(!mustMod){
                return System_OK;
            }
        }
    }

    /* Make new dns address array. */
    memset(new_dns,0,sizeof(new_dns));
    if(location == ADD_IN_HEAD){
        new_dns[0] = dns;
        for(i=1;i<MAX_DNS_NUM;i++){
            if(dns.s_addr == dns_p[i-1].s_addr){
                continue;
            }
            new_dns[i] = dns_p[i-1];
        }
    }
    else{
        for(i=0;i<MAX_DNS_NUM-1;i++){
            /* dns_p[i] is zero , so the follow must be zero too. */
            if(!dns_p[i].s_addr){
                break;
            }

            if(dns.s_addr == dns_p[i].s_addr){
                continue;
            }
            new_dns[i] = dns_p[i];
        }
        new_dns[i] = dns;
    }

    fp = fopen(RESOLV_FILE,"w");
    if(!fp){
        return System_WriteConfErr;
    }

    /* Write new dns addresses to resolv.conf */
    for(i=0;i<MAX_DNS_NUM;i++){
        if(!new_dns[i].s_addr){
            break;
        }
        fprintf(fp,"nameserver %s\n",inet_ntoa(new_dns[i]));
    }

    fclose(fp);

    return System_OK;
}

/****************************************************************
 *  Function    :  GetPreferDNS()                               *
 *                                                              *
 *  Input       :  struct in_addr *                             *
 *                                                              *
 *  Output      :  System_OK                                    *
 *                 System_ReadConfErr                           *
 *                 System_WriteConfErr                          *
 ****************************************************************/

int GetPreferDNS(struct in_addr *dns_p){
    int ret = 0;
    if(!dns_p){
        return System_WriteConfErr;
    }

    memset(dns_p,0,sizeof(struct in_addr));

    ReadSystemSettings(&system_conf);
    switch(system_conf.Connection){
    case CONNECTION_DHCP:
        ret = ReadDHCPSettings(&dhcp_conf);
        if(ret){
            break;
        }
        dns_p->s_addr = ntohl(dhcp_conf.dns[0]);
        break;
    case CONNECTION_SINGLE_PPPOE:
        ret = ReadSinglePPPoESettings(&single_pppoe_conf);
        if(ret){
            break;
        }
        dns_p->s_addr = ntohl(single_pppoe_conf.dns[0]);
        break;
    case CONNECTION_MULTI_PPPOE:
        break;
#ifdef _WAN_MODEM_
    case CONNECTION_MODEM:
        ret = ReadModemSettings(&mdm_conf);
        if(ret){
            break;
        }
        dns_p->s_addr = !mdm_conf.mdm_dns_enable ? 0 : ntohl(mdm_conf.mdm_dns1);
        break;
#endif
    default:
        break;
    }
    if(ret){
        return System_ReadConfErr;
    }
    else{
        return System_OK;
    }
}

int get_wan_type()
{
	SYSTEM_CONF sys_conf;
	ReadSystemSettings(&sys_conf);
	return sys_conf.Connection;
}

int set_wan_type(int wan_type)
{
	SYSTEM_CONF sys_conf;
	if(wan_type>=0&&wan_type<CONNECTION_NUM){
		ReadSystemSettings(&sys_conf);
		if(sys_conf.Connection!=wan_type){
			StopUPnP();
			if((sys_conf.Connection==CONNECTION_SINGLE_PPPOE)&&(wan_type!=CONNECTION_SINGLE_PPPOE)){
				SINGLE_PPPOE_CONF spoe_conf;
				ReadSinglePPPoESettings(&spoe_conf);
				if((TYPE_UNNUMBERED == spoe_conf.ipType)||(TYPE_UNNUMBERED_PRIVATE == spoe_conf.ipType)){
					LAN_CONF lan_conf;
					RestoreLANSettings(&lan_conf);
					SaveLanSettings(&lan_conf);	
					StartLAN(0);
				}
			}
		sys_conf.Connection=wan_type;
		SaveSystemSettings(&sys_conf);
		}
		return 0;
	}
	return 1;
}


