#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>

#include <arpa/inet.h>
#include <unistd.h>
#include "PRO_file.h"
#include "PRO_GetStr.h"
#include "PRO_GetInt.h"
#include "PRO_SetStr.h"
#include "PRO_GetChar.h"
#include "PRO_GetShort.h"
#include "PRO_GetLong.h"

#include "PRO_SetMultiItemStr.h"
#include "wireless.h"
#include "h_const.h"

WIRELESS_CONF wireless_conf;

/*===================================================================*
 * FUNCTION    : ASCIIKey2BINKey()                                   *
 *                                                                   *
 * INPUT       : Ascii  ==> Ascii key                                *
 *               Binary ==> Binary key                               *
 *               BinaryLen ==> Binary Key length                     *
 *                                                                   *
 * OUTPUT      : NULL                                                *
 *                                                                   *
 * Description : Convert ascii key to binary key, binary key length  *
 *               is BinaryLen. If Ascii is not 0-9a-fA-F binary byte *
 *               will be 0.                                          *
 *                                                                   *
 *===================================================================*/
static void ASCIIKey2BINKey(char *Ascii, unsigned char *Binary,int BinaryLen){
    int i;
    
    memset(Binary,0,BinaryLen);
    for(i=0;i<BinaryLen;i++){
        int a[2];
        if(Ascii[i*2]>='0' && Ascii[i*2]<='9'){
            a[0] = Ascii[i*2]-'0';
        }
        else{
            if(Ascii[i*2]>='a' && Ascii[i*2]<='f'){
                a[0] = Ascii[i*2]-'a'+10;
            }
            else{
                if(Ascii[i*2]>='A' && Ascii[i*2]<='F'){
                    a[0] = Ascii[i*2]-'A'+10;
                }
                else{
                    a[0] = 0;
                }
                
            }
        }
        
        if(Ascii[i*2+1]>='0' && Ascii[i*2+1]<='9'){
            a[1] = Ascii[i*2+1]-'0';
        }
        else{
            if(Ascii[i*2+1]>='a' && Ascii[i*2+1]<='f'){
                a[1] = Ascii[i*2+1]-'a'+10;
            }
            else{
                if(Ascii[i*2+1]>='A' && Ascii[i*2+1]<='F'){
                    a[1] = Ascii[i*2+1]-'A'+10;
                }
                else{
                    a[1] = 0;
                }
                
            }
        }
        
        Binary[i] = (a[0]<<4)+a[1];            
    }
    return;
}

/*===================================================================*
 * FUNCTION    : BINKey2ASCIIKey()                                   *
 *                                                                   *
 * INPUT       : Ascii  ==> Ascii key                                *
 *               Binary ==> Binary key                               *
 *               BinaryLen ==> Binary Key length                     *
 *                                                                   *
 * OUTPUT      : NULL                                                *
 *                                                                   *
 * Description : Convert binary key to ascii key, binary key length  *
 *               is BinaryLen. If binary key is invalid ascii byte   *
 *               will be '0'.                                        *
 *                                                                   *
 *===================================================================*/
static void BINKey2ASCIIKey(char *Ascii, unsigned char *Binary,int BinaryLen){
    int i;
    
    memset(Ascii,0,BinaryLen*2);
    for(i=0;i<BinaryLen;i++){
        sprintf(Ascii+i*2,"%02x",Binary[i]);
    }
    return;
}
    
char wirless_sec[] = "Wireless";
/*===================================================================*
 * FUNCTION    : ReadWirelessSettings                                *
 *                                                                   *
 * INPUT       : Wireless conf point                                 *
 *                                                                   *
 * OUTPUT      : WIRELESS_OK ==> Successful                          *
 *               Other  ==> Fail                                     *
 *                                                                   *
 * Description : Read Wireless settings to pConf                     *
 *                                                                   *
 *===================================================================*/
int ReadWirelessSettings(WIRELESS_CONF *pConf)
{
	char key[32];

	char wep_key[27]; /* 128 bit key is 13 bytes. */
	char mac_key[13]; 
	FILE *fh;
	
	if(!pConf)
		return WIRELESS_ReadConfErr;

    fh=fopen(SYSCONF,"rt");

	if(!fh) 
		return WIRELESS_ReadConfErr;
	rewind(fh);

	sprintf(key,"BSSID"); 
	PRO_GetStr(wirless_sec,key,pConf->pcmcia.BSSID,32+1,fh);
	
	sprintf(key,"Channel"); 
	PRO_GetChar(wirless_sec,key,(char *)&pConf->pcmcia.Channel,fh);


	sprintf(key,"Preamble"); 
	PRO_GetChar(wirless_sec,key,(char *)&pConf->pcmcia.Preamble,fh);

	sprintf(key,"DTIMPeriod"); 
	PRO_GetChar(wirless_sec,key,(char *)&pConf->pcmcia.DTIMPeriod,fh);

	sprintf(key,"BasicDataRate"); 
	PRO_GetLong(wirless_sec,key,(long *)&pConf->pcmcia.BasicDataRate,fh);

	sprintf(key,"SupportedDataRate"); 
	PRO_GetLong(wirless_sec,key,(long *)&pConf->pcmcia.SupportedDataRate,fh);
	
	sprintf(key,"TxDataRate"); 
	PRO_GetLong(wirless_sec,key,(long *)&pConf->pcmcia.TxDataRate,fh);

	sprintf(key,"Encryption"); 
	PRO_GetChar(wirless_sec,key,(char *)&pConf->pcmcia.Encryption,fh);

	sprintf(key,"defaultKeyID"); 
	PRO_GetChar(wirless_sec,key,(char *)&pConf->pcmcia.defaultKeyID,fh);

	sprintf(key,"key_length"); 
	PRO_GetChar(wirless_sec,key,(char *)&pConf->pcmcia.key_length,fh);

	sprintf(key,"key1");
	memset(wep_key,0,sizeof(wep_key));
	PRO_GetStr(wirless_sec,key,wep_key,sizeof(wep_key),fh);
	ASCIIKey2BINKey(wep_key,&pConf->pcmcia.Keys[0][0],13);
	

	sprintf(key,"key2");
	memset(wep_key,0,sizeof(wep_key));
	PRO_GetStr(wirless_sec,key,wep_key,sizeof(wep_key),fh);
	ASCIIKey2BINKey(wep_key,&pConf->pcmcia.Keys[1][0],13);

	sprintf(key,"key3");
	memset(wep_key,0,sizeof(wep_key));
	PRO_GetStr(wirless_sec,key,wep_key,sizeof(wep_key),fh);
	ASCIIKey2BINKey(wep_key,&pConf->pcmcia.Keys[2][0],13);

	sprintf(key,"key4");
	memset(wep_key,0,sizeof(wep_key));
	PRO_GetStr(wirless_sec,key,wep_key,sizeof(wep_key),fh);
	ASCIIKey2BINKey(wep_key,&pConf->pcmcia.Keys[3][0],13);

	
	sprintf(key,"Mac");
	memset(mac_key,0,sizeof(mac_key));
	PRO_GetStr(wirless_sec,key,mac_key,sizeof(mac_key),fh);
	ASCIIKey2BINKey(mac_key,pConf->pcmcia.Mac,6);

	sprintf(key,"Authentication"); 
	PRO_GetChar(wirless_sec,key,(char *)&pConf->pcmcia.Authentication,fh);

	sprintf(key,"BeaconInterval"); 
	PRO_GetShort(wirless_sec,key,(short *)&pConf->pcmcia.BeaconInterval,fh);

	sprintf(key,"RTS"); 
	PRO_GetShort(wirless_sec,key,(short *)&pConf->pcmcia.RTS,fh);

	sprintf(key,"Fragmentation"); 
	PRO_GetShort(wirless_sec,key,(short *)&pConf->pcmcia.Fragmentation,fh);

	sprintf(key,"Mode"); 
	PRO_GetShort(wirless_sec,key,(short *)&pConf->pcmcia.Mode,fh);

	sprintf(key,"EnhanceSecurity"); 
	PRO_GetChar(wirless_sec,key,(char *)&pConf->pcmcia.EnhanceSecurity,fh);

	sprintf(key,"ClientAccessMode"); 
	PRO_GetChar(wirless_sec,key,(char *)&pConf->ClientAccessMode,fh);

	return 0;
}



int CheckWirelessSettings(WIRELESS_CONF *pConf){
	
	return WIRELESS_OK;
	
}

int SaveWirelessSettings(WIRELESS_CONF *pConf)
{
	multi_items	all_items[WIRELESS_ITEM_NUMBER];
	
	char Channel[3],Preamble[2],DTIMPeriod[4],BasicDataRate[3],SupportedDataRate[3];
	char TxDataRate[3],Encryption[2],defaultKeyID[2],key_length[3],Keys[4][27],Mac[13];
	char Authentication[2],BeaconInterval[6],RTS[6],Fragmentation[6],Mode[2],EnhanceSecurity[2];
    char ClientAccessMode[3];
    	
	int ret;
 
	ret=CheckWirelessSettings(pConf);
	if(ret){
		return ret;
	}

	sprintf(all_items[WIRELESS_ITEM_BSSID].key,"BSSID");
	all_items[WIRELESS_ITEM_BSSID].value=pConf->pcmcia.BSSID;  
  
	sprintf(all_items[WIRELESS_ITEM_CHANNEL].key,"Channel");
	sprintf(Channel,"%d",pConf->pcmcia.Channel);	
	all_items[WIRELESS_ITEM_CHANNEL].value=Channel;  

	sprintf(all_items[WIRELESS_ITEM_PREAMBLE].key,"Preamble");
	sprintf(Preamble,"%d",pConf->pcmcia.Preamble);	
	all_items[WIRELESS_ITEM_PREAMBLE].value=Preamble;  

	sprintf(all_items[WIRELESS_ITEM_DTIMPERIOD].key,"DTIMPeriod");
	sprintf(DTIMPeriod,"%d",pConf->pcmcia.DTIMPeriod);	
	all_items[WIRELESS_ITEM_DTIMPERIOD].value=DTIMPeriod;  

	sprintf(all_items[WIRELESS_ITEM_BASICDATARATE].key,"BasicDataRate");
	sprintf(BasicDataRate,"%d",(int)pConf->pcmcia.BasicDataRate);	
	all_items[WIRELESS_ITEM_BASICDATARATE].value=BasicDataRate;  

	sprintf(all_items[WIRELESS_ITEM_SUPPORTEDDATARATE].key,"SupportedDataRate");
	sprintf(SupportedDataRate,"%d",(int)pConf->pcmcia.SupportedDataRate);	
	all_items[WIRELESS_ITEM_SUPPORTEDDATARATE].value=SupportedDataRate;  

	sprintf(all_items[WIRELESS_ITEM_TXDATARATE].key,"TxDataRate");
	sprintf(TxDataRate,"%d",(int)pConf->pcmcia.TxDataRate);	
	all_items[WIRELESS_ITEM_TXDATARATE].value=TxDataRate;  

	sprintf(all_items[WIRELESS_ITEM_ENCRYPTION].key,"Encryption");
	sprintf(Encryption,"%d",pConf->pcmcia.Encryption);	
	all_items[WIRELESS_ITEM_ENCRYPTION].value=Encryption;  

	sprintf(all_items[WIRELESS_ITEM_DEFAULTKEYID].key,"defaultKeyID");
	sprintf(defaultKeyID,"%d",pConf->pcmcia.defaultKeyID);	
	all_items[WIRELESS_ITEM_DEFAULTKEYID].value=defaultKeyID;  

	sprintf(all_items[WIRELESS_ITEM_KEY_LENGTH].key,"key_length");
	sprintf(key_length,"%d",pConf->pcmcia.key_length);	
	all_items[WIRELESS_ITEM_KEY_LENGTH].value=key_length;  

	sprintf(all_items[WIRELESS_ITEM_KEY1].key,"key1");
	BINKey2ASCIIKey(&Keys[0][0],&pConf->pcmcia.Keys[0][0],13);	
	all_items[WIRELESS_ITEM_KEY1].value=&Keys[0][0];  

	sprintf(all_items[WIRELESS_ITEM_KEY2].key,"key2");
	BINKey2ASCIIKey(&Keys[1][0],&pConf->pcmcia.Keys[1][0],13);	
	all_items[WIRELESS_ITEM_KEY2].value=&Keys[1][0];  

	sprintf(all_items[WIRELESS_ITEM_KEY3].key,"key3");
	BINKey2ASCIIKey(&Keys[2][0],&pConf->pcmcia.Keys[2][0],13);	
	all_items[WIRELESS_ITEM_KEY3].value=&Keys[2][0];  

	sprintf(all_items[WIRELESS_ITEM_KEY4].key,"key4");
	BINKey2ASCIIKey(&Keys[3][0],&pConf->pcmcia.Keys[3][0],13);	
	all_items[WIRELESS_ITEM_KEY4].value=&Keys[3][0];  

	sprintf(all_items[WIRELESS_ITEM_MAC].key,"Mac");
	BINKey2ASCIIKey(Mac,pConf->pcmcia.Mac,6);
	all_items[WIRELESS_ITEM_MAC].value=Mac;

	sprintf(all_items[WIRELESS_ITEM_AUTHENTICATION].key,"Authentication");
	sprintf(Authentication,"%d",pConf->pcmcia.Authentication);	
	all_items[WIRELESS_ITEM_AUTHENTICATION].value=Authentication;  

	sprintf(all_items[WIRELESS_ITEM_BEACONINTERVAL].key,"BeaconInterval");
	sprintf(BeaconInterval,"%d",pConf->pcmcia.BeaconInterval);	
	all_items[WIRELESS_ITEM_BEACONINTERVAL].value=BeaconInterval;  

	sprintf(all_items[WIRELESS_ITEM_RTS].key,"RTS");
	sprintf(RTS,"%d",pConf->pcmcia.RTS);	
	all_items[WIRELESS_ITEM_RTS].value=RTS;  

	sprintf(all_items[WIRELESS_ITEM_FRAGMENTATION].key,"Fragmentation");
	sprintf(Fragmentation,"%d",pConf->pcmcia.Fragmentation);	
	all_items[WIRELESS_ITEM_FRAGMENTATION].value=Fragmentation;  

	sprintf(all_items[WIRELESS_ITEM_MODE].key,"Mode");
	sprintf(Mode,"%d",pConf->pcmcia.Mode);	
	all_items[WIRELESS_ITEM_MODE].value=Mode;  

	sprintf(all_items[WIRELESS_ITEM_ENHANCESECURITY].key,"EnhanceSecurity");
	sprintf(EnhanceSecurity,"%d",pConf->pcmcia.EnhanceSecurity);	
	all_items[WIRELESS_ITEM_ENHANCESECURITY].value=EnhanceSecurity;  

	sprintf(all_items[WIRELESS_ITEM_CLIENTACCESSMODE].key,"ClientAccessMode");
	sprintf(ClientAccessMode,"%d",pConf->ClientAccessMode);	
	all_items[WIRELESS_ITEM_CLIENTACCESSMODE].value=ClientAccessMode;
		
	
	ret=PRO_SetMultiItemStr(wirless_sec,all_items,WIRELESS_ITEM_NUMBER, SYSCONF);
	if(ret){
		return WIRELESS_SaveConfErr;
	}
	return WIRELESS_OK;
}

#define IWCONFIG "/sbin/iwconfig"
int SetWireless()
{
	SetWirelessNormal();
	SetWirelessWep();
	return 0;
}
int SetWirelessNormal()
{
	WIRELESS_CONF *pConf;
	char cmd[128];	
	ReadWirelessSettings(&wireless_conf);
	pConf=&wireless_conf;
	sprintf(cmd,"%s %s essid %s channel %d >/var/log/iw.log 2>&1",IWCONFIG, ISL_LAN, pConf->pcmcia.BSSID, pConf->pcmcia.Channel+1);
	system(cmd);
	return 0;
}

int SetWirelessWep()
{
	WIRELESS_CONF *pConf;
	char cmd[128];
	char wep[4], auth[6], key[30]={0};
	int keyid;
	ReadWirelessSettings(&wireless_conf);
	pConf=&wireless_conf;
	sprintf(wep,"%s", pConf->pcmcia.Encryption?"on":"off");
	switch(pConf->pcmcia.Authentication){
		case 0:
		default:
			sprintf(auth,"%s","none");
			break;
		case 1:
			sprintf(auth,"%s","open");
			break;
		case 2:
			sprintf(auth,"%s","share");
			break;
		case 3:
			sprintf(auth,"%s","both");
			break;
	}
	if(pConf->pcmcia.Encryption){
		keyid=pConf->pcmcia.defaultKeyID;
		BINKey2ASCIIKey(key,&pConf->pcmcia.Keys[keyid][0],pConf->pcmcia.key_length);
		sprintf(cmd,"%s %s wep %s auth %s keyid %d key %d 0x%s >/var/log/iw.log 2>&1",IWCONFIG , ISL_LAN, wep, auth, keyid, keyid, key);
	}else
		sprintf(cmd,"%s %s wep %s auth %s  >/var/log/iw.log 2>&1",IWCONFIG, ISL_LAN, wep, "both");
	system(cmd);
	return 0;
}
