#ifndef __WIRELESS_H__
#define __WIRELESS_H__

typedef struct{    
   char BSSID[32+1];       //default : "default"   
   char Channel;           //default : 1,  (range 1, 14)
   char Preamble;                    // 0 : Long, 1 : Short preamble, default : 0
   char DTIMPeriod;                  // 1~255, 0 is reseved, default : 1                
   unsigned long BasicDataRate;     //default : 3,  (range 1, 15)
   unsigned long SupportedDataRate; //default : 15, (range 1, 15)
   unsigned long TxDataRate;        //default : 15, (range 1, 15)
   char Encryption;        //default : 0 (Off),   (range: 0, 1)
   char defaultKeyID;      //default : 0 (KEY 1), (range: 0, 3)
   char key_length;        //default : 5,  (5 [64 bit] or 13 [128 bit])
   unsigned char Keys[4][14];       //default : 0....0
   unsigned char Mac[6];            //default : 000000, use PC Card Mac Address
   char Authentication;    //default : 1(Open System), (range 1, 2(share key))
   unsigned short BeaconInterval;    //default : 100
   unsigned short RTS;               //default : 2432, range (0..3000)
   unsigned short Fragmentation;     //default : 2346, range (256..2346) Even number
   unsigned short Mode;              //default : 0 (AP + Auto)
                                     //          bit 0~1 : 0 (Auto),    1 (2.0 PC CARD), 2 (2.5 PC CARD)
                                     //          bit 2: 0(Infrastructure), 1(802.11 Ad-Hoc)
                                     //          bit 3: 1 (ad-hoc mode)
                                     //          bit 4~11: Reserve
                                     //          bit 14 13 12
                                     //               0  0  0   -> Dynamic (g & b mode)
                                     //               0  0  1   -> g mode only
                                     //               others    -> reserved (g & b mode)
                                     //          bit 15  : 0 (AP mode), 1 (Station mode)
   char EnhanceSecurity;             //default : 0 (broadcast SSID in beacon frame)                         
                                     //   bit 0 : Hide SSID in beacon frame
                                     //   bit 1 : Block response to "Unspecified-SSID"
} PCMCIA_PARAMETERS;

#define TX_RATE_1M      0x00000001
#define TX_RATE_2M      0x00000002
#define TX_RATE_5_5M    0x00000004
#define TX_RATE_11M     0x00000008
#define TX_RATE_22M     0x00000010
#define TX_RATE_6M      0x00000010
#define TX_RATE_9M      0x00000020
#define TX_RATE_12M     0x00000040
#define TX_RATE_18M     0x00000080
#define TX_RATE_24M     0x00000100
#define TX_RATE_36M     0x00000200
#define TX_RATE_48M     0x00000400
#define TX_RATE_54M     0x00000800
#define TX_RATE_AUTO    0xffffffff


typedef struct{
    PCMCIA_PARAMETERS pcmcia;
    unsigned char ClientAccessMode;    
}WIRELESS_CONF;

extern WIRELESS_CONF wireless_conf;

#define ALL_ACCESS_LAN      0x01        /* All wireless client PCs can access wire PC in LAN side. */
#define SOME_ACCESS_LAN     0x02        /* Only selected wireless client PCs can access wire PC in LAN side. */
#define ALL_ACCESS_WAN      0x04        /* All wireless client PCs can access Internet */
#define SOME_ACCESS_WAN     0x08        /* Only selected wireless client PCs can access Internet */

enum{
    WIRELESS_ITEM_BSSID,
    WIRELESS_ITEM_CHANNEL,
    WIRELESS_ITEM_PREAMBLE,
    WIRELESS_ITEM_DTIMPERIOD,
    WIRELESS_ITEM_BASICDATARATE,
    WIRELESS_ITEM_SUPPORTEDDATARATE,
    WIRELESS_ITEM_TXDATARATE,
    WIRELESS_ITEM_ENCRYPTION,
    WIRELESS_ITEM_DEFAULTKEYID,
    WIRELESS_ITEM_KEY_LENGTH,
    WIRELESS_ITEM_KEY1,
    WIRELESS_ITEM_KEY2,
    WIRELESS_ITEM_KEY3,
    WIRELESS_ITEM_KEY4,
    WIRELESS_ITEM_MAC,
    WIRELESS_ITEM_AUTHENTICATION,
    WIRELESS_ITEM_BEACONINTERVAL,
    WIRELESS_ITEM_RTS,
    WIRELESS_ITEM_FRAGMENTATION,
    WIRELESS_ITEM_MODE,
    WIRELESS_ITEM_ENHANCESECURITY,
    WIRELESS_ITEM_CLIENTACCESSMODE,
    WIRELESS_ITEM_NUMBER
};

#define	WIRELESS_OK					0			//Succeed
#define	WIRELESS_SaveConfErr		-1			//Fail to save Wireless settings.
#define	WIRELESS_ReadConfErr		-2			//Fail to read Wireless settings.


int ReadWirelessSettings(WIRELESS_CONF *pConf);
int SaveWirelessSettings(WIRELESS_CONF *pConf);
int SetWireless();
int SetWirelessNormal();
int SetWirelessWep();
#endif /* __WIRELESS_H__ */
