#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <signal.h>
#include <netdb.h>
#include <syslog.h>
#include "libntp.h"

#define DEFAULT_PORT 23460
// #define DNS_LOOK_UP 5*60
#define DNS_LOOK_UP 5
#define DNS_RETRY_INTERVAL 60*60

extern char *optarg;

static int ntp_enable=0;
static int power_on_delay=60;
static int backoff=15;
static int delay = 1;

typedef struct {
	char server[256];
	short valid;	
} ntp_server;

void my_sleep(unsigned int val)
{
	int interval=val;
	while(interval>0)
		interval=sleep(interval);
}

/* Frist is Primary NTP Server  and next is Second NTP Server */
#if 0
static ntp_server ntp_server_list[]={
	/* GMT - 1, 2, 3 */
	{"ben.cs.wisc.edu", 1},
	// {"204.123.2.5", 1},
	/* GMT - 4, 5, 6 */
	{"time-nw.nist.gov", 1},
	/* GMT - 7, 8, 9 */
	{"bigben.cac.washington.edu", 1},
	/* GMT - 10, 11, 12 */
	{"nist1-dc.glassey.com", 1},
	/* GMT + 9, 10, 11 */
	{"nist1.aol-va.truetime.com", 1},
	/* GMT + 6, 7, 8 */
	{"time.xmission.com", 1},
	/* GMT + 3, 4, 5 */
	{"tick.uh.edu", 1},
	/* GMT + 0, 1, 2 */
	{"otc1.psu.edu", 1},
};
#else
static ntp_server ntp_server_list[]={
	/* GMT - 1, 2, 3 */
	{"time-a.netgear.com", 1},
	// {"204.123.2.5", 1},
	/* GMT - 4, 5, 6 */
	{"time-b.netgear.com", 1},
	/* GMT - 7, 8, 9 */
	{"time-c.netgear.com", 1},
	/* GMT - 10, 11, 12 */
	{"time-d.netgear.com", 1},
	/* GMT + 9, 10, 11 */
	{"time-e.netgear.com", 1},
	/* GMT + 6, 7, 8 */
	{"time-f.netgear.com", 1},
	/* GMT + 3, 4, 5 */
	{"time-g.netgear.com", 1},
	/* GMT + 0, 1, 2 */
	{"time-h.netgear.com", 1},
};
#endif

// return 0: DNS ok
// return 1: DNS failed
int try_dns_lookup(char *host)
{
	int i=3; /* 3 times */
	struct hostent *ntpserver=NULL;
	
	while(i-- > 0){
		ntpserver = gethostbyname(host);
		if (ntpserver == NULL) {
			sleep(DNS_LOOK_UP);
			continue;
		}
		else if (ntpserver->h_length != 4) {
			sleep(DNS_LOOK_UP);
			continue;
		}
		else {
			return 0;
		}
	}
	return -1;
}

void signal_handle(int sign)
{
	switch(sign){
		case SIGUSR1:
			if(ntp_enable==0){
				fprintf(stderr,"F<%s>L<%d>\n",__FILE__,__LINE__);
				ntp_enable=1;
				/* power on delay 60 - 300 secs */
				power_on_delay=60+(int) (60.0*rand()/(RAND_MAX+1.0));
				if (delay == 1) {
					fprintf(stderr,"F<%s>L<%d>\n",__FILE__,__LINE__);
					my_sleep(power_on_delay);
				}
			}
			break;
		case SIGUSR2:
			ntp_enable=0;
			break;
	}

	}

// parameter: time server to do DNS lookup first
// return: 0 -> time server
//         1 -> time server
//        -1 -> DNS lookup failed on both time servers
int dns_ntp_server(int server0, ntp_server * my_server) {

	int server1 = 0;
	
	if (server0 == 0) {
		server1 = 1;
	} else if (server0 == 1) {
		server1 = 0;
	}

	if (try_dns_lookup(my_server[server0].server) == 0) {
		return server0;
	} else if (try_dns_lookup(my_server[server1].server) == 0) {
		return server1;
	} 
		
	return -1;
}


// request NTP time
// return: NTP_ERR -> no valid NTP packet received
// 	   NTP_KOD -> received a KOD packet
int req_ntp_time(int sockfd, int timeserver, ntp_server * my_server) {
	
	int retval1 = 0;
	int retval2 = 0;
	int retval3 = 0;
	char ntpserver[16];
	char buf[1500]; 
		
	openlog("ntp",0,LOG_SYSLOG);
	syslog(LOG_INFO,"Send out NTP request to %s", my_server[timeserver].server);
	retval1 = send_request(sockfd, my_server[timeserver].server, 123);
	retval2 = recv_response(sockfd,buf,30,ntpserver);
	
	if ((retval1 != NTP_OK) || (retval2 != NTP_OK)) {
		closelog();
		return NTP_ERR;
	}
	
	
	retval3 = handle_ntp_reply(buf);

	if (retval3 == NTP_ERR) {
		return NTP_ERR;
	} else if (retval3 == NTP_KOD) {
		return NTP_KOD;
	} else if (retval3 == NTP_OK) {
		syslog(LOG_INFO,"Receive NTP Reply from %s", my_server[timeserver].server);
		closelog();
		return NTP_OK;
	}

	return NTP_ERR;
}	
	
// get the primary and secondary time server 
// according to the time zone
int get_timeserver(ntp_server * my_server, int zone) {

	int i = 0;
	int j = 0;
	
	j = zone + 2;
	
	if (zone > 7 || zone < 0) {
		return -1;
	}
	
	if (my_server[0].valid == 1) {
		i = 1;
	}

	while (i < 2) {
		strcpy(my_server[i].server, ntp_server_list[zone].server);
		my_server[i].valid = ntp_server_list[zone].valid;
		zone++;
		zone = zone % 8;
		i++;
	}

	return 0;
}




void ntp_process_flow(int sockfd, ntp_server * my_server)
{
	int retval = 0;
	int timeserver=0;


	while(1){
		
		// this is the upper part DNS lookup 
		// (see the Netgear home router flow chart V1.5)
		while (1) {
			timeserver = dns_ntp_server(0, my_server);
			
			if (timeserver == -1) {
				my_sleep(DNS_RETRY_INTERVAL); /* 1 hr */
			} else {
				break;
			}
		}	
	

		/* backoff 15 - 60 secs */
		backoff=15+(int) (45.0*rand()/(RAND_MAX+1.0));
	
		/*
		// check if NTP client is enabled again
		// if not, then go back to the beginning of the loop
		if (ntp_enable == 0) {
			return;
		}
		*/

		while (1) {
			
			if (my_server[0].valid == 0 && my_server[1].valid == 0) {
				sleep(1);
				continue;
			}
			fprintf(stderr,"F<%s>L<%d>\n",__FILE__,__LINE__);
			retval = req_ntp_time(sockfd, timeserver, my_server);
		
			
			if (retval == NTP_OK) {	
				// sync system clock
				// sleep 70 hours, then start with dns lookup again
				my_sleep (252000);
				break;
			
			} else if (retval == NTP_KOD) {
				// remove the current timeserver from my_server
				my_server[timeserver].valid = 0;
			
			} else if (retval == NTP_ERR) {
				// not getting valid NTP packet, do backoff
				while (backoff <= 252000) {
					my_sleep (backoff);
					backoff = backoff * 2;
					if (timeserver == 0) {
						timeserver = dns_ntp_server(1, my_server);
						if (timeserver != -1) {
							break;
						}
					} else if (timeserver == 1) {
						timeserver = dns_ntp_server(0, my_server);
						if (timeserver != -1) {
							break;
						}
					}
				}

				if (backoff <= 252000) {
					continue;
				} else {
					break;
				}
			}
		}
	}
}


void test(int zone) {

	return;
}

int main(int argc, char **argv)
{
	int sockfd = -1;
	int i = 0;
	char c;
	int zone = 0;

	ntp_server my_server[2];

	
	signal(SIGUSR1,&signal_handle);
	signal(SIGUSR2,&signal_handle);

	memset(my_server, 0x00, sizeof(my_server));
	/*
	close(0);
	close(1);
	close(2);
	*/
	daemon(1, 1);

	for (;;) {
		c = getopt( argc, argv, "h:z:c:d:f:");
		//fprintf(stderr,"c:<%c>c<%d>optarg<%s>",c,c,optarg);
		if (c == 255/*EOF*/) break;
		switch (c) {
			case 'h':
				strcpy(my_server[0].server, optarg);
				my_server[0].valid = 1;
				break;
			case 'z':
				zone = atoi(optarg);
				fprintf(stderr,"zone<%d>",zone);
				break;
			case 'c':

				break;
			case 'd':
				delay = 0;
				break;
			case 'f':

				break;

			default:
				//fprintf(stderr,"F<%s>L<%d>\n",__FILE__,__LINE__);
				exit(1);
		}
	}


	if (get_timeserver(my_server, zone) != 0) {
	//fprintf(stderr,"F<%s>L<%d>\n",__FILE__,__LINE__);
		return 0;
	}
        while (i < 2) {
		i++;
	}
	
	// this is the beginning of the LOOP
	while (1) {
	
		/* when ntp is disabled, sleep 10 sec */
		while(ntp_enable == 0){
		//fprintf(stderr,"F<%s>L<%d>\n",__FILE__,__LINE__);
			my_sleep(10);
		}	
		//fprintf(stderr,"F<%s>L<%d>\n",__FILE__,__LINE__);
		fflush(NULL);
		while((sockfd=setup_ntp_socket(INADDR_ANY,23460))<0){
				sleep(1);
		}
	
		fflush(NULL);
		ntp_process_flow(sockfd, my_server);	
	
		close(sockfd);
	}
	
	return 0;
}
