#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <net/if.h>
#include <sys/ioctl.h>
#define MAX_LEN   256
#define MAX_KEY_LEN   64
#define MAX_SECTION_LEN 64

#define WAN_GetStatusOK             0
#define WAN_GetStatusErr            -1
#define OPEN_FILE_ERROR             -2
#define IPV6_ReadConfErr	    -3
#define SYS_USER_LEN                21
#define SYS_PASSWD_LEN              11
#define SYS_CRYPT_LEN               14

#define System_OK                   0
#define System_ReadConfErr          -1
#define System_WriteConfErr         -2


#define REMOTE_IP		"remote_ip"
#define WAN_IP			"wan_ip"
#define REMOTE_ADD		"remote_add"
#define TUN_RELAY		"tun_relay"
#define SELECT			"select"
#define TUN_RELAY_DEFINE	"tun_relay_define"
#define IPV6_MaxIPLen		40	//length of ipv6 address fe80:0001:0002:0003:0004:0005:0006:0007
#define IPV6_WAN		"IPV6_WAN"
#define IPV6_TYPE_TUNNEL	"ipv6_type_tunnel"
#define IPV6_V4IPLen		16	//172.123.123.123
#include <arpa/inet.h>
typedef struct{
    char mac[6];
    short flag;
    struct in_addr ip;
    struct in_addr mask;
    struct in_addr gw;
}IFACE_STATUS;
typedef struct{
    char UserName[SYS_USER_LEN];        /* End with null */
    char Password[SYS_CRYPT_LEN];       /* End with null */
    char Connection;
}SYSTEM_CONF;

SYSTEM_CONF system_conf;
IFACE_STATUS if_status;

enum{
    CONNECTION_DHCP,
    CONNECTION_SINGLE_PPPOE,
    CONNECTION_MULTI_PPPOE,
    CONNECTION_MODEM
};




#define SYSCONF "/var/system.conf"
#ifndef OK
#define OK   0
#endif

char command[128];
static char system_sec[] = "System";
char *Connection_db[]={"Direct","PPPoE","Multi_PPPoE","Modem"};

/****************************************************************
 *  Function    :  GetConnectionStatus()                        *
 *                                                              *
 *  Input       :  Connection Method get form system conf       *
 *                                                              *
 *  Output      :  WAN_GetStatusOK                              *
 *                 WAN_GetStatusErr                             *
 *  Note        :  Will fill in status in pStatus.              *
 ****************************************************************/
int GetConnectionStatus(IFACE_STATUS *pStatus){
    char if_name[6];
    char mac[6];

    /* Try to get connection Type. */
    ReadSystemSettings(&system_conf);

    memset(pStatus,0,sizeof(IFACE_STATUS));

        /* For I can not get ppp iface's MAC ,get ethx MAC first.*/
        GetIfaceStatus("eth1",pStatus);
        memcpy(mac,pStatus->mac,6);

    switch(system_conf.Connection){
    case CONNECTION_DHCP: /* DHCP */
        strcpy(if_name,"eth1");
        break;
    case CONNECTION_SINGLE_PPPOE: /* Single PPPoE */
    case CONNECTION_MULTI_PPPOE: /*  Multi-PPPoE */
    case CONNECTION_MODEM:        /*  Modem */
        strcpy(if_name,"ppp0");
        break;
    default:
        return WAN_GetStatusErr;
    }

    GetIfaceStatus(if_name,pStatus);
    /* Then copy MAC back. */
    memcpy(pStatus->mac,mac,6);

    return WAN_GetStatusOK;
}
int str_db_search(char **db, const char *str){
    int i;

    for(i=0;db[i];i++){
        if(strcasecmp(db[i],str)==0){
            break;
        }
    }
    if(db[i]==0){
        i=-1;
    }
    return i;
}


/*===================================================================*
 * FUNCTION    : ReadSystemSettings                                  *
 *                                                                   *
 * INPUT       : System conf point                                   *
 *                                                                   *
 * OUTPUT      : System_OK ==> Successful                            *
 *               Other  ==> Fail                                     *
 *                                                                   *
 * Description : Read System settings to pConf                       *
 *                                                                   *
 *===================================================================*/
int ReadSystemSettings(SYSTEM_CONF *pConf){
    char key[32];
    char connection[40];
    FILE *fh;

        if(!pConf)
                return System_ReadConfErr;

        fh=fopen(SYSCONF,"rt");

        if(!fh)
                return System_ReadConfErr;
        rewind(fh);

        sprintf(key,"UserName");
        PRO_GetStr(system_sec,key,pConf->UserName,SYS_USER_LEN,fh);

        sprintf(key,"Password");
        PRO_GetStr(system_sec,key,pConf->Password,SYS_CRYPT_LEN,fh);

        sprintf(key,"Connection");
        PRO_GetStr(system_sec,key,connection,sizeof(connection),fh);
        pConf->Connection = str_db_search(Connection_db,connection);

        fclose(fh);

        return System_OK;
}
/********************************************************************************
 *  GetIfaceStatus()        Get interface status and put the data to *pStatus   *
 *                                                                              *
 *  0                   ==>     Successed                                       *
 *  other               ==>     Fail                                            *
 *                                                                              *
 ********************************************************************************/
int GetIfaceStatus(char *if_name, IFACE_STATUS *pStatus){
    struct ifreq ifr;
    int sockd;
    struct sockaddr_in *mysockaddr;

    memset(pStatus,0,sizeof(IFACE_STATUS));
    sockd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockd == -1) { /* Open socket error. */
                return WAN_GetStatusErr;
        }
        /* Init iface name. */
        strcpy(ifr.ifr_name, if_name);

    /* Try to get HW address. */
    if(ioctl(sockd, SIOCGIFHWADDR, &ifr) < 0) {
                close(sockd);
                return WAN_GetStatusErr;
        }
        memcpy(pStatus->mac,ifr.ifr_hwaddr.sa_data,6);

    /* Try to get IP address. */
    if(ioctl(sockd, SIOCGIFADDR, &ifr) < 0) {
                close(sockd);
                return WAN_GetStatusErr;
        }
        mysockaddr = (struct sockaddr_in *)&(ifr.ifr_addr);
        pStatus->ip = mysockaddr->sin_addr;

    /* Try to get netmask. */
    if(ioctl(sockd, SIOCGIFNETMASK, &ifr) < 0) {
                close(sockd);
                return WAN_GetStatusErr;
        }
        mysockaddr = (struct sockaddr_in *)&(ifr.ifr_netmask);
        pStatus->mask = mysockaddr->sin_addr;

#if 0
    /* Try to get gw. */
    if(ioctl(sockd, SIOCGIFDSTADDR, &ifr) < 0) {

                return WAN_GetStatusErr;
        }
        mysockaddr = (struct sockaddr_in *)&(ifr.ifr_addr);
        pStatus->gw = mysockaddr->sin_addr;
#else
{
    FILE *fp;
    char sys_cmd[128];
    #define ROUTE_RESULT    "/var/routeResult"

    sprintf(sys_cmd,"/sbin/route -n | grep \"^0.0.0.0\" > %s",ROUTE_RESULT);
    system(sys_cmd);

    fp = fopen(ROUTE_RESULT,"r");
    if(fp){

        char line[128];
        int i;
        while(fgets(line,sizeof(line),fp) != NULL){
            for(i=16;line[i] != ' ';i++);
            line[i] = '\0';
            inet_aton(&line[16],&pStatus->gw);
        }
        fclose(fp);
    }
    remove(ROUTE_RESULT);
}
#endif /* #if 0 */

    /* Try to get flag. */
    if(ioctl(sockd, SIOCGIFFLAGS, &ifr) < 0) {
                close(sockd);
                return WAN_GetStatusErr;
        }
        pStatus->flag = ifr.ifr_flags;
    close(sockd);

    return WAN_GetStatusOK;
}

void extract_ip(char *ip, unsigned char dst[4])
{
        char *p=NULL,*p1=NULL;

        if (!ip||*ip == '\n'||*ip == 0)
                goto err;
        dst[0] = 0;
        dst[1] = 0;
        dst[2] = 0;
        dst[3] = 0;
        p1=strdup(ip);
        if(!p1)
                return;
        p=strrchr(p1,'.');
        if(!p||p==p1)
                goto err;
        *p=0;
        if(p==p1+strlen(p1)-1)
                dst[3]=0;
        else
                dst[3]=atoi(p+1);
        p=strrchr(p1,'.');
        if(!p||p==p1)
                goto err;
        *p=0;
        dst[2]=atoi(p+1);
        p=strrchr(p1,'.');
        if(!p||p==p1)
                goto err;
        *p=0;
        dst[1]=atoi(p+1);
        dst[0]=atoi(p1);
        free(p1);
        return;
err:
        dst[0] = 0;
        dst[1] = 0;
        dst[2] = 0;
        dst[3] = 0;
        free(p1);
        return;
}

int PRO_2sect(char *sect, FILE *fp)
{
   char buf[MAX_LEN],sec[MAX_KEY_LEN],*p;
   int  ln;

   p = strstr(sect,"\n");
   if (p)
      *p = 0;
   if (sect[0] != '[')
   {
      sprintf(sec,"[%s]",sect);
      p = sec;
   }
   else
      p = sect;
   rewind(fp);
   ln = strlen(p);
   while(fgets(buf,sizeof(buf),fp))
   {
      if (buf[0] != '[')
         continue;
      if (strncmp(buf,p,ln)==0)
         return 0;
   }
   return 1;
}


int PRO_GetStr(char *sect, char *key, char * val, int size, FILE * fp)
{
   int  i,bl;
   char *buf,*p,*p0;
   char b[MAX_LEN]; /* a line read from ini file */

   if (val==NULL || fp==NULL || key==NULL)
      return -1;

   if (sect) {
      if (PRO_2sect(sect,fp) != OK) // section not found
         return -1;
   } else
      rewind(fp);

   if (size > MAX_LEN) {
      bl = size+MAX_KEY_LEN;
      buf = malloc(bl);
      if (!buf)
         return -1;
   } else {
      bl = sizeof(b);
      buf = b;   
   }

   i = strlen(key);
   while(fgets(buf,bl,fp))
   {
      if (buf[0]=='[')
         break; // end of section
      p=buf;
      while(*p==' ')
         p++;
      if (strncmp(p,key,i)==0)
      {
         p0=p+i;
         while (*p0==' ')
            p0++;
         if (*p0 != '=')
            continue;
         p0++;
         while (*p0==' ')
            p0++;
         p=buf+strlen(buf)-1;
         while(*p==0x0d || *p==0x0a)
            p--;
         *(p+1)=0;
         if (strlen(p0) > size)
            *(p0+size-1)=0;
         strcpy(val,p0);
         return OK;
      } //if (strncmp()
   } //while (fgets())
   return -1;
}


int PRO_GetShort(char *sect, char *key, short *val,FILE * fp)
{
   char buf[MAX_KEY_LEN];
   int ret;

   ret = PRO_GetStr(sect,key,buf,sizeof(buf),fp);
   if (ret==OK)
      *val = atol(buf);

   return ret;
}
int start_radvd(char *prefix)
{
	FILE *fp;
	fp = fopen("/var/radvd.conf","w");
	if(fp==NULL)
		return OPEN_FILE_ERROR;
	fputs("interface eth0\n",fp);
	fputs("{\n",fp);
	fputs("\tAdvSendAdvert on; \n",fp);
 	fill_ip(prefix);
 	sprintf(command,"\tprefix %s::/64",prefix);
        fputs(command,fp);
        
        fputs("\t{\n\t\tAdvOnLink on;\n\t\tAdvAutonomous on;\n\t\tAdvRouterAddr off;\n\t};\n",fp);
	fputs("}; \n",fp);
	fclose(fp);
	if(!access("/var/run/radvd.pid",F_OK))
		system("/bin/killall radvd");

	system("/sbin/radvd -C /var/radvd.conf  > /dev/null 2>&1");
	return OK;
}

int fill_ip(char *string)
{
        char *p;
        char buf[40];
        unsigned int i=0,j=0;
        p = string;
        for(;*p!='\0';p++){
                if(*p == ':')
                i++;
        }
        p = string;
        memset(buf,'\0',40);
        if(i==2){
                for(;*p!='\0';p++){
                        if ((*p==':')&&(*(p-1)==':')){
                                memcpy(&buf[j],"0:0",3);
                                j += 3;
                        }
                        buf[j++]=*p;
                }
        }
        if (i==3){
                for(;*p!='\0';p++){
                        if ((*p==':')&&(*(p-1)==':')){
                                buf[j++]='0';
                        }
                        buf[j++]=*p;
                }
        }
        if(string[strlen(string)-1]==':'){
                buf[j]='0';
        }
        strcpy(string,buf);
        return 0;
}

int down_v6_IP(char *addr,short prefix_len,char *interface)
{
	if (prefix_len){
		sprintf(command,"/sbin/ifconfig %s del %s/%d > /dev/null 2>&1\n",interface,addr,prefix_len);
		system(command);
	}
	return OK;
}

int config_v6_IP(char *addr,short prefix_len,char *interface)
{
	if (prefix_len){
		sprintf(command,"/sbin/ifconfig %s add %s/%d  > /dev/null  2>&1\n",interface,addr,prefix_len);
		system(command);
	}
	return OK;
}

int restart_tunnel(void)
{
	FILE *fp;
	char key[64],temp_buf[48],addr[42];
	short ipv6_type_tunnel,tun_relay,tun_relay_define;
	unsigned char remote_add[6],wan_ip[6],select1[40],prefix[42],remote_ip[20];

	fp=fopen(SYSCONF,"r");
	if(!fp){
		return IPV6_ReadConfErr;
	}

	memset(temp_buf,0,sizeof(temp_buf));
	GetConnectionStatus(&if_status);
	strcpy(temp_buf,inet_ntoa(if_status.ip));
	extract_ip(temp_buf,wan_ip);
	
	memset(temp_buf,0,sizeof(temp_buf));
	strcpy(key,REMOTE_ADD);
	PRO_GetStr(IPV6_WAN,key,temp_buf,IPV6_V4IPLen,fp);
	extract_ip(temp_buf,remote_add);

	memset(temp_buf,0,sizeof(temp_buf));
	strcpy(key,REMOTE_IP);
	PRO_GetStr(IPV6_WAN,key,temp_buf,IPV6_V4IPLen,fp);
	extract_ip(temp_buf,remote_ip);

	strcpy(key,IPV6_TYPE_TUNNEL);
	PRO_GetShort(IPV6_WAN,key,&ipv6_type_tunnel,fp);

	strcpy(key,TUN_RELAY_DEFINE);
	PRO_GetShort(IPV6_WAN,key,&tun_relay_define,fp);
	
	strcpy(key,SELECT);
	PRO_GetStr(IPV6_WAN,key,select1,IPV6_MaxIPLen,fp);
	
	fclose(fp);
	
	if (ipv6_type_tunnel){
		sprintf(addr,"2002:%02x%02x:%02x%02x:1::1",wan_ip[0],wan_ip[1],wan_ip[2],wan_ip[3]);
	    	config_v6_IP(addr, 64,"eth0");
		sprintf(prefix,"2002:%02x%02x:%02x%02x:1",wan_ip[0],wan_ip[1],wan_ip[2],wan_ip[3]);
		start_radvd(prefix);
							
	    	memset(addr,'\0',42);
	    	if (tun_relay_define)
	    		sprintf(addr,"%d.%d.%d.%d\n",remote_ip[0],remote_ip[1],remote_ip[2],remote_ip[3]);
		else{
			if(strcmp(select1,"140.113.54.119")==0)
				strcpy(addr,"140.113.54.119");
			if(strcmp(select1,"kddilab.6to4.jp")==0)
				strcpy(addr,"202.255.45.5");
			if(strcmp(select1,"6to4.ipv6.uni-leipzig.de")==0)
				strcpy(addr,"139.18.38.73");
	    	}
					    		
		start_624_tunnel(prefix,addr);
	}
	
}

int start_624_tunnel(char *prefix,unsigned char *remote_ip)
{
	stop_tunnel();
	system("/sbin/ifconfig sit0 up > /dev/null 2>&1");
	memset(command,'\0',128);
	sprintf(command,"/sbin/ip -6 tunnel add sit1 mode sit remote %s local any > /dev/null 2>&1",remote_ip);
	system(command);
	system("/sbin/ip link set dev sit1 up > /dev/null 2>&1");
	memset(command,'\0',128);
	sprintf(command,"/sbin/ip -6 addr add %s::1/48 dev sit1 > /dev/null 2>&1",prefix);
	system(command);
	system("/sbin/ip -6 route add 2000::/3 dev sit1 > /dev/null 2>&1");
//	system("/bin/echo 1 > /proc/sys/net/ipv6/conf/all/forwarding");
	return OK;
}

int stop_tunnel(void)
{
	char key[64],temp_buf[48],addr[42],wan_ip[4];

	memset(temp_buf,0,sizeof(temp_buf));
	GetConnectionStatus(&if_status);
	strcpy(temp_buf,inet_ntoa(if_status.ip));
	extract_ip(temp_buf,wan_ip);

	sprintf(addr,"2002:%02x%02x:%02x%02x:1::1",wan_ip[0],wan_ip[1],wan_ip[2],wan_ip[3]);
	down_v6_IP(addr,64,"eth0");

	system("/sbin/ip tunnel del sit1  > /dev/null 2>&1");
	system("/sbin/ifconfig sit0 down > /dev/null 2>&1");
	return OK;
}

