#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/mtd/mtd.h>
#include <sys/ioctl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "flash.h"

typedef	struct	mtd_partition{
	int  id;
	int  size;
	char dev[10];
}mtd_partition;

//#define	CONFIG_OK	"sErCoMm"

static mtd_partition mtd[PARTITION_NUM]={
	{	MTD_BOOT,		0x00040000,	"/dev/mtd0"},
	{	MTD_CONFIG,		0x00020000,	"/dev/mtd1"},
	{	MTD_KERNEL,		0x00100000,	"/dev/mtd2"},
    {	MTD_SYSTEM,		0x002a0000,	"/dev/mtd3"},
};

int flash_erase_mtd(int which)
{
	int	fd=0;	
	int	i;
	mtd_info_t	meminfo;
	erase_info_t	erase;
	int erase_size;
	
	for(i=0;i<PARTITION_NUM;i++){
		if(which&mtd[i].id){
			fd=open(mtd[i].dev,O_RDWR);
			if(fd<0)
				return FL_OPEN_DEV_ERR;
			memset(&meminfo,0,sizeof(meminfo));
			if(ioctl(fd,MEMGETINFO,&meminfo)!=0){	
				close(fd);
				return FL_READ_DEV_ERR;
			}
			memset(&erase,0,sizeof(erase));
			erase.length=meminfo.erasesize;
			erase.start=0;
			    erase_size = mtd[i].size;
			while(erase.start < erase_size){
				if(ioctl(fd,MEMERASE,&erase)!=0){
					close(fd);
					return FL_ERASE_ERR;
				}
				erase.start += meminfo.erasesize;
			}
			close(fd);
		}
	}
	return 0;
}

int flash_get_configf_size(int which)
{
	int	fd;
	int	i;
	int	size;

	if(which!=MTD_CONFIG)
		return -1;
	
	for(i=0;i<PARTITION_NUM;i++){
		if(which&mtd[i].id){
			fd=open(mtd[i].dev,O_RDONLY);
			if(fd<0)
				return -1;

			if(read(fd,&size,sizeof(int))!=sizeof(int)){
				close(fd);
				return -1;
			}
			close(fd);
			
			if(size>=0)
				return size;
			else
				return -1;
		}
	}
			
	return -1;
}
			
int flash_read_config(int which,char *fname,ssize_t size)
{
	int	fd;
	int	i;
	char	*p;
	
	if(which!=MTD_CONFIG)
		return FL_NOT_CONFIG_DEV;
	if(size>BLOCK_SIZE)
		return FL_TOO_LONG_SIZE;
	for(i=0;i<PARTITION_NUM;i++){
		if(which&mtd[i].id){
			fd=open(mtd[i].dev,O_RDONLY);
			if(fd<0)
				return FL_OPEN_DEV_ERR;
				
			lseek(fd,sizeof(int),SEEK_SET);

			p=(char *)malloc(size);
			if(!p){
				close(fd);
				return FL_NO_ENOUGH_MEM;
			}
			if(read(fd,p,size)!=size){
				close(fd);
				return FL_READ_DEV_ERR;
			}
			close(fd);
			fd=open(fname,O_WRONLY|O_CREAT);
			if(fd<0)
				return FL_OPEN_FILE_ERR;
			if(write(fd,p,size)!=size){
				close(fd);
				return FL_WRITE_FILE_ERR;
			}
			close(fd);
			return 0;
		}
	}
			
	return 0;
}

int flash_write_buf(const char *buf, ssize_t size, int which)
{
	int	fd=0;
	int	i;
	int	ret;
	
	if(buf==NULL)
		return FL_BUF_EMPTY;
	for(i=0;i<PARTITION_NUM;i++){
		if(which&mtd[i].id){
			if(size>mtd[i].size)
				return FL_TOO_LONG_SIZE;
			fd=open(mtd[i].dev,O_RDWR);
			if(fd<0)
				return FL_OPEN_DEV_ERR;
			ret=flash_erase_mtd(which);
			if(ret){
				close(fd);
				return ret;
			}
			ret=write(fd,buf,size);

			if(ret!=size){
				close(fd);
				return FL_WRITE_DEV_ERR;
			}
			
			close(fd);
			return 0;
		}
	}
	return 0;
}			
	

int flash_write_config(char *fname,int which)
{
	int	fd=0;
	int	i;
	int	ret;
	char	*buf=NULL;
	struct	stat st;
	
	if(which!=MTD_CONFIG)
		return FL_NOT_CONFIG_DEV;
	
	if(stat(fname,&st)<0)
		return FL_BAD_CONFIG;
		
	if(st.st_size>=BLOCK_SIZE)
		return FL_BAD_CONFIG;
	
	fd=open(fname,O_RDONLY);
	if(fd<0)
		return FL_OPEN_FILE_ERR;
			
	buf=(char *)malloc(st.st_size+1);
	if(!buf)
		return -1;
	memset(buf,0,st.st_size+1);
	ret=read(fd,buf,st.st_size);
	if(ret != st.st_size){
		free(buf);
		buf=NULL;
		close(fd);
		return FL_BAD_CONFIG;
	}
	close(fd);
		
	ret=flash_erase_mtd(which);
	if(ret){
		free(buf);
		buf=NULL;
		return ret;
	}
		
	for(i=0;i<PARTITION_NUM;i++){
		if(which&mtd[i].id){
			fd=open(mtd[i].dev,O_RDWR);
			if(fd<0){
				free(buf);
				buf=NULL;
				return FL_OPEN_DEV_ERR;
			}
				
			lseek(fd,sizeof(int),SEEK_SET);
			
			ret=write(fd,buf,st.st_size);
			if(ret!=st.st_size){
				close(fd);
				free(buf);
				buf=NULL;
				return FL_WRITE_DEV_ERR;
			}
			
			lseek(fd,0,SEEK_SET);
			if(write(fd,(int *)&(st.st_size),sizeof(int))<0){
				close(fd);
				free(buf);
				buf=NULL;
				return FL_WRITE_DEV_ERR;
			}
				
			close(fd);
			free(buf);
			buf=NULL;
			return 0;
		}
	}
	free(buf);
	buf=NULL;
	return 0;
}		

	
