#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <ctype.h>
#include "led.h"
extern size_t strspn( const char *s, const char *accept);

/* #include <utypes.h> */


/*************************************************
 *              Lan Reset (1 Output)
 *              Push Button(3 Input)
 *              Power (5 OUTPUT)
 *              Status (6 Output)  
 **************************************************/
 
typedef unsigned long U32;

#define	PREG_BASE	0xB8400000	/* Base address to Peripheral Reg  */
#define PER_BASE 	PREG_BASE

/**********************************************************/
/* GPIO registers										  */
/**********************************************************/
#define GPIO_DATA_REG			((volatile U32 *)(PER_BASE + 0xE0))	/* GPIO Data reg				*/
#define GPIO_DATA1_REG			((volatile U32 *)(PER_BASE + 0x174))	/* GPIO Data reg				*/
#define GPIO_DATA2_REG			((volatile U32 *)(PER_BASE + 0x178))	/* GPIO Data reg				*/
#define GPIO_DATA3_REG			((volatile U32 *)(PER_BASE + 0x17C))	/* GPIO Data reg				*/
#define GPIO_CFG1_REG			((volatile U32 *)(PER_BASE + 0xE4)) 	/* GPIO CFG1 reg				*/
#define GPIO_CFG2_REG			((volatile U32 *)(PER_BASE + 0xE8))	/* GPIO CFG2 reg	*/
#define GPIO_CFG3_REG           ((volatile U32 *)(PER_BASE + 0x170))	/* GPIO CFG3 reg	*/
#define GPIO_CFG4_REG           ((volatile U32 *)(PER_BASE + 0x18C))	/* GPIO CFG4 reg	*/
#define GPIO_CFG6_REG			((volatile U32 *)(PER_BASE + 0x194))	/* GPIO CFG6 reg	*/
#define GPIO_DATA5_REG          ((volatile U32 *)(PER_BASE + 0x184))    /* 5th GPIO Data reg   */
#define GPIO_CFG7_REG           ((volatile U32 *)(PER_BASE + 0x198))


#define MAXLEDSAVAILABLE	7

/*******************************************************************************
*
* initLED - initialize GPIO pins for the LED
*
* Input :  gpioPin to be used as LED (pin is set to output)
* 
********************************************************************************/

void initLED (int gpioPin)
{
    /* set the appropriate GPIO pins to be outputs */
    /* *GPIO_CFG1_REG controls GPIO 0-7 */
    /* *GPIO_CFG2_REG controls GPIO 8-15 */
//    if (gpioPin < 8)
//    {
//        *GPIO_CFG1_REG &= ~(0xF << (gpioPin * 4));
//        *GPIO_CFG1_REG |= 0x8 << (gpioPin * 4);
//    }
//    else
//    {
//        *GPIO_CFG2_REG &= ~(0xF << (gpioPin * 4));
//        *GPIO_CFG2_REG |= 0x8 << (gpioPin * 4);
//    }
    *GPIO_CFG7_REG &= ~(1<< 29);
    if(gpioPin < 8)
    {
        *GPIO_CFG1_REG &= ~(0xF << (gpioPin * 4));
        *GPIO_CFG1_REG |= 0x8 << (gpioPin * 4);
    }else if(gpioPin >=8 && gpioPin <16){
        *GPIO_CFG2_REG &= ~(0xF << ((gpioPin % 8) * 4));
        *GPIO_CFG2_REG |= 0x8 << ((gpioPin % 8) * 4);
    }else if(gpioPin >=16 && gpioPin <24){
        *GPIO_CFG3_REG &= ~(0xF << ((gpioPin % 8) * 4));
        *GPIO_CFG3_REG |= 0x8 << ((gpioPin % 8) * 4);
    }else if(gpioPin >=24 && gpioPin <32){
        *GPIO_CFG4_REG &= ~(0xF << ((gpioPin % 8) * 4));
        *GPIO_CFG4_REG |= 0x8 << ((gpioPin % 8) * 4);
    }else if(gpioPin >= 40 && gpioPin <= 47)
    {
        *GPIO_CFG6_REG &=~(0xF <<((gpioPin % 8) * 4));
        *GPIO_CFG6_REG |= 0x8 << ((gpioPin % 8) * 4);
    }
}

/*******************************************************************************
*
* turnOnLED - turn LED on associated with GPIO pin
*
* Input :  gpioPin used as LED (pin is set to output
* 
********************************************************************************/
#ifdef _BA8000PRO_11G_
void turnOnLED (int gpioPin)
#endif
#ifdef _TZ50_
void turnOffLED (int gpioPin)
#endif
{	
    if(gpioPin < 8)
        *GPIO_DATA_REG |= 1 << gpioPin;    /* turns LED off */
    else if(gpioPin >=8 && gpioPin <16)
        *GPIO_DATA1_REG |= 1 << (gpioPin % 8);
    else if(gpioPin >=16 && gpioPin <24)
        *GPIO_DATA2_REG |= 1 << (gpioPin % 8);
    else if(gpioPin >=24 && gpioPin <32)
        *GPIO_DATA3_REG |= 1 << (gpioPin % 8);
    else if(gpioPin >= 40 && gpioPin <=47)
        *GPIO_DATA5_REG |= 1 << (gpioPin % 8);
}

/*******************************************************************************
*
* turnOffLED - turn LED off associated with GPIO pin
*
* Input :  gpioPin used as LED (pin is set to output
* 
********************************************************************************/
#ifdef _BA8000PRO_11G_
void turnOffLED (int gpioPin)
#endif
#ifdef _TZ50_
void turnOnLED (int gpioPin)
#endif
{
    if(gpioPin < 8)
        *GPIO_DATA_REG &= ~(1 << gpioPin);    /* turns LED on */
    else if(gpioPin >= 8 && gpioPin <16)
        *GPIO_DATA1_REG &=~(1 << (gpioPin % 8));
    else if(gpioPin >= 16 && gpioPin <24)
        *GPIO_DATA2_REG &=~(1 << (gpioPin % 8));
    else if(gpioPin >= 24 && gpioPin <32)
        *GPIO_DATA3_REG &=~(1 << (gpioPin % 8));
    else if(gpioPin >= 40 && gpioPin <=47)
        *GPIO_DATA5_REG &=~(1 << (gpioPin % 8));
}

volatile int waiting;

void delay(int delaycount)
{
/*   int i; */
  
  sleep(delaycount);
/*   for (i = 0; i < delaycount; i++) */
/* 	{ */
/* 	  waiting = i; */
/* 	} */
}
#ifdef _TZ50_
void initPushButton (void)
{
    *GPIO_CFG7_REG &= ~(1<< 29);
    *GPIO_CFG3_REG &= ~(0xF << (4 * 4));    /* 2 is push button. */
}

int ButtonIsPushed(void)
{    
     
   unsigned int  dbPushButtom=0; 
 printf("led.c line<%d>\n",__LINE__);
   dbPushButtom = *GPIO_DATA2_REG;   

   dbPushButtom &= 0x10;
   //dbPushButtom >>= 2;
   if (dbPushButtom) {
      return 0;
   } else {
      return 1;
   }
    printf("led.c line<%d>\n",__LINE__);
}
#endif
#ifdef _BA8000PRO_11G_
void initPushButton (void)
{
    *GPIO_CFG7_REG &= ~(1<< 29);
    *GPIO_CFG6_REG &= ~(0xF << (2 * 4));    /* 2 is push button. */
}

int ButtonIsPushed(void)
{    
   unsigned int  dbPushButtom=0; 
 printf("led.c line<%d>\n",__LINE__);
   dbPushButtom = *GPIO_DATA5_REG;   

   dbPushButtom &= 0x04;
   //dbPushButtom >>= 2;
   if (dbPushButtom) {
      return 0;
   } else {
      return 1;
   }
    printf("led.c line<%d>\n",__LINE__);
}
#endif

/* ledflash <delay> <led> <led> <led> ... */

#if 0 
int main(int argc, char **argv)
{
	int i;
	int skip;
	int lastled = 0;
	int led;
	int delaycount = 0;
	unsigned long selectedleds = 0;

	printf("ledflash <delay count> <led> <led> <led> ...\n");
	printf("recommended delay count (seconds) is between 1 and 10 \n");
	printf("led can be 0 to %d inclusive\n", MAXLEDSAVAILABLE-1);

	for (i = 1; i < argc; i++)
	{
		skip = strspn(argv[i], " ");
		if (!isdigit(argv[i][skip]))
		{
			printf("Argument <%s> doesn't appear to be a number\n", argv[i]);
			continue;
		}
		led = atoi(&argv[i][skip]);

		if (i == 1)
		{
			delaycount = led;
			continue;
		}
		
		if (led >= MAXLEDSAVAILABLE)
		{
			printf("<%s> must be less than %d\n", argv[i], MAXLEDSAVAILABLE);
			continue;
		}
		selectedleds |= 1 << led;
	}
	
	if (selectedleds == 0)
	{
		printf("No leds selected\n");
		return 1;
	}
	
	printf("delay count selected %d\n", delaycount);

	for (led = 0; led < MAXLEDSAVAILABLE; led++)
	{
		if (selectedleds & (1<<led))
		{
			initLED(led);
			turnOffLED(led);
			lastled = led;
			printf("led %d selected\n", led);
		}
	}

	/* turn on/off leds */

	while (1)
	{
		turnOffLED(lastled);
		led = lastled + 1;
		while (led != lastled)
		{
			if (led < MAXLEDSAVAILABLE)
			{
				if (selectedleds & (1<<led))
					lastled = led;
				else
					led++;
			}
			else
				led = 0;
		}
		delay(delaycount);
		turnOnLED(lastled);

		/* scan devs checks for ctrl-c and other device activity */
/* 		scandevs(); */
		delay(delaycount);
	}
	return(0);
}

#endif /*#if 0 */
