#ifndef _FIREWALL_
#define _FIREWALL_


typedef struct service
{
	char	name[15];	
	char	type[5];	// UDP TCP BOTH
	int 	start_port;	// 1~65535
	int 	end_port;	// 1~65535
}service;



typedef struct rule
{
	int 	enable;
	service	serv;
	int 	action;			
	char	lan[35];	//any: 0/0 single: a.b.c.d range: 1.2.3.4-5.6.7.8
	char	wan[35];	//any: 0/0 single: a.b.c.d range: 1.2.3.4-5.6.7.8
	int	log;
}rule;


#define A_PREROUTING	"$IPTABLES -t nat -A PREROUTING -i $WANIF"
#define I_PREROUTING	"$IPTABLES -t nat -I PREROUTING -i $WANIF"
#define D_PREROUTING	"$IPTABLES -t nat -D PREROUTING -i $WANIF"
#define A_FORWARD	"$IPTABLES -A FORWARD -i $WANIF"
#define I_FORWARD	"$IPTABLES -I FORWARD -i $WANIF"
#define D_FORWARD	"$IPTABLES -D FORWARD -i $WANIF"
#define A_INPUT		"$IPTABLES -A INPUT -i $WANIF"
#define I_INPUT		"$IPTABLES -I INPUT -i $WANIF"
#define D_INPUT		"$IPTABLES -D INPUT -i $WANIF"

#define A_FORWARD_O	"$IPTABLES -A FORWARD -i $LANIF"
#define I_FORWARD_O	"$IPTABLES -I FORWARD -i $LANIF"
#define D_FORWARD_O	"$IPTABLES -D FORWARD -i $LANIF"

#define A_FORWARD_LOG	"$IPTABLES -A FORWARD -m state --state NEW -i $WANIF"
#define I_FORWARD_LOG	"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF"
#define D_FORWARD_LOG	"$IPTABLES -D FORWARD -m state --state NEW -i $WANIF"
#define A_INPUT_LOG	"$IPTABLES -A INPUT -m state --state NEW -i $WANIF"
#define I_INPUT_LOG	"$IPTABLES -I INPUT -m state --state NEW -i $WANIF"
#define D_INPUT_LOG	"$IPTABLES -D INPUT -m state --state NEW -i $WANIF"

#define A_FORWARD_LOG_O	"$IPTABLES -A FORWARD -m state --state NEW -i $LANIF"
#define I_FORWARD_LOG_O	"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF"
#define D_FORWARD_LOG_O	"$IPTABLES -D FORWARD -m state --state NEW -i $LANIF"

#define LOG_SUFFIX	"-j LOG --log-prefix"

#define ERR_NO_ERR	0
#define ERR_INVALID	-1
#define ERR_CHECK_SCHE	-2
#define ERR_OPENFILE	-3
#define ERR_NO_SERV	-4
	

#define BLOCK	0
#define ACCEPT	1

#define OUT_SCHEDULE	0
#define IN_SCHEDULE	1

#define ACT_ONLY_BLOCK	0
#define ACT_SCHE_BLOCK	1
#define ACT_ONLY_ALLOW	2
#define ACT_SCHE_ALLOW	3

#define LOG_NEVER	0	
#define LOG_ALWAYS	1
#define	LOG_MATCH	2
#define LOG_NOT_MATCH	3

/*
 * start firewall protection
 * @return	0 success otherwise error
 */
int start_firewall(void);

/*
 * stop firewall protection
 * @return	0 success otherwise error
 */
int stop_firewall(void);

/*
 * create firewall configuration
 * @return	0 success otherwise error
 */
int create_firewall_cfg(void);

/* 
 * Decide which type of ip address it is.
 * return 3(all)/2(range)/1(single)
 */
int ip_type(char *ipstr);

/* 
 * get service according to service name
 * @return  	success  0, otherwise error
 */
int get_serv(char *name, service *fw_serv);

/* 
 * create crontab for crond to re-run firewall rule 
 * @return  	success  0, otherwise error
 */
int create_crontab(void);

/* 
 * check current time to determine if now is in schedule or not
 * @return  	0 out schedule time
 *		1 in schedule time 
 *		otherwise error
 */
int check_sche(void);

/* 
 * Get IP address from an interface name 
 * @return ip address of interface
 * 
 */
char *getIPAddress(char *ifname);


/* 
 * determine to ACCEPT or BLOCK according to
 * schedule and rule setting 
 */
int action_by_schedule(int schedule, int action);


#endif
