#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include "fw_cfg.h"
#include "nvram.h"


/*
 * Get service according to service name
 * if success return 0, else return ERR_NO_SERV
 */
int get_serv(char *name, service *fw_serv)
{

//	char *ptr;
//	char token=0x01;
	char *fw_services=nvram_safe_get("fw_services");
	char *fw_services_def=nvram_safe_get("fw_services_def");


	/* parse each entry of service */
//	for(ptr=strtok(fw_services,&token);ptr;ptr=strtok(NULL,&token))
	while(*fw_services!='\0')
	{

		//parsing : service_name:both:299-500
		sscanf(fw_services,"%[^:]:%[^:]:%d-%d",
		       fw_serv->name,
		       fw_serv->type,
		       &(fw_serv->start_port),
		       &(fw_serv->end_port)
		      );

		 	while(*fw_services!='\0' && *fw_services!=0x01) fw_services++;
		    	if(*fw_services!='\0') fw_services++;

		// compare by using service name
		if(strcmp(fw_serv->name,name)==0)
			return ERR_NO_ERR;
	}

	/* parse each entry of service */
//	for(ptr=strtok(fw_services_def,&token);ptr;ptr=strtok(NULL,&token))
	while(*fw_services_def!='\0')
	{

		//parsing : service_name:both:299-500
		sscanf(fw_services_def,"%[^:]:%[^:]:%d-%d",
		       fw_serv->name,
		       fw_serv->type,
		       &(fw_serv->start_port),
		       &(fw_serv->end_port)
		      );

		 	while(*fw_services_def!='\0' && *fw_services_def!=0x01) fw_services_def++;
		    	if(*fw_services_def!='\0') fw_services_def++;
			
		// compare by using service name
		if(strcmp(fw_serv->name,name)==0)
			return ERR_NO_ERR;
	}

	return ERR_NO_SERV;
}

int ip_type(char *ipstr)
{

	if(strchr(ipstr,'/')) 
		return 3; //all
	else if (strchr(ipstr,'-'))
		return 2; //range
	else
		return 1;  //single

}


/*
 * check current time to determine if now is in schedule or not
 * @return  	0 out schedule time
 *		1 in schedule time 
 */
int check_sche(void)
{
	int ret=ERR_CHECK_SCHE;

	struct tm *check;
	time_t now;
	char *schedule=nvram_safe_get("fw_schedule");
	char *time_daylight=nvram_safe_get("time_daylight");
	int shour, smin, ehour, emin;
	char week[7];
	int now_time=0;
	int start_time=0;
	int end_time=0;
	
	/* setting correct time zone */
       	setenv("TZ",nvram_safe_get("time_zone"),1);
	
	printf("TZ=%s\n",nvram_safe_get("time_zone"));		

	sscanf(schedule,"%[^:]:%d:%d-%d:%d",week,&shour,&smin,&ehour,&emin);
	
	printf("%d:%d\n",shour,smin);
	
	/* find time now */
	now = time(NULL);
	check=localtime(&now);
	/* daylight saving time */
	if(*time_daylight=='1')
		check->tm_hour++;
	
	start_time=shour*60 + smin;
	now_time=check->tm_hour*60 + check->tm_min +1; /* shift 1 min*/
	end_time=ehour*60 +emin;
	
	if(week[check->tm_wday]=='1'){
		if(shour==0 && ehour==24)  //all day
		{
			//puts("schedule set all day");
			ret=1;
		}
		else
		{
			if(start_time > end_time){
				if( (start_time <= now_time) || ( end_time >=now_time))
					ret=1;
				else
					ret=0;
			}else{
				
				if( (start_time <= now_time) && ( end_time >=now_time))
					ret=1;
				else
					ret=0;

			}
		}
	}

	free(schedule);

	return ret;
}


char *getIPAddress(char *ifname)
{
	struct ifreq ifr;
	struct sockaddr_in *saddr;
	int fd;
	char *address=NULL;

	fd=socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
	if(fd==-1)
	{
		perror("socket creating failed");
		return NULL;
	}

	strcpy(ifr.ifr_name, ifname);
	if (ioctl(fd, SIOCGIFADDR, &ifr) == 0)
	{
		saddr = (struct sockaddr_in *)&ifr.ifr_addr;
		address=strdup(inet_ntoa(saddr->sin_addr));
		close(fd);
		return address;
	}
	else
	{
		close(fd);
		return NULL;
	}
			
	if(address!=NULL) free(address);
	return NULL;

}

/* 
 * determine to ACCEPT or BLOCK according to
 * schedule and rule setting 
 */
int action_by_schedule(int schedule, int action)
{

	if(schedule)
	{
		switch(action){
			case ACT_ONLY_BLOCK:
			case ACT_SCHE_BLOCK:
			      return BLOCK;
			default:
			      return ACCEPT;
		}
	}
	else
	{

		switch(action){
			case ACT_ONLY_BLOCK:
			case ACT_SCHE_ALLOW:
			      return BLOCK;
			default:
			      return ACCEPT;
		}
	}
}

