#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "fw_cfg.h"
#include "nvram.h"


#define	FW_BASIC_CONF	"/var/firewall.conf"
#define FW_RULE		"/var/rules"
/* crond will update ddns at 12:00 everyday */
#define CRON_DDNS	"0 12 * * * root /sbin/rc ddns start\n"
#define _BRIDGE_

int create_conf(int schedule);
int create_crontab(void);
int create_rule(int schedule);

/*
 * start firewall protection
 * @return	0 success , others error
 */
int start_firewall(void)
{

	int ret=ERR_NO_ERR;

	/* create configurations */

	ret=create_firewall_cfg();

	/* starting program */
	
	/* TODO: make firewall can be start by module
	 */
	
	system("/sbin/fw-scripts/firewall start");

	return ret;
}

/*
 * stop firewall protection
 * @return	0 success 1 error
 */
int stop_firewall(void)
{

	/* reset crontab to default status */
//	system("/bin/cp /usr/etc/crontab /etc/crontab");
	system("/bin/cp /usr/etc/crontab /var/crontab");
	
	/* stop firewall */
	system("/sbin/fw-scripts/firewall stop");

	return ERR_NO_ERR;
}


/*
 *	schedule format
 *	0123456 00:00-24:00
 *	1111111:hh:mm-hh:mm  
 */
int create_crontab(void)
{
	FILE *fp;
	char *schedule=nvram_safe_get("fw_schedule");
	char *mail_sh_day=nvram_safe_get("mail_sh_day");
	char *mail_sh_hour=nvram_safe_get("mail_sh_hour");
	char *mail_sendlog=nvram_safe_get("mail_sendlog");
	char *time_zone=nvram_safe_get("time_zone");
	char *time_daylight=nvram_safe_get("time_daylight");
	int i;
	int shour, smin, ehour, emin;
	char week[7];
	char tmp[15];

	memset(tmp,'\0',15);
	
	//system("/bin/cp /usr/etc/crontab /etc/crontab"); _CH_DEBUG_
	system("/bin/cp /usr/etc/crontab /var/crontab");
	
	//if((fp=fopen("/etc/crontab","a+"))==NULL)
	if((fp=fopen("/var/crontab","a+"))==NULL)
		return -1;
	/* export timezone variable */
	if(*time_daylight=='1')
		fprintf(fp,"DAYLIGHT=%s\n",time_daylight);
		
	fprintf(fp,"TZ=%s\n\n",time_zone);
		
	/* update ddns every morning 0:00 and update time every morning 0:10 */
	fprintf(fp,"%s",CRON_DDNS);

	if(strcmp(schedule,"")!=0){
		
			
		sscanf(schedule,"%[^:]:%d:%d-%d:%d",week,&shour,&smin,&ehour,&emin);
		
		/* 
		 * special case : when schedule set to all day and everyday, we don't 
		 * have to restart firewall
		 */ 

		if(strcmp(schedule,"1111111:0:0-24:0")==0)
		{
			
			fprintf(fp,"# firewall schedule set to always\n");

		}
		else
		
		/* handle all day condition: There will be only one entry in crontab */
		if(shour==0 && ehour==24)  //all day
		{
			/* time */
			fprintf(fp,"0 0 * * ");
	
			/* week day */
			for(i=0;i<7;i++){
				if(week[i]=='1')
					(strlen(tmp)>0)?sprintf(tmp,"%s,%d",tmp,i):sprintf(tmp,"%d",i);
			}
	
			/* rest part */
			fprintf(fp,"%s root /sbin/rc firewall start\n",tmp);
		}
		else
		{
			/* write schedule entry of start time*/
			fprintf(fp,"%d %d * * ",smin,shour); //min, hour, day, month

			/* dealing with week days */
			for(i=0;i<7;i++){
				if(week[i]=='1')
					(strlen(tmp)>0)?sprintf(tmp,"%s,%d",tmp,i):sprintf(tmp,"%d",i);
			}
			fprintf(fp,"%s root /sbin/rc firewall start\n",tmp);
			/* have to clear used buffer */
			memset(tmp,'\0',15);
			/* write schedule entry of end time*/
			fprintf(fp,"%d %d * * ",emin,ehour); //min, hour, day, month

			/* dealing with week days */
			for(i=0;i<7;i++){
				if(week[i]=='1')
					(strlen(tmp)>0)?sprintf(tmp,"%s,%d",tmp,i):sprintf(tmp,"%d",i);
			}	
			fprintf(fp,"%s root /sbin/rc firewall start\n",tmp);
		}
	}

	/* write schedule entry of mail log*/
	if(*mail_sendlog=='2')  //Send Hourly
		fprintf(fp,"0 * * * * root killall -1 syslogd\n");
	if(*mail_sendlog=='3')  //Send Daily
		fprintf(fp,"0 %s * * * root killall -1 syslogd\n",mail_sh_hour);
	if(*mail_sendlog=='4')  //Send Weekly
		fprintf(fp,"0 %s * * %s root killall -1 syslogd\n",mail_sh_hour,mail_sh_day);

	fclose(fp);
	free(schedule);

	return 0;
}

/*
 * create firewall configuration
 * @return	0 success -1 error
 */
int create_firewall_cfg(void)
{
	int ret=ERR_NO_ERR;
	int schedule;
	char *fw_nat=nvram_safe_get("fw_nat");
	
	/* check schedule */

	schedule=check_sche();
#if 0 
	if(schedule==1)
		puts("in schedule");
	else
		puts("out schedule");
#endif
	create_crontab();

	/* create firewall.conf */
	ret=create_conf(schedule);
	if(ret)
		return ret;
#if 0
	/* if NAT is disabled, no rules 
	 * need to be created
	 */

	if(*fw_nat=='0')
		return ERR_NO_ERR;
#endif
	/* create rules */
	ret=create_rule(schedule);
	if(ret)
		return ret;

	return ERR_NO_ERR;
}

int create_conf(int schedule)
{
	FILE *fp;

	char token=0x01;
	char *fw_nat=nvram_safe_get("fw_nat");
	char *fw_dmz_enable=nvram_safe_get("fw_dmz_enable");
	char *fw_block=nvram_safe_get("fw_block");
	char *fw_block_keyword=nvram_safe_get("fw_block_keyword");
	char *fw_block_trust_enable=nvram_safe_get("fw_block_trust_enable");
	char *fw_response_ping=nvram_safe_get("fw_response_ping");
	char *fw_remote=nvram_safe_get("fw_remote");
	char *fw_spi=nvram_safe_get("fw_spi");

	/* Create firewall.conf */
	if((fp=fopen(FW_BASIC_CONF,"w+"))==NULL)
		return ERR_OPENFILE;
	/* WAN information */
	fprintf(fp,"\n\n#---WAN information---\n");
	fprintf(fp,"WANIF=\"%s\"\n",nvram_safe_get("wan_ifname"));
	fprintf(fp,"EXTERNALIP=\"%s\"\n",getIPAddress(nvram_safe_get("wan_ifname")));
	/* LAN information */
	fprintf(fp,"\n\n#---LAN information---\n");
	
#ifdef _BRIDGE_
	fprintf(fp,"LANIF=\"%s\"\n","br0");
#else
	fprintf(fp,"LANIF=\"%s\"\n",nvram_safe_get("lan_if"));
#endif
	fprintf(fp,"INTERNALIP=\"%s\"\n",nvram_safe_get("lan_ipaddr"));
	fprintf(fp,"LOCALNET=\"%s/%s\"\n",nvram_safe_get("lan_ipaddr"),nvram_safe_get("lan_netmask"));

	/* NAT en/dis-able */
	fprintf(fp,"\n\n#---NAT---\n");
	if(*fw_nat=='1')
		fprintf(fp,"NAT=\"YES\"\n");
	else if(*fw_nat=='0')
	{
		fprintf(fp,"NAT=\"NO\"\n");
	}
	else
	{
		printf(" fw_nat value is invalid \n");
		return ERR_INVALID;
	}

	/* DOS */
	fprintf(fp,"\n\n#---DOS---\n");
	if(*fw_spi=='1')
	{
		fprintf(fp,"DOS_ENABLE=\"YES\"\n");
	}
	else
		fprintf(fp,"DOS_ENABLE=\"NO\"\n");

	/* DMZ */
	fprintf(fp,"\n\n#---DMZ---\n");
	if(*fw_dmz_enable=='1')
	{
		fprintf(fp,"HAVE_DMZ=\"YES\"\n");
		fprintf(fp,"DMZ=\"%s\"\n",nvram_safe_get("fw_dmz"));
	}
	else
		fprintf(fp,"HAVE_DMZ=\"NO\"\n");

	/* Response Ping */
	fprintf(fp,"\n\n#---Response to ping from WAN---\n");
	if(*fw_response_ping=='1')
		fprintf(fp,"RESPONSE_PING=\"YES\"\n");
	else
		fprintf(fp,"RESPONSE_PING=\"NO\"\n");
	/* Content filter */
	fprintf(fp,"\n\n#---Content Filter---\n");

	if(*fw_block=='1'|| (*fw_block=='2'&&schedule==1))
	{
		int i=0;
			fprintf(fp,"CHECK_URL=\"YES\"\n");

		/* replace token with space */
		for(i=0;i<strlen(fw_block_keyword);i++)
		{
			if(*(fw_block_keyword+i)==token)
				*(fw_block_keyword+i)=0x20;
		}

		fprintf(fp,"BAD_STRING=\"%s\"\n",fw_block_keyword);
		if(*fw_block_trust_enable=='1')
			fprintf(fp,"TRUST_IP=\"%s\"\n",nvram_safe_get("fw_block_trust"));


	}
	else
		fprintf(fp,"CHECK_URL=\"NO\"\n");
	free(fw_block_keyword);

	/* Remote Control */
	fprintf(fp,"\n\n#---Remote Control---\n");
	if(*fw_remote=='1')
	{

		char *fw_remote_type=nvram_safe_get("fw_remote_type");

		fprintf(fp,"REMOTE_CONTROL=\"YES\"\n");

		if(*fw_remote_type=='3')
		{
			fprintf(fp,"REMOTE_TYPE=\"3\"\n");
			fprintf(fp,"REMOTE_NET=\"0/0\"\n");
		}
		else if(*fw_remote_type=='2')
		{
			fprintf(fp,"REMOTE_TYPE=\"2\"\n");
			fprintf(fp,"REMOTE_NET=\"%s-%s\"\n",nvram_safe_get("fw_remote_range_start"),nvram_safe_get("fw_remote_range_end"));
		}
		else if(*fw_remote_type=='1'){
			fprintf(fp,"REMOTE_TYPE=\"1\"\n");
			fprintf(fp,"REMOTE_NET=\"%s\"\n",nvram_safe_get("fw_remote_single"));
		}
		fprintf(fp,"REMOTE_PORT=\"%s\"\n",nvram_safe_get("fw_remote_port"));

	}
	else
		fprintf(fp,"REMOTE_CONTROL=\"NO\"\n");
	

        /* ALGs : currently there is no enable/disable function. */
                fprintf(fp,"\n\n#---ALGs---\n");
		fprintf(fp,"MSN=\"YES\"\n");
		fprintf(fp,"Yahoo=\"YES\"\n");
		fprintf(fp,"ICQ=\"YES\"\n");
		fprintf(fp,"Netmeeting=\"YES\"\n");
                                                        
	fclose(fp);

	return ERR_NO_ERR;
}


int create_rule(int schedule)
{

	char *fw_in_rules=nvram_safe_get("fw_in_rules");
	char *fw_out_rules=nvram_safe_get("fw_out_rules");
	FILE *fp;

	/* Create Rule script */
	if((fp=fopen(FW_RULE,"w+"))==NULL)
		return ERR_OPENFILE;

	fprintf(fp,"#!/bin/sh\n\n");
	fprintf(fp,"# ------------------------------------------------------------- #\n");
	fprintf(fp,"# --------- [  INBOUNDS RULES ] ------------------------------- #\n");
	fprintf(fp,"# ------------------------------------------------------------- #\n");

	/* Parsing each rule */
	while(*fw_in_rules!='\0')
	{
		service serv;
		rule	rules;

		sscanf(fw_in_rules,"%d:%[^:]:%d:%[^:]:%[^:]:%d",
		       &(rules.enable),
		       rules.serv.name,
		       &(rules.action),
		       rules.lan,
		       rules.wan,
		       &(rules.log)
		      );

		get_serv(rules.serv.name,&serv);

		/*
		 * if we DNAT to 0.0.0.0, this destination will be an error in 
		 * routing table, change 0.0.0.0 to 1.0.0.0, just a trick to fix
		 * the problem.
		 */
		
		if(strcmp(rules.lan,"0.0.0.0")==0)
			strcpy(rules.lan,"1.0.0.0");
		
		/* If is rule is set to enable, fill the service data type 
		 * with correct information , otherwise, just discard it.
		 */

		if(rules.enable==1)
		{

			switch(ip_type(rules.wan))
				{
				case 1: //one
				case 3: //all
					switch(rules.log){
					case LOG_NEVER:
						break;
					case LOG_ALWAYS:
						if(strcmp(rules.serv.name,"VPN-IPSEC")==0)
						{
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -p 50 -j LOG --log-prefix \"[%s Data match]\"\n",rules.serv.name);
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -p 51 -j LOG --log-prefix \"[%s Data match]\"\n",rules.serv.name);
						}
						if(strcmp(rules.serv.name,"VPN-L2TP")==0)
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -p 115 -j LOG --log-prefix \"[%s Tunnel match]\"\n",rules.serv.name);
						if(strcmp(rules.serv.name,"VPN-PPTP")==0)
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -p 47 -j LOG --log-prefix \"[%s Data match]\"\n",rules.serv.name);
						if(strcmp(serv.type,"both")==0)
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -p tcp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -p udp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
						}
						else
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -p %s --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.type,serv.start_port,serv.end_port,rules.serv.name);
						break;
					case LOG_MATCH:
						if(strcmp(rules.serv.name,"VPN-IPSEC")==0)
						{
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -s %s -d %s -p 50 -j LOG --log-prefix \"[%s Data match]\"\n",rules.wan,rules.lan,rules.serv.name);
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -s %s -d %s -p 51 -j LOG --log-prefix \"[%s Data match]\"\n",rules.wan,rules.lan,rules.serv.name);
						}
						if(strcmp(rules.serv.name,"VPN-L2TP")==0)
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -s %s -d %s -p 115 -j LOG --log-prefix \"[%s Tunnel match]\"\n",rules.wan,rules.lan,rules.serv.name);
						if(strcmp(rules.serv.name,"VPN-PPTP")==0)
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -s %s -d %s -p 47 -j LOG --log-prefix \"[%s Data match]\"\n",rules.wan,rules.lan,rules.serv.name);
						if(strcmp(serv.type,"both")==0)
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -s %s -d %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -s %s -d %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -s %s -d %s -p %s --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.type,serv.start_port,serv.end_port,rules.serv.name);
						break;
					case LOG_NOT_MATCH:
						if(strcmp(rules.serv.name,"VPN-IPSEC")==0)
						{
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -s ! %s -d %s -p 50 -j LOG --log-prefix \"[%s Data not match]\"\n",rules.wan,rules.lan,rules.serv.name);
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -s ! %s -d %s -p 51 -j LOG --log-prefix \"[%s Data not match]\"\n",rules.wan,rules.lan,rules.serv.name);
						}
						if(strcmp(rules.serv.name,"VPN-L2TP")==0)
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -s ! %s -d %s -p 115 -j LOG --log-prefix \"[%s Tunnel not match]\"\n",rules.wan,rules.lan,rules.serv.name);
						if(strcmp(rules.serv.name,"VPN-PPTP")==0)
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -s ! %s -d %s -p 47 -j LOG --log-prefix \"[%s Data not match]\"\n",rules.wan,rules.lan,rules.serv.name);
						if(strcmp(serv.type,"both")==0)
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -s ! %s -d %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -s ! %s -d %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -s ! %s -d %s -p %s --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.type,serv.start_port,serv.end_port,rules.serv.name);
						break;
					}


					/* Do Action according to schedule */

					if(schedule)
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_BLOCK:
							/* Special rule for VPN server */
							if(strcmp(rules.serv.name,"VPN-IPSEC")==0){
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p 50 -j DROP\n",rules.wan, rules.lan);
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p 51 -j DROP\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-L2TP")==0){
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p 115 -j DROP\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-PPTP")==0){
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p 47 -j DROP\n",rules.wan, rules.lan);
							}
						
							if(strcmp(serv.type,"both")==0)
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}else
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p %s --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.type,serv.start_port,serv.end_port);
							break;
						case ACT_ONLY_ALLOW:
						case ACT_SCHE_ALLOW:
							if(strcmp(rules.serv.name,"VPN-IPSEC")==0){
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p 50 -j ACCEPT\n",rules.wan, rules.lan);
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p 51 -j ACCEPT\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-L2TP")==0){
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p 115 -j ACCEPT\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-PPTP")==0){
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p 47 -j ACCEPT\n",rules.wan, rules.lan);
							}						
							if(strcmp(serv.type,"both")==0)
							{	
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else
								fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -s %s -d %s -p %s --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.type,serv.start_port,serv.end_port);
							break;
						}
					}
					else
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_ALLOW:
							/* Special rule for VPN server */
							if(strcmp(rules.serv.name,"VPN-IPSEC")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p 50 -j DROP\n",rules.wan, rules.lan);
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p 51 -j DROP\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-L2TP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p 115 -j DROP\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-PPTP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p 47 -j DROP\n",rules.wan, rules.lan);
							}						
						
							if(strcmp(serv.type,"both")==0)
							{
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p %s --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.type,serv.start_port,serv.end_port);
							break;
						case ACT_SCHE_BLOCK:
						case ACT_ONLY_ALLOW:
							if(strcmp(rules.serv.name,"VPN-IPSEC")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p 50 -j ACCEPT\n",rules.wan, rules.lan);
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p 51 -j ACCEPT\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-L2TP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p 115 -j ACCEPT\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-PPTP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s %s -d %s -p 47 -j ACCEPT\n",rules.wan, rules.lan);
							}						
							if(strcmp(serv.type,"both")==0)
							{
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s  %s -d %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s  %s -d %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -s  %s -d %s -p %s --dport %d:%d -j ACCEPT\n",rules.wan,rules.lan,serv.type,serv.start_port,serv.end_port);
							break;
						}
					}//end of in schedule/out schedule
					break;
				case 2: //range
					switch(rules.log){
					case LOG_NEVER:
						break;
					case LOG_ALWAYS:
						if(strcmp(rules.serv.name,"VPN-IPSEC")==0)
						{
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -p 50 -j LOG --log-prefix \"[%s Data match]\"\n",rules.serv.name);
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -p 51 -j LOG --log-prefix \"[%s Data match]\"\n",rules.serv.name);
						}
						if(strcmp(rules.serv.name,"VPN-L2TP")==0)
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -p 115 -j LOG --log-prefix \"[%s Tunnel match]\"\n",rules.serv.name);
						if(strcmp(rules.serv.name,"VPN-PPTP")==0)
								fprintf(fp,"$IPTABLES -I FORWARD -i $WANIF -p 47 -j LOG --log-prefix \"[%s Data match]\"\n",rules.serv.name);
						if(strcmp(serv.type,"both")==0)
							{
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -p tcp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -p udp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
							}
						else
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -p %s --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.type,serv.start_port,serv.end_port,rules.serv.name);
						break;
					case LOG_MATCH:
						if(strcmp(rules.serv.name,"VPN-IPSEC")==0)
						{
								fprintf(fp,"$IPTABLES -I FORWARD -m iprange --src-range %s -d %s -i $WANIF -p 50 -j LOG --log-prefix \"[%s Data match]\"\n",rules.wan,rules.lan,rules.serv.name);
								fprintf(fp,"$IPTABLES -I FORWARD -m iprange --src-range %s -d %s -i $WANIF -p 51 -j LOG --log-prefix \"[%s Data match]\"\n",rules.wan,rules.lan,rules.serv.name);
						}
						if(strcmp(rules.serv.name,"VPN-L2TP")==0)
								fprintf(fp,"$IPTABLES -I FORWARD -m iprange --src-range %s -d %s -i $WANIF -p 115 -j LOG --log-prefix \"[%s Tunnel match]\"\n",rules.wan,rules.lan,rules.serv.name);
						if(strcmp(rules.serv.name,"VPN-PPTP")==0)
								fprintf(fp,"$IPTABLES -I FORWARD -m iprange --src-range %s -d %s -i $WANIF -p 47 -j LOG --log-prefix \"[%s Data match]\"\n",rules.wan,rules.lan,rules.serv.name);
						if(strcmp(serv.type,"both")==0)
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -m iprange --src-range %s -d %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -m iprange --src-range %s -d %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}else
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -m iprange --src-range %s -d %s -p %s --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.type,serv.start_port,serv.end_port,rules.serv.name);
						break;
					case LOG_NOT_MATCH:
						if(strcmp(rules.serv.name,"VPN-IPSEC")==0)
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m iprange ! --src-range %s -d %s -i $WANIF -p 50 -j LOG --log-prefix \"[%s Data not match]\"\n",rules.wan,rules.lan,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m iprange ! --src-range %s -d %s -i $WANIF -p 51 -j LOG --log-prefix \"[%s Data not match]\"\n",rules.wan,rules.lan,rules.serv.name);
						}
						if(strcmp(rules.serv.name,"VPN-L2TP")==0)
							fprintf(fp,"$IPTABLES -I FORWARD -m iprange ! --src-range %s -d %s -i $WANIF -p 115 -j LOG --log-prefix \"[%s Tunnel not match]\"\n",rules.wan,rules.lan,rules.serv.name);
						if(strcmp(rules.serv.name,"VPN-PPTP")==0)
							fprintf(fp,"$IPTABLES -I FORWARD -m iprange ! --src-range %s -d %s -i $WANIF -p 47 -j LOG --log-prefix \"[%s Data not match]\"\n",rules.wan,rules.lan,rules.serv.name);
						if(strcmp(serv.type,"both")==0)
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -m iprange ! --src-range %s -d %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -m iprange ! --src-range %s -d %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $WANIF -m iprange ! --src-range %s -d %s -p %s --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.type,serv.start_port,serv.end_port,rules.serv.name);
						break;
					}

					/* Do Action according to schedule */

					if(schedule)
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_BLOCK:
							if(strcmp(rules.serv.name,"VPN-IPSEC")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 50 -j DROP\n",rules.wan, rules.lan);
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 51 -j DROP\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-L2TP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 115 -j DROP\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-PPTP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 47 -j DROP\n",rules.wan, rules.lan);
							}						
						if(strcmp(serv.type,"both")==0)
						{
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
						}else
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p %s --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.type,serv.start_port,serv.end_port);
							break;
						case ACT_ONLY_ALLOW:
						case ACT_SCHE_ALLOW:
							if(strcmp(rules.serv.name,"VPN-IPSEC")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 50 -j ACCEPT\n",rules.wan, rules.lan);
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 51 -j ACCEPT\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-L2TP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 115 -j ACCEPT\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-PPTP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 47 -j ACCEPT\n",rules.wan, rules.lan);
							}
						if(strcmp(serv.type,"both")==0)
						{
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
						}
						else
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p %s --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.type,serv.start_port,serv.end_port);
							break;
						}
					}
					else
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_ALLOW:
							if(strcmp(rules.serv.name,"VPN-IPSEC")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 50 -j DROP\n",rules.wan, rules.lan);
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 51 -j DROP\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-L2TP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 115 -j DROP\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-PPTP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 47 -j DROP\n",rules.wan, rules.lan);
							}	
						if(strcmp(serv.type,"both")==0)
						{
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
						}else
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p %s --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.type,serv.start_port,serv.end_port);
							break;
						case ACT_SCHE_BLOCK:
						case ACT_ONLY_ALLOW:
							if(strcmp(rules.serv.name,"VPN-IPSEC")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 50 -j ACCEPT\n",rules.wan, rules.lan);
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 51 -j ACCEPT\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-L2TP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 115 -j ACCEPT\n",rules.wan, rules.lan);
							}
							if(strcmp(rules.serv.name,"VPN-PPTP")==0){
								fprintf(fp,"$IPTABLES -i $WANIF -A FORWARD -m iprange --src-range %s -d %s -p 47 -j ACCEPT\n",rules.wan, rules.lan);
							}						
						if(strcmp(serv.type,"both")==0)
						{
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
						}else
							fprintf(fp,"$IPTABLES -A FORWARD -i $WANIF -m iprange --src-range %s -d %s -p %s --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.type,serv.start_port,serv.end_port);
							break;
						}
					}//end of in schedule/out schedule
					break;
				} //end of switch
		

				/*
				 *  CREATE PREROUTING RULES 
				 */

		                 /* Special rules for VPN server */
                        	if(strcmp(rules.serv.name,"VPN-IPSEC")==0)
                        	{
					//esp
					fprintf(fp,"$IPTABLES -t nat -A PREROUTING -i $WANIF -p 50 -j DNAT --to %s\n",rules.lan);
				}
					
                        	if(strcmp(rules.serv.name,"VPN-L2TP")==0)
                        	{
					
                        		//L2TP (proto 115)
                        		fprintf(fp,"$IPTABLES -t nat -A PREROUTING -i $WANIF -p 115 -j DNAT --to %s\n",rules.lan);
				}
                      
                        	if(strcmp(rules.serv.name,"VPN-PPTP")==0)
                        	{
                    			//GRE (proto 47)
                        		//fprintf(fp,"$IPTABLES -t nat -A PREROUTING -i $WANIF -p 47 -j DNAT --to %s\n",rules.lan);
                        	}
                        
                        	if(strcmp(serv.type,"both")==0)
                        	{
                        		fprintf(fp,"$IPTABLES -t nat -A PREROUTING -i $WANIF -p tcp --dport %d:%d -j DNAT --to %s:%d-%d\n",serv.start_port,serv.end_port,rules.lan ,serv.start_port,serv.end_port);
                        		fprintf(fp,"$IPTABLES -t nat -A PREROUTING -i $WANIF -p udp --dport %d:%d -j DNAT --to %s:%d-%d\n",serv.start_port,serv.end_port,rules.lan ,serv.start_port,serv.end_port);
                        	}
                        	else
                        		fprintf(fp,"$IPTABLES -t nat -A PREROUTING -i $WANIF -p %s --dport %d:%d -j DNAT --to %s:%d-%d\n",serv.type,serv.start_port,serv.end_port,rules.lan ,serv.start_port,serv.end_port);

			fprintf(fp,"SYSLOG \"Forwarding traffic %s %s %s:%d-%d -> %s:%d-%d\"\n",rules.serv.name,serv.type,rules.wan,serv.start_port,serv.end_port,rules.lan,serv.start_port,serv.end_port);
		}// end of formating one rule
		while(*fw_in_rules!='\0' && *fw_in_rules!=0x01) fw_in_rules++;
		if(*fw_in_rules!='\0') fw_in_rules++;
	} //end of for loop

	fprintf(fp,"# ------------------------------------------------------------- #\n");
	fprintf(fp,"# --------- [  OUTBOUNDS ] ------------------------------------ #\n");
	fprintf(fp,"# ------------------------------------------------------------- #\n");


	/* Parsing each rule */
	while(*fw_out_rules!='\0')
	{
		service serv;
		rule	rules;

		sscanf(fw_out_rules,"%d:%[^:]:%d:%[^:]:%[^:]:%d",
		       &(rules.enable),
		       rules.serv.name,
		       &(rules.action),
		       rules.lan,
		       rules.wan,
		       &(rules.log)
		      );

		get_serv(rules.serv.name,&serv);

		/*
		 *  Creating rule only when rule is set to enable
		 */  
		
		if(rules.enable)
		{

			if(strcmp(serv.type,"both")==0)
			{

				switch(ip_type(rules.wan))
				{
				case 1: //one
				case 3: //all
					switch(rules.log){
					case LOG_NEVER:
						break;
					case LOG_ALWAYS:
						fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -p tcp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
						fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -p udp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
						break;
					case LOG_MATCH:
						if(ip_type(rules.lan)!=2)  //all single
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -s %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -s %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else //range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -m iprange --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -m iprange --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						break;
					case LOG_NOT_MATCH:
						if(ip_type(rules.lan)!=2)  //all single
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -s ! %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -s ! %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d ! %s -s %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d ! %s -s %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else //range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -m iprange ! --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -m iprange ! --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d ! %s -m iprange --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d ! %s -m iprange --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						break;
					}

					/* Do Action according to schedule */
					if(schedule)
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_BLOCK:
							if(ip_type(rules.lan)!=2)  // lan not range, wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);				
							}
							else	// lan range , wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							break;			
						case ACT_ONLY_ALLOW:
						case ACT_SCHE_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);				
							}
							else	// lan range , wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p tcp --dport %d:%d -j ACCPET\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							break;
						}
					}
					else
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);				
							}
							else	// lan range , wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}			
							break;
						case ACT_SCHE_BLOCK:
						case ACT_ONLY_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);				
							}
							else	// lan range , wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p tcp --dport %d:%d -j ACCPET\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}			
							break;
						}
					}//end of in schedule/out schedule
					break;
				case 2: //range
					switch(rules.log){
					case LOG_NEVER:
						break;
					case LOG_ALWAYS:
						fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -p tcp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
						fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -p udp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
						break;
					case LOG_MATCH:
						if(ip_type(rules.lan)!=2)  // lan not range, wan range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -s %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -s %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else // lan wan range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						break;
					case LOG_NOT_MATCH:

						if(ip_type(rules.lan)!=2)  // lan not range, wan range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -s ! %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -s ! %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange ! --dst-range %s -s %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange ! --dst-range %s -s %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else // lan wan range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange ! --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange ! --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -m iprange ! --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -m iprange ! --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						break;
					}

					/* Do Action according to schedule */
					if(schedule)
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_BLOCK:
							if(ip_type(rules.lan)!=2)  // lan not range, wan range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else //wan lan range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							break;
						case ACT_ONLY_ALLOW:
						case ACT_SCHE_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else //wan lan range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							break;
						}
					}
					else
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							break;
						case ACT_SCHE_BLOCK:
						case ACT_ONLY_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							break;
						}
					}//end of in schedule/out schedule

					break;


				} //end of switch
			}//both
			else
				if(strcmp(serv.type,"tcp")==0){

					switch(ip_type(rules.wan))
					{
					case 1: //one
					case 3: //all
						switch(rules.log){
						case LOG_NEVER:
							break;
						case LOG_ALWAYS:
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -p tcp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
							break;
						case LOG_MATCH:
							if(ip_type(rules.lan)!=2)  //all single
							{
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -s %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							}
							else //range
							{
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -m iprange --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							}
							break;
						case LOG_NOT_MATCH:
							if(ip_type(rules.lan)!=2)  //all single
							{
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -s ! %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d ! %s -s %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							}
							else //range
							{
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -m iprange ! --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d ! %s -m iprange --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							}
							break;
						}


						/* Do Action according to schedule */

						if(schedule)
						{
							switch(rules.action){
							case ACT_ONLY_BLOCK:
							case ACT_SCHE_BLOCK:
								if(ip_type(rules.lan)!=2)  // lan not range, wan not range
									{
										fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
									}
                                                                	else //lan range, wan not range
									{
										fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
									}
								break;
							case ACT_ONLY_ALLOW:
							case ACT_SCHE_ALLOW:
								if(ip_type(rules.lan)!=2)  // lan not range, wan not range
									{
										fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
                                                                            	}
                                                                            	else //lan range, wan not range
                                                                            	{
                                                                            		fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								            	}
								break;
							}
						}
						else
						{
							switch(rules.action){
							case ACT_ONLY_BLOCK:
							case ACT_SCHE_ALLOW:
								if(ip_type(rules.lan)!=2)  // lan not range, wan not range
									{
										fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							         	}
									else //lan range, wan not range
									{
										fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
            								}								
									break;
							case ACT_SCHE_BLOCK:
							case ACT_ONLY_ALLOW:
								if(ip_type(rules.lan)!=2)  // lan not range, wan not range
									{
										fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							         	}
									else //lan range, wan not range
									{
										fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);									
            								}																
									break;							
							}
						}//end of in schedule/out schedule
						break;
					case 2: //wan range
						switch(rules.log){
						case LOG_NEVER:
							break;
						case LOG_ALWAYS:
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -p tcp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
							break;
						case LOG_MATCH:
							if(ip_type(rules.lan)!=2)  // lan not range, wan range
							{
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -s %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							}
							else // lan wan range
							{
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							}
							break;
						case LOG_NOT_MATCH:

							if(ip_type(rules.lan)!=2)  // lan not range, wan range
							{
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -s ! %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange ! --dst-range %s -s %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							}
							else // lan wan range
							{
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -m iprange ! --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
								fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange ! --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							}


							break;
						}

						/* Do Action according to schedule */

						if(schedule)
						{
							switch(rules.action){
							case ACT_ONLY_BLOCK:
							case ACT_SCHE_BLOCK:
								if(ip_type(rules.lan)!=2)  // lan not range, wan range
								{
									fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								}
								else // lan range , wan range
								{
									fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								}
								break;
							case ACT_ONLY_ALLOW:
							case ACT_SCHE_ALLOW:
								if(ip_type(rules.lan)!=2)  // lan not range, wan range
								{
									fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								}
								else // lan range, wan range 
								{
									fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								}
								break;
							}
						}
						else
						{
							switch(rules.action){
							case ACT_ONLY_BLOCK:
							case ACT_SCHE_ALLOW:
								if(ip_type(rules.lan)!=2)  // lan not range, wan range
								{
									fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								}
								else
								{
									fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								}
								break;
							case ACT_SCHE_BLOCK:
							case ACT_ONLY_ALLOW:
								if(ip_type(rules.lan)!=2)  // lan not range, wan range
								{
									fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								}
								else // lan range, wan range
								{
									fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p tcp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
								}
								break;
							}
						}//end of in schedule/out schedule
						break;
					} //end of switch
				}  //end of tcp
			if(strcmp(serv.type,"udp")==0)
			{

				switch(ip_type(rules.wan))
				{
				case 1: //one
				case 3: //all
					switch(rules.log){
					case LOG_NEVER:
						break;
					case LOG_ALWAYS:
						fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -p udp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
						break;
					case LOG_MATCH:
						if(ip_type(rules.lan)!=2)  //all single
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -s %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else //range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -m iprange --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						break;
					case LOG_NOT_MATCH:
						if(ip_type(rules.lan)!=2)  //all single
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -s ! %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d ! %s -s %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else //range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d %s -m iprange ! --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -d ! %s -m iprange --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule not match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						break;
					}


					/* Do Action according to schedule */

					if(schedule)
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_BLOCK:
							if(ip_type(rules.lan)!=2)  // lan not range, wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else // lan range, wan not range		
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);							
							}			
							break;
						case ACT_ONLY_ALLOW:
						case ACT_SCHE_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else // lan range, wan not range		
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);							
							}			
							break;							
						}
					}
					else
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else // lan range, wan not range		
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);							
							}			
							break;
						case ACT_SCHE_BLOCK:
						case ACT_ONLY_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan not range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -s %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else // lan range, wan not range		
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -d %s -m iprange --src-range %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);							
							}										
							break;
						}
					}//end of in schedule/out schedule
					break;
				case 2: //range
					switch(rules.log){
					case LOG_NEVER:
						break;
					case LOG_ALWAYS:
						fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -p udp --dport %d:%d -j LOG --log-prefix \"[%s match]\"\n",serv.start_port,serv.end_port,rules.serv.name);
						break;
					case LOG_MATCH:
						if(ip_type(rules.lan)!=2)  // lan not range, wan range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -s %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else // lan wan range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						break;
					case LOG_NOT_MATCH:

						if(ip_type(rules.lan)!=2)  // lan not range, wan range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -s ! %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange ! --dst-range %s -s %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}
						else // lan wan range
						{
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange --dst-range %s -m iprange ! --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
							fprintf(fp,"$IPTABLES -I FORWARD -m state --state NEW -i $LANIF -m iprange ! --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j LOG --log-prefix \"[%s rule match]\"\n",rules.wan,rules.lan,serv.start_port,serv.end_port,rules.serv.name);
						}


						break;
					}

					/* Do Action according to schedule */

					if(schedule)
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_BLOCK:
							if(ip_type(rules.lan)!=2)  // lan not range, wan range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							break;
						case ACT_ONLY_ALLOW:
						case ACT_SCHE_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j ACCEPT\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							break;
						}
					}
					else
					{
						switch(rules.action){
						case ACT_ONLY_BLOCK:
						case ACT_SCHE_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							break;
						case ACT_SCHE_BLOCK:
						case ACT_ONLY_ALLOW:
							if(ip_type(rules.lan)!=2)  // lan not range, wan range
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -s %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							else
							{
								fprintf(fp,"$IPTABLES -A FORWARD -i $LANIF -m iprange --dst-range %s -m iprange --src-range %s -p udp --dport %d:%d -j DROP\n",rules.wan, rules.lan,serv.start_port,serv.end_port);
							}
							break;
						}
					}//end of in schedule/out schedule
					break;
				} //end of switch
			}
			fprintf(fp,"SYSLOG \"Forwarding traffic %s %s %s:%d-%d -> %s:%d-%d\"\n",rules.serv.name,serv.type,rules.wan,serv.start_port,serv.end_port,rules.lan,serv.start_port,serv.end_port);
		}// end of formating one rule
		while(*fw_out_rules!='\0' && *fw_out_rules!=0x01) fw_out_rules++;
		if(*fw_out_rules!='\0') fw_out_rules++;
	} //end of for loop

	fclose(fp);
	chmod("/var/rules",0755);

	return ERR_NO_ERR;
}

