#ifndef _FIREWALL_
#define _FIREWALL_


typedef struct service
{
	char	name[15];	
	char	type[5];	// UDP TCP BOTH
	int 	start_port;	// 1~65535
	int 	end_port;	// 1~65535
}service;



typedef struct rule
{
	int 	enable;
	service	serv;
	int 	action;			
	char	lan[35];	//any: 0/0 single: a.b.c.d range: 1.2.3.4-5.6.7.8
	char	wan[35];	//any: 0/0 single: a.b.c.d range: 1.2.3.4-5.6.7.8
	int	log;
}rule;



#define ERR_NO_ERR	0
#define ERR_INVALID	-1
#define ERR_CHECK_SCHE	-2
#define ERR_OPENFILE	-3
#define ERR_NO_SERV	-4
	

#define ACT_ONLY_BLOCK	0
#define ACT_SCHE_BLOCK	1
#define ACT_ONLY_ALLOW	2
#define ACT_SCHE_ALLOW	3

#define LOG_NEVER	0	
#define LOG_ALWAYS	1
#define	LOG_MATCH	2
#define LOG_NOT_MATCH	3

/*
 * start firewall protection
 * @return	0 success otherwise error
 */
int start_firewall(void);

/*
 * stop firewall protection
 * @return	0 success otherwise error
 */
int stop_firewall(void);

/*
 * create firewall configuration
 * @return	0 success otherwise error
 */
int create_firewall_cfg(void);

/* 
 * Decide which type of ip address it is.
 * return 3(all)/2(range)/1(single)
 */
int ip_type(char *ipstr);

/* 
 * get service according to service name
 * @return  	success  0, otherwise error
 */
int get_serv(char *name, service *fw_serv);

/* 
 * create crontab for crond to re-run firewall rule 
 * @return  	success  0, otherwise error
 */
int create_crontab(void);

/* 
 * check current time to determine if now is in schedule or not
 * @return  	0 out schedule time
 *		1 in schedule time 
 *		otherwise error
 */
int check_sche(void);

/* 
 * Get IP address from an interface name 
 * @return ip address of interface
 * 
 */
char *getIPAddress(char *ifname);

#endif
