#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <signal.h>
#include <stdint.h>
#include <linux/limits.h>
#include <time.h>
#include "nvram.h"
#include "apps.h"
#include "socket_tools.h"

#define RC_ERR		-1
#define RC_OK		0
#define RC_UNKNOWN	1

int SYSTEM(const char *format, ...)
{
	static char buf[4096]="";
	va_list arg;

        memset(buf,0,4096);
	va_start(arg, format);
	vsnprintf(buf,4096, format, arg);
	va_end(arg);
     //glb_dbg("SYSTEM command is %s", buf);
      // sprintf(buf+strlen(buf)," 2>/dev/null");
	system(buf);
	usleep(1);
	return RC_OK;
}
#define BUFSZ PIPE_BUF 
void myPipe(char *command, char **output)
{
  FILE *fp;
  char buf[BUFSZ];
  int len;
  
  *output=malloc(1);
  strcpy(*output, "");
  if((fp=popen(command, "r"))==NULL)
     return;
  while((fgets(buf, BUFSZ, fp)) != NULL){
     len=strlen(*output)+strlen(buf);	  
     if((*output=realloc(*output, (sizeof(char) * (len+1))))==NULL)
	return;
      strcat(*output, buf);
  }
  pclose(fp);
}


int rc_init(void)
{
	/* get nvram from flash */
	if(nvram_load()!=0){
		SYSTEM("/bin/cp %s %s\n",NVRAM_DEFAULT,NVRAM_TMP_PATH);
		nvram_commit();
	}

	/* if restore_default == 1 than restore to default */
	if(*nvram_get("restore_default")=='1'){
		SYSTEM("/bin/busybox cp %s %s",NVRAM_DEFAULT,NVRAM_TMP_PATH);
		nvram_commit();
	}
	
	/* upgrade default vaule */
	if(*nvram_safe_get("upgrade_default")=='1'){
		char *def=nvram_get_fun("fw_services_def",NVRAM_DEFAULT)?:"";
		nvram_set("fw_services_def",def);
		nvram_set("upgrade_default","0");
		nvram_commit();
	}
	init_lan();
	start_wireless();
#ifdef WIFI_ENABLE
	/* check wireless */
	//start_wireless();
	if(0)//wireless()==0)
		nvram_set("wifi_present","1");
	else
		nvram_set("wifi_present","0");
#else
	nvram_set("wifi_present","0");
#endif
	return 0;
}

void rc_usage(){
	puts("\nUsage: rc [wan|service|ap name] [start|stop|restart|create]\n");
	exit(0);
}

/* if ap_name is NULL than start all */
int rc_start(apfun_tab *ap_tab,char *ap_name)
{
	apfun_tab *ap;
//printf("rc_start\n");
	for(ap=ap_tab;ap->name;ap++) {
		if(!ap->startfun)
	       	continue;
		if(ap_name!=NULL && strcmp(ap_name,ap->name)!=0)
			continue;
//printf("apfun_tab->name=%s\n",ap->name);
		if(((*ap->startfun)())<0)
	       		return RC_ERR;   
		if(ap_name)
			return RC_OK;
	        usleep(1);	
	}		
	return (!ap_name || ap->name)?RC_OK:RC_UNKNOWN;
	
}
/* if ap_name is NULL than stop all */
int rc_stop(apfun_tab *ap_tab,char *ap_name)
{
	apfun_tab *ap;
	for(ap=ap_tab;ap->name;ap++) {
		if(!ap->stopfun)
	       	continue;
		if(ap_name!=NULL && strcmp(ap_name,ap->name)!=0)
			continue;
		if(((*ap->stopfun)())<0)
	       		return RC_ERR;    
		if(ap_name)
			return RC_OK;
	        usleep(1);	
	}	
	return (!ap_name || ap->name)?RC_OK:RC_UNKNOWN;
}

int rc_create(apfun_tab *ap_tab ,char *ap_name)
{
	apfun_tab *ap;
	for(ap=ap_tab;ap->name;ap++) {
	        if(!ap->createfun)
       		continue;
		if(ap_name!=NULL && strcmp(ap->name,ap_name)!=0)
				continue;
		if(((*ap->createfun)())<0)
	       		return RC_ERR;    
		if(ap_name)
			return RC_OK;
	        usleep(1);	
	}		
	return (!ap_name || ap->name)?RC_OK:RC_UNKNOWN;
}

int rc_action(apfun_tab *ap_tab ,char *ap_name,char *action)
{
	apfun_tab *ap=ap_tab;
	
	if (strcmp(action,"start")==0)
		return rc_start(ap,ap_name);

	if (strcmp(action,"stop")==0)
		return rc_stop(ap,ap_name);
	
	if (strcmp(action,"restart")==0)		
		return rc_stop(ap,ap_name) & rc_start(ap,ap_name);
	
	if (strcmp(action,"create")==0)
		return rc_create(ap,ap_name);
		
	rc_usage();

	return RC_OK;

}

int main(int argc,char *argv[])
{
	int index=1;
	apfun_tab *ap;
	char *ap_name=NULL;
	char *action=NULL;

	
	if(argc < 2) 
		rc_usage();
#if 1	
	while(index < argc){
		if(strstr(argv[index-1],"rc"))
			break;
		index++;
	}
#endif
	/* initial */
	setenv("PATH","/bin:/sbin:/usr/sbin:/usr/bin:/var/airgo/usr/sbin",1);

	if(access(NVRAM_TMP_PATH,F_OK) < 0){
		if(rc_init()<0){
			puts("nvram error!!\n");
			exit(1);
		}
	}
	//start_wireless();	
	/* wan function */	
	if(strcmp(argv[index],"wan")==0){
		if(argv[index+1]==NULL)
			return -1;
		/* frist use start */
		if(strcmp(argv[index+1],"start")==0){
			return start_wan(1);
		}else if (strcmp(argv[index+1],"stop")==0){
			return stop_wan();
		}else if (strcmp(argv[index+1],"restart")==0){
			return stop_wan() & start_wan(0);
		}else
			rc_usage();
			
	}
	
#if 0
	/*samba function*/
	if(strcmp(argv[index],"samba")==0){
	    if(argv[index+1]==NULL)
			return -1;
		/* frist use start */
		if(strcmp(argv[index+1],"start")==0){
			return start_smbd();
		}else if (strcmp(argv[index+1],"stop")==0){
			return stop_smbd();
		}else if (strcmp(argv[index+1],"restart")==0){
			return stop_smbd() & start_smbd(0);
		}else
			rc_usage();
			
	}
#endif
	
	/* special command */
	/*
        if (strcmp(argv[index],"list")==0)			
		return wlan_socket_cmd("STALIST");
	if(strcmp(argv[index],"fcc")==0)
		return wlan_socket_cmd(argv[index+1]);	
        */	
	ap=ap_fun_tab;
	//printf("ap_name=%s action=%s\n",argv[index],action=argv[index+1]);
	if((strcmp(argv[index],"start")==0) 
			||(strcmp(argv[index],"stop")==0) 
			||(strcmp(argv[index],"restart")==0) 
			||(strcmp(argv[index],"create")==0)){
		ap_name=NULL;
		action=argv[index];
		if(action==NULL || argv[index+1])
			rc_usage();
	}else if((strcmp(argv[index],"service")==0)){
		ap=wan_fun_tab;
		ap_name=NULL;
		action=argv[index+1];
		if(action==NULL || argv[index+2])
			rc_usage();
	}else{
		ap_name=argv[index];
		action=argv[index+1];
		if(ap_name==NULL || action==NULL || argv[index+2])
			rc_usage();
	}
			
	//printf("ap_name=%s action=%s\n",ap_name,action);	
	/* if can't find application in ap_fun_tab than find in wan_fun_tab*/
	if(rc_action(ap,ap_name,action)==RC_UNKNOWN){
		ap=wan_fun_tab;
		if(rc_action(ap,ap_name,action)==RC_UNKNOWN)
			puts("can't find this application");
	}
		
	return 0;
}
