#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "rt_rip.h"
#include "rt_util.h"
#include "nvram.h"
#include "rc.h"

#define ZEBRA "/sbin/zebra"
#define RIPD "/sbin/ripd"


int create_rip_daemon_cfg(void)
{
  FILE *fp;
  char *dir,*ver;
  char *ptr;
  char *fw_nat=nvram_safe_get("fw_nat");
  
  if((fp=fopen(ZEBRA_CONF,"w+"))==NULL)
     return -1;

  fprintf(fp,"hostname router\n");
  fprintf(fp,"password zebra\n");
  fprintf(fp,"interface %s\n",nvram_safe_get("lan_if"));
  ptr=nvram_safe_get("rt_rip_version");
  if(*ptr=='2'){
  	if(*(ptr+1)=='M')
  		fprintf(fp,"\tmulticast\n");
  	if(*(ptr+1)=='B')
  		fprintf(fp,"\tno multicast\n");
  }
	fclose(fp);		
  

  if((fp=fopen(RIPD_CONF,"w+"))==NULL)
     return -1;


  fprintf(fp,"hostname rip\n");
  fprintf(fp,"password zebra\n");
  ver=nvram_safe_get("rt_rip_version");
  dir=nvram_safe_get("rt_rip_direction");

  if (*fw_nat=='0')
  {
	fprintf(fp,"interface %s\n",nvram_safe_get("wan_if"));

  	if(*dir!='o')
		fprintf(fp,"\tip rip receive version 1 2\n");
  }
  else
  {
	fprintf(fp,"interface %s\n",nvram_safe_get("lan_if"));
		fprintf(fp,"\tip rip receive version 1 2\n");
  }
  
  fprintf(fp,"router rip\n");	
  if(*ver=='1')
  	fprintf(fp,"version 1\n");	
  else
  	fprintf(fp,"version 2\n");	
  if (*fw_nat=='0')
  {
  	fprintf(fp,"\tnetwork %s/%s\n"
			  ,nvram_safe_get("wan_ipaddr")
		  	  ,nvram_safe_get("wan_netmask"));
 	fprintf(fp,"\tnetwork %s\n",nvram_safe_get("wan_if"));
  }
  else
  {
	fprintf(fp,"\tnetwork %s/%s\n"
		  ,nvram_safe_get("lan_ipaddr")
		  ,nvram_safe_get("lan_netmask"));
	fprintf(fp,"\tnetwork %s\n",nvram_safe_get("lan_if"));
  }
  if(*dir=='i')  //in
	fprintf(fp,"no redistribute kernel\n");
  else
	fprintf(fp,"redistribute kernel\n");
		
	fclose(fp);
	
  free(dir);
  free(ver);
  free(ptr);
	
  return 0;
}

/*
 * start rip daemon 
 * @return	0 success -1 error
 */
int start_rip_daemon(void)
{
  char *dir;
  char tmp[3]="";
  

  /* check rip daemon direction is specified or not */
  
  dir=nvram_safe_get("rt_rip_direction");
  if(*dir=='0')
    return 0;
  else if(*dir=='o')
	strcpy(tmp,"-r");

  create_rip_daemon_cfg();
  system("/sbin/zebra -d -f /etc/zebra.conf");
  SYSTEM("/sbin/ripd %s -d -f /etc/ripd.conf",tmp);


  return 0;
}

/*
 * stop rip daemon 
 * @return	0 success -1 error
 */
int stop_rip_daemon(void)
{
  system("killall ripd");
  system("killall zebra");
  return 0;
}



