#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdarg.h>
#include "rc.h"
#include "default_cfg.h"
#include "nvram.h"
#define LEASES_TIME 60*60*24*3

struct leases_t {
	unsigned char chaddr[16];
	u_int32_t yiaddr;
	u_int32_t expires;
};

void change_lease(void)
{
	FILE *fp;
	struct leases_t lease;
	char *lan_ip=nvram_get("lan_ipaddr");
	struct in_addr addr;
	
	inet_aton(lan_ip,&addr);
		
	if((fp=fopen("/var/udhcpd.leases","r+"))==NULL){
		fp=fopen("/var/udhcpd.leases","w");
		fclose(fp);
		return;
	}
	fread(&lease,sizeof(struct leases_t),1,fp);
	fseek(fp,0,0);
	fclose(fp);
	
	if(lease.yiaddr!=0){
		if((addr.s_addr & 0x00ffffff) != (lease.yiaddr & 0x00ffffff)){
			fp=fopen("/var/udhcpd.leases","w");
			fclose(fp);	
		}
	}

}

int search_leases(unsigned char *mac, u_int32_t ipaddr)
{
	FILE *fp;
	struct leases_t lease[254];
	int i=0;
	int found=0;
	char *lan_ip=nvram_get("lan_ipaddr");
	struct in_addr addr;
	
	inet_aton(lan_ip,&addr);
	
	ipaddr=(addr.s_addr & 0xff) | (ipaddr & ~0xff);	
		
	if((fp=fopen("/var/udhcpd.leases","r+"))==NULL)
		return -1;
	fread(lease,sizeof(struct leases_t),254,fp);
	fseek(fp,0,0);
	while(lease[i].yiaddr!=0 && i<=254){
		if(ipaddr==lease[i].yiaddr || addr.s_addr==lease[i].yiaddr){
			lease[i].expires=0;
		}
		if(memcmp(mac,lease[i].chaddr,16)!=0){
			fwrite(&lease[i++],sizeof(struct leases_t),1,fp);
		}else{
			found=1;
			lease[i].yiaddr=ipaddr;
			lease[i].expires=htonl(31536000);
			fwrite(&lease[i++],sizeof(struct leases_t),1,fp);
		}
	}
	if(found==0){
		lease[i].yiaddr=ipaddr;
		memcpy(lease[i].chaddr,mac,16);
		lease[i].expires=htonl(31536000);
		fwrite(&lease[i],sizeof(struct leases_t),1,fp);
	}
	fclose(fp);
	return 0;
}

int dhcp_write_leases(char *reserve)
{
	//FILE *fp;
	struct leases_t leases;
	struct in_addr addr;
	struct in_addr addr_s;
	struct in_addr addr_e;
	
	u_int8_t macaddr;
	
	char mac[32];
	char ip[16];
	char *pt=reserve;
	int i=0;
	char tmp[3];
	
	
	while(*pt++!=';'); 
	
	while(*pt!=';' && *pt!='\0' && *pt!=0x01) ip[i++]=*pt++;
	ip[i]='\0';
	pt++;i=0;
	
	while(*pt!=';' && *pt!='\0'&& *pt!=0x01){
		if(*pt!=':')
	       		mac[i++]=*pt;
		pt++;
	}
	mac[i]='\0';

  	memset(&leases,0,sizeof(struct leases_t));

	pt=mac;
	for(i=0;i<6;i++){
		strncpy(tmp,pt,2);
		macaddr=strtol(tmp, (char **)NULL, 16);
		leases.chaddr[i]=macaddr;
		pt+=2;
	}	
	
	inet_aton(ip,&addr);	
	leases.yiaddr=addr.s_addr;
	leases.expires=htonl(31536000);
	
	inet_aton(nvram_safe_get("dhcp_start_ip"),&addr_s);	
	inet_aton(nvram_safe_get("dhcp_end_ip"),&addr_e);	
	
	if((addr_e.s_addr >= addr.s_addr) && (addr.s_addr >= addr_s.s_addr)){	
		search_leases(leases.chaddr,leases.yiaddr);
	}
#if 0
	if((fp=fopen(DHCPD_LEASES,"a+"))==NULL)
		return -1;
	fwrite(leases.chaddr, 16, 1, fp);
	fwrite(&(leases.yiaddr), 4, 1, fp);
	fwrite(&(leases.expires), 4, 1, fp);
	
	fclose(fp);
#endif
	return 0;	
}


int stop_dhcp_server(void)
{
	system("killall udhcpd");
	return 0;
}

int create_dhcp_server_cfg(void)
{
	FILE *fp;
	char tmp[4096]="";
	char *pt;
	char token=0x01;
	int i=0;
	if((fp=fopen(DHCPD_CONF,"w+"))==NULL) return -1;
	fprintf(fp,"server\t\t%s\n",nvram_safe_get("lan_ipaddr"));
	fprintf(fp,"start\t\t%s\n",nvram_safe_get("dhcp_start_ip"));
	fprintf(fp,"end\t\t%s\n",nvram_safe_get("dhcp_end_ip"));
	//fprintf(fp,"interface\t%s\n",nvram_safe_get("lan_if"));
	fprintf(fp,"interface\tbr0\n");
	fprintf(fp,"option\tsubnet\t%s\n",nvram_safe_get("lan_netmask"));
	fprintf(fp,"option\trouter\t%s\n",nvram_safe_get("lan_ipaddr"));
	fprintf(fp,"option\tdns\t%s\n",nvram_safe_get("lan_ipaddr"));
	fprintf(fp,"option\tlease\t%d\n",LEASES_TIME);
	
       	fclose(fp);

	pt=nvram_safe_get("dhcp_reserved");
		
	change_lease();
	while(*pt!='\0'){
		while(*pt!=token && *pt!='\0')
			tmp[i++]=*pt++;
		dhcp_write_leases(tmp);
		
		if(*pt!='\0') pt++;
		i=0;
	}
	return 0;

}
int start_dhcp_server(void)	
{
	char *pt;
		
	pt=nvram_safe_get("dhcp_server_enable");
	
	if(*pt=='1'){
		create_dhcp_server_cfg();
		system("/sbin/udhcpd&");
	}
	return 0;
}

