#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "nvram.h"
#include "err_msg.h"


#define CHECK_OK   	1 
#define CHECK_WARNING   0 
#define CHECK_ERROR    -1

/* compare value */
#define PPP_USERNAME_LEN 60
#define PPP_PASSWORD_LEN 50
#define VPI_START 0
#define VPI_END	255
#define VCI_START 0
#define VCI_END   65534
#define PPP_IDLE_MIN 0
#define PPP_IDLE_MAX   99
#define MTU_MIN 500	
#define MTU_MAX 1500
#define HTTP_PASSWORD 30
#define HTTP_TIMEOUT_MIN 1
#define HTTP_TIMEOUT_MAX 255
#define MAIL_SERVER_LEN  50
#define MAIL_ADDRESS_LEN 50
#define REMOTE_PORT_MIN 1
#define REMOTE_PORT_MAX 65534
#define WIFI_SSID_LEN 32 

/* compare value */

void get_err_msg(int index,char *msg,...);

/* test function */
#define RANGE_TEST(start,end,value)	((value <start || value >end)?CHECK_ERROR:CHECK_OK)
#define checkbox_test(value)		((value=='1' || value=='0')?CHECK_OK:CHECK_ERROR)
#define GET_VALUE(name,buf,pt)  if((pt=nvram_get(name))==NULL){ \
				get_err_msg(NullErr,buf,name);\
				return CHECK_ERROR;}
		
int range_test(int start,int end,int value,char *buf,char *name)
{
	if(value <start || value >end){
		get_err_msg(OutOfRange,buf,name,start,end);
		return CHECK_ERROR;
	}
	return CHECK_OK;
}	

int length_test(int maxlen,char *data,char *buf,char *name)
{
	if(strlen(data)>maxlen){
		get_err_msg(LenToLong,buf,name,maxlen);
		return CHECK_ERROR;
	}
	return CHECK_OK;
}

int ip_test(char *ip,char *buf,char *name)
{
	char ip1[4],ip2[4],ip3[4],ip4[4];
	
	if(ip==NULL) 
		return CHECK_ERROR;
	
	sscanf(ip,"%[^.].%[^.].%[^.].%s",ip1,ip2,ip3,ip4);
	if((RANGE_TEST(1,254,atoi(ip1))<0) 
		|| (RANGE_TEST(0,255,atoi(ip2))<0)
		|| (RANGE_TEST(0,255,atoi(ip3))<0)
		|| (RANGE_TEST(1,254,atoi(ip4))<0)){
	get_err_msg(OutOfRange2,buf,name,"1.0.0.1","254.255.255.254");
		return CHECK_ERROR;
	}
	return CHECK_OK;
}

int mask_test(char *ip,char *buf,char *name)
{
	char ip1[4],ip2[4],ip3[4],ip4[4];
	
	if(ip==NULL) 
		return CHECK_ERROR;
	
	sscanf(ip,"%[^.].%[^.].%[^.].%s",ip1,ip2,ip3,ip4);
	if((RANGE_TEST(0,255,atoi(ip1))<0) 
		|| (RANGE_TEST(0,255,atoi(ip2))<0)
		|| (RANGE_TEST(0,255,atoi(ip3))<0)
		|| (RANGE_TEST(0,255,atoi(ip4))<0)){
	get_err_msg(OutOfRange2,buf,name,"0.0.0.0","255.255.255.255");
		return CHECK_ERROR;
	}
	return CHECK_OK;
}
		
char check_keyword[1024];

/* test function */
char *get_keyword(char *name)
{
	FILE *fd;
	char tmp[1024];
	char buf[4096];
	char *pt;
	int i=0;
	
	if((fd=fopen(ERR_MSG_FILE,"r"))==NULL)
		return name;
	fread(buf,4096,1,fd);
	sprintf(tmp,"%s=",name);
	if((pt=strstr(buf,tmp))==NULL)
		return name;
	pt+=strlen(tmp); 
	while(*pt!='\n')   check_keyword[i++]=*(pt++);
	check_keyword[i] = 0x00;
	fclose(fd);
	return check_keyword;
}

/* get error message */
void get_err_msg(int index,char *msg,...)
{
	FILE *fd;
	char tmp[128];
	char buf[4096];
	char *pt;
	int i=0;
	va_list arg;
	
  	strcpy(msg,get_keyword("Unknown Error"));
	if((fd=fopen(ERR_MSG_FILE,"r"))==NULL)
		return;
	fread(buf,4096,1,fd);
	sprintf(tmp,"%d=",index);
	if((pt=strstr(buf,tmp))==NULL)
		return;
	pt=pt+strlen(tmp); 
	while(*pt!='\n')   msg[i++]=*(pt++);
	msg[i] = 0x00;
        va_start(arg,msg);
        vsnprintf(buf,4096,msg, arg);
        va_end(arg);
	strcpy(msg,buf);
	fclose(fd);
}
/* get error message */


/* basic check */
int basic_check(char *buf)
{
	char *pt;
	
	GET_VALUE("ppp_login",buf,pt);
	
	if(*pt=='1'){
	GET_VALUE("ppp_username",buf,pt);
	if(length_test(PPP_USERNAME_LEN,pt,buf,get_keyword("User Name"))==CHECK_ERROR)
		return CHECK_ERROR;
		
	GET_VALUE("ppp_password",buf,pt);
	if(length_test(PPP_PASSWORD_LEN,pt,buf,get_keyword("Password"))==CHECK_ERROR)
		return CHECK_ERROR;
	}
	GET_VALUE("ppp_idletime",buf,pt);
	
	if(range_test(PPP_IDLE_MIN,PPP_IDLE_MAX,atoi(pt),buf,get_keyword("Idle time"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	GET_VALUE("wan_fix_dns",buf,pt);
	
	if(*pt=='1'){
		GET_VALUE("wan_dns1",buf,pt);
		if(ip_test(pt,buf,get_keyword("Primary DNS IP Address"))==CHECK_ERROR)
			return CHECK_ERROR;
			
		GET_VALUE("wan_dns2",buf,pt);
		/*allow secondary dns is null*/
		if(strcmp(pt,"")!=0) {
		if(ip_test(pt,buf,get_keyword("Secondary DNS IP Address"))==CHECK_ERROR)
			return CHECK_ERROR;
		}
	}
	
	return CHECK_OK;
}

/* adsl check */
int adsl_check(char *buf)
{
	char *pt;
	/* vpi check */
	GET_VALUE("ppp_vpi",buf,pt);
	
	if(range_test(VPI_START,VPI_END,atoi(pt),buf,get_keyword("VPI"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	/* vci check */
	
	GET_VALUE("ppp_vci",buf,pt);
	
	if(range_test(VCI_START,VCI_END,atoi(pt),buf,get_keyword("VCI"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	return CHECK_OK;	
}

/* password check */
int password_check(char *buf)
{
	char *pt;
	GET_VALUE("http_password",buf,pt);
	if(length_test(HTTP_PASSWORD,pt,buf,get_keyword("Password"))==CHECK_ERROR)
		return CHECK_ERROR;

	GET_VALUE("http_timeout",buf,pt);
	if(range_test(HTTP_TIMEOUT_MIN,HTTP_TIMEOUT_MAX,atoi(pt),buf,get_keyword("Login time out"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	return CHECK_OK;
}

/* lan check  */
int lan_check(char *buf)
{
	char *pt;
	char ip1[4],ip2[4],ip3[4],ip4[4];
	char dip1[4],dip2[4],dip3[4],dip4[4];
	char tmp[16];
	int  warning=0;
	/* lan ip*/
	GET_VALUE("lan_ipaddr",buf,pt);
	
	if(ip_test(pt,buf,get_keyword("IP Address"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	sscanf(pt,"%[^.].%[^.].%[^.].%s",ip1,ip2,ip3,ip4);

	/* netmask */
	GET_VALUE("lan_netmask",buf,pt);
	
	if(mask_test(pt,buf,get_keyword("Network Mask"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	/* mtu */
	GET_VALUE("lan_mtu",buf,pt);
	
	if(range_test(MTU_MIN,MTU_MAX,atoi(pt),buf,get_keyword("MTU size"))==CHECK_ERROR)
		return CHECK_ERROR;
	

	/* dhcp start ip*/
	GET_VALUE("dhcp_start_ip",buf,pt);
	
	if(ip_test(pt,buf,get_keyword("DHCP Start IP Address"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	/* check ip range*/
	sscanf(pt,"%[^.].%[^.].%[^.].%s",dip1,dip2,dip3,dip4);
	
	if((strcmp(ip1,dip1)!=0) || (strcmp(ip2,dip2)!=0)
		       	|| (strcmp(ip3,dip3)!=0)){
		sprintf(tmp,"%s.%s.%s.%s",ip1,ip2,ip3,dip4);
		nvram_set("dhcp_start_ip",tmp);
		warning=1;
	}
	
	/* dhcp end ip */
	GET_VALUE("dhcp_end_ip",buf,pt);
	
	if(ip_test(pt,buf,get_keyword("DHCP End IP Address"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	/* check ip range*/
	sscanf(pt,"%[^.].%[^.].%[^.].%s",dip1,dip2,dip3,dip4);
	
	if((strcmp(ip1,dip1)!=0) || (strcmp(ip2,dip2)!=0)
		       	|| (strcmp(ip3,dip3)!=0)){
		sprintf(tmp,"%s.%s.%s.%s",ip1,ip2,ip3,dip4);
		nvram_set("dhcp_end_ip",tmp);
		warning=1;
	}
	
	/* trusted ip */
	GET_VALUE("fw_block_trust",buf,pt);
	if(ip_test(pt,buf,get_keyword("Trusted IP Address"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	/* check ip range*/
	sscanf(pt,"%[^.].%[^.].%[^.].%s",dip1,dip2,dip3,dip4);
	
	if((strcmp(ip1,dip1)!=0) || (strcmp(ip2,dip2)!=0)
		       	|| (strcmp(ip3,dip3)!=0)){
		get_err_msg(LanRange,buf,get_keyword("Trusted IP Address"));
		return CHECK_ERROR;
	}
	/* dmz ip */
	GET_VALUE("fw_dmz",buf,pt);
	if(strcmp(pt,"")!=0){
		if(ip_test(pt,buf,get_keyword("DMZ IP Address"))==CHECK_ERROR)
			return CHECK_ERROR;
	
		/* check ip range*/
		sscanf(pt,"%[^.].%[^.].%[^.].%s",dip1,dip2,dip3,dip4);
	
		if((strcmp(ip1,dip1)!=0) || (strcmp(ip2,dip2)!=0)
			       	|| (strcmp(ip3,dip3)!=0)){
			get_err_msg(LanRange,buf,get_keyword("DMZ IP Address"));
			return CHECK_ERROR;
		}
	}
	
	if(warning){
		get_err_msg(DHCPWarn,buf);
		return CHECK_WARNING;
	}
	else
		return CHECK_OK;
}
int schedule_check(char *buf)
{
	char *pt;		
	GET_VALUE("ntp_server",buf,pt);
	if(strcmp(pt,"")==0)
		return CHECK_OK;
	
	if(ip_test(pt,buf,get_keyword("NTP server IP Address"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	return CHECK_OK;
}
int email_check(char *buf)
{
	char *pt;
	GET_VALUE("mail_enable",buf,pt);
	if(*pt=='0') 
		return CHECK_OK;
	
	GET_VALUE("mail_server",buf,pt);
	if(length_test(MAIL_SERVER_LEN,pt,buf,get_keyword("Mail server's name"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	GET_VALUE("mail_recipient",buf,pt);
	if(length_test(MAIL_ADDRESS_LEN,pt,buf,get_keyword("Email Address"))==CHECK_ERROR)
		return CHECK_ERROR;
	return CHECK_OK;
}
int remotemg_check(char *buf)
{
	char *pt;
	GET_VALUE("fw_remote_port",buf,pt);
	if(range_test(REMOTE_PORT_MIN,REMOTE_PORT_MAX,atoi(pt),buf,get_keyword("Remote port number"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	return CHECK_OK;
}

int wireless_check(char *buf)
{
	char *pt;
	GET_VALUE("wifi_ssid",buf,pt);
	if(length_test(WIFI_SSID_LEN,pt,buf,get_keyword("SSID"))==CHECK_ERROR)
		return CHECK_ERROR;
	
	return CHECK_OK;
	
}

int main(void){
	char buf[4096];
	if(basic_check(buf)!=CHECK_OK)
		printf("%s\n",buf);
	return 0;
}
